Package: attachment
Title: Deal with Dependencies
Version: 1.1.0
Authors@R: c(
    person("Vincent", "Guyader", , "vincent@thinkr.fr", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0003-0671-9270")),
    person("Sébastien", "Rochette", , "sebastienrochettefr@gmail.com", role = c("aut"),
           comment = c(ORCID = "0000-0002-1565-9313", "previous maintainer")),
    person("Murielle", "Delmotte", , "murielle@thinkr.fr", role = "aut",
           comment = c(ORCID = "0000-0002-1339-2424")),
    person("Swann", "Floc'hlay", , "swann@thinkr.fr", role = "aut",
           comment = c(ORCID = "0000-0003-1477-830X")),
    person("ThinkR", role = c("cph", "fnd"))
  )
Description: Manage dependencies during package development. This can
    retrieve all dependencies that are used in ".R" files in the "R/"
    directory, in ".Rmd" files in "vignettes/" directory and in 'roxygen2'
    documentation of functions. There is a function to update the
    "DESCRIPTION" file of your package with 'CRAN' packages or any other
    remote package.  All functions to retrieve dependencies of ".R"
    scripts and ".Rmd" or ".qmd" files can be used independently of a
    package development.
License: GPL-3
URL: https://thinkr-open.github.io/attachment/,
        https://github.com/ThinkR-open/attachment
BugReports: https://github.com/ThinkR-open/attachment/issues
VignetteBuilder: knitr
Config/fusen/version: 0.6.0
Config/Needs/website: ThinkR-open/thinkrtemplate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
Language: en-US
Depends: R (>= 3.4)
Imports: cli, desc (>= 1.2.0), glue (>= 1.3.0), knitr (>= 1.20),
        magrittr (>= 1.5), rmarkdown (>= 1.10), roxygen2, stats,
        stringr (>= 1.3.1), utils, withr, yaml
Suggests: lifecycle, renv (>= 0.8.4), rstudioapi, testthat (>= 3.0.0)
NeedsCompilation: no
Packaged: 2026-08-19 14:45:00 UTC; ubuntu
Author: Vincent Guyader [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-0671-9270>),
  Sébastien Rochette [aut] (ORCID:
    <https://orcid.org/0000-0002-1565-9313>, previous maintainer),
  Murielle Delmotte [aut] (ORCID:
    <https://orcid.org/0000-0002-1339-2424>),
  Swann Floc'hlay [aut] (ORCID: <https://orcid.org/0000-0003-1477-830X>),
  ThinkR [cph, fnd]
Maintainer: Vincent Guyader <vincent@thinkr.fr>
Repository: CRAN
Date/Publication: 2026-08-20 05:23:51 UTC
Built: R 4.5.2; ; 2026-08-20 09:07:13 UTC; unix

Package: dummypackage
Title: What the Package Does (one line, title case)
Version: 0.0.0.9000
Authors@R: 
    person("First", "Last", , "first.last@example.com", role = c("aut", "cre"))
Description: What the package does (one paragraph).
License: What license is it under?
Depends: 
    R (>= 3.5.0)
Imports: 
    magrittr,
    stats
Suggests: 
    glue,
    knitr,
    rmarkdown,
    testthat,
    utils,
    stringr
LinkingTo:
    Rcpp
VignetteBuilder: 
    knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.1

# Generated by roxygen2: do not edit by hand

export("%>%")
export(att_amend_desc)
export(att_from_data)
export(att_from_description)
export(att_from_examples)
export(att_from_namespace)
export(att_from_qmd)
export(att_from_qmds)
export(att_from_rmd)
export(att_from_rmds)
export(att_from_rscript)
export(att_from_rscripts)
export(att_to_desc_from_is)
export(att_to_desc_from_pkg)
export(att_to_description)
export(complete_remote_branch)
export(create_dependencies_file)
export(create_renv_for_dev)
export(create_renv_for_prod)
export(find_remotes)
export(install_from_description)
export(install_if_missing)
export(set_remotes_to_desc)
importFrom(cli,cat_bullet)
importFrom(desc,description)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(knitr,purl)
importFrom(magrittr,"%>%")
importFrom(roxygen2,roxygenise)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_match)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_trim)
importFrom(utils,install.packages)
importFrom(utils,packageDescription)
importFrom(utils,read.table)
importFrom(yaml,read_yaml)
importFrom(yaml,write_yaml)

<!-- README.md is generated from README.Rmd. Please edit that file -->
<!-- badges: start -->

[![Coverage
status](https://codecov.io/gh/ThinkR-open/attachment/branch/main/graph/badge.svg)](https://app.codecov.io/github/ThinkR-open/attachment/tree/main)
[![CRAN
status](https://www.r-pkg.org/badges/version/attachment)](https://cran.r-project.org/package=attachment)
![downloads](http://cranlogs.r-pkg.org/badges/attachment)
[![R-CMD-check](https://github.com/ThinkR-open/attachment/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ThinkR-open/attachment/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/ThinkR-open/attachment/graph/badge.svg)](https://app.codecov.io/gh/ThinkR-open/attachment)
<!-- badges: end -->

# attachment <img src="man/figures/logo.png" align="right" alt="" width="120" />

The goal of attachment is to help to deal with package dependencies
during package development. It also gives useful tools to install or
list missing packages used inside Rscripts or Rmds.

When building a package, we have to add `@importFrom` in our
documentation or `pkg::fun` in the R code. The most important is not to
forget to add the list of dependencies in the “Imports” or “Suggests”
package lists in the DESCRIPTION file.

Why do you have to repeat twice the same thing ?  
And what happens when you remove a dependency for one of your functions
? Do you really want to run a “Find in files” to verify that you do not
need this package anymore ?

Let {attachment} help you ! This reads your NAMESPACE, your functions in
R directory and your vignettes, then update the DESCRIPTION file
accordingly. Are you ready to be lazy ?

See full documentation realized using {pkgdown} at
<https://thinkr-open.github.io/attachment/>

## Installation

CRAN version

``` r
install.packages("attachment")
```

Development version

``` r
install.packages('attachment', repos = c('https://thinkr-open.r-universe.dev', 'https://cloud.r-project.org'))
```

## Declare all dependencies in DESCRIPTION during package development

What you really want is to fill and update your description file along
with the modifications of your documentation. Indeed, only the following
function will really be called. Use and abuse during the development of
your package !

``` r
attachment::att_amend_desc()
```

{attachment} detects all calls to `library(pkg)`, `@importFrom pkg fun`,
`pkg::fun()` in the different classical directories of your R package,
then list them in the correct “Imports” or “Suggests” category in the
DESCRIPTION file, according to their position in the package.

### Declare extra dependencies for extra uses

If you want to add extra packages like {pkgdown} or {covr} that are not
listed in any script in your package, a call for your development
packages would be:

``` r
attachment::att_amend_desc(extra.suggests = c("pkgdown", "covr"), update.config = TRUE)
```

Note the `update.config = TRUE` parameter that will save the parameters
used in the call of `att_amend_desc()` to the package configuration
file: “dev/config_attachment.yaml”.

If you run `att_amend_desc()` a second time afterwards, directly from
the console, it will use the last set of parameters extracted from the
configuration file.

Indeed, we recommend to store the complete command line in a
“dev/dev_history.R” file to update and run it when needed. If the
parameters do not change, you can run `attachment::att_amend_desc()`
directly in the console, wherever you are, it will use the configuration
file.

### Automatically fill the “Remotes” field

If you would like to detect the sources of your installations so that
you can add dependencies in the “Remotes” field of your DESCRIPTION
file, to mimic your local installation, you will use:

``` r
attachment::set_remotes_to_desc()
```

## Example on a fake package

``` r
# Copy example package in a temporary directory
tmpdir <- tempfile(pattern = "fakepkg")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
#> [1] TRUE
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)

# Fill the DESCRIPTION file automatically
# `inside_rmd` is specifically designed here to allow to run this command line in the "Readme.Rmd" file
desc_file <- attachment::att_amend_desc(path = dummypackage, inside_rmd = TRUE, update.config = TRUE)
#> 'update.config' was set to TRUE, hence, 'use.config' was forced to FALSE
#> Saving attachment parameters to yaml config file
#> Updating dummypackage documentation
#> Setting `RoxygenNote` to "7.3.2"
#> Writing 'NAMESPACE'
#> ℹ Loading dummypackage
#> Writing 'NAMESPACE'
#> ℹ Loading dummypackage
#> Package(s) Rcpp is(are) in category 'LinkingTo'. Check your Description file to be sure it is really what you want.

# Add Remotes if you have some installed
attachment::set_remotes_to_desc(path.d = desc_file)
#> There are no remote packages installed on your computer to add to description
#> NULL

# Clean state
unlink(tmpdir, recursive = TRUE)
```

## More on finding Remotes repositories (non installed from CRAN)

Find packages installed out of CRAN. This helps fill the “Remotes” field
in DESCRIPTION file with `set_remotes_to_desc()`.  
Behind the scene, it uses `find_remotes()`.

- See the examples below if {fusen} is installed from GitHub
  - Also works for GitLab, Bioconductor, Git, Local installations

``` r
# From GitHub
remotes::install_github("ThinkR-open/fusen",
                        quiet = TRUE, upgrade = "never")
attachment::find_remotes("fusen")
#> $fusen
#> [1] "ThinkR-open/fusen"
```

## Find and install missing dependencies required for your R scripts

To quickly install missing packages from a DESCRIPTION file, use:

``` r
attachment::install_from_description()
#> All required packages are installed
```

To quickly install missing packages needed to compile Rmd files or run R
scripts, use:

``` r
attachment::att_from_rmds(path = ".") %>% attachment::install_if_missing()

attachment::att_from_rscripts(path = ".") %>% attachment::install_if_missing()
```

Function `attachment::create_dependencies_file()` will create a
`dependencies.R` file in `inst/` directory. This R script contains the
procedure to quickly install missing dependencies:

``` r
# Remotes ----
# remotes::install_github("ThinkR-open/fcuk")
# Attachments ----
to_install <- c("covr", "desc", "devtools", "glue", "knitr", "magrittr", "rmarkdown", "stats", "stringr", "testthat", "utils")
for (i in to_install) {
  message(paste("looking for ", i))
  if (!requireNamespace(i)) {
    message(paste("     installing", i))
    install.packages(i)
  }
}
```

## Allow the CI to install all dependencies required for your bookdown, pagedown, quarto, …

If you write a {bookdown} and want to publish it on Github using GitHub
Actions or GitLab CI for instance, you will need a DESCRIPTION file with
list of dependencies just like for a package. In this case, you can use
the function to description from import/suggest:
`att_to_desc_from_is()`.

``` r
usethis::use_description()
# bookdown Imports are in Rmds
imports <- c("bookdown", attachment::att_from_rmds("."))
attachment::att_to_desc_from_is(path.d = "DESCRIPTION",
                                imports = imports, suggests = NULL)
```

Then, install dependencies with

``` r
remotes::install_deps()
```

## List packages required in any script or notebook

Of course, you can also use {attachment} out of a package to list all
package dependencies of R scripts using `att_from_rscripts()` or Rmd/qmd
files using `att_from_rmds()`.  
If you are running this inside a Rmd, you may need parameter
`inside_rmd = TRUE`.

``` r
library(attachment)
dummypackage <- system.file("dummypackage", package = "attachment")

att_from_rscripts(path = dummypackage)
#> [1] "stats"        "testthat"     "dummypackage"
att_from_rmds(path = file.path(dummypackage, "vignettes"), inside_rmd = TRUE)
#> [1] "knitr"     "rmarkdown" "glue"
```

## Vignettes included

Package {attachment} has vignettes to present the different functions
available. There is also a recommendation to have a `dev_history.R` in
the root directory of your package. (*Have a look at
[dev_history.R](https://github.com/ThinkR-open/attachment/blob/main/dev/dev_history.R)
in the present package*)

``` r
vignette("a-fill-pkg-description", package = "attachment")
vignette("b-bookdown-and-scripts", package = "attachment")
vignette("use_renv", package = "attachment")
vignette("create-dependencies-file", package = "attachment")
```

The vignettes are available on the {pkgdown} page, in the “Articles”
menu: <https://thinkr-open.github.io/attachment/>

## Code of Conduct

Please note that the attachment project is released with a [Contributor
Code of
Conduct](https://thinkr-open.github.io/attachment/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms

# No Remotes ----
# Attachments ----
to_install <- c("desc", "glue", "knitr", "magrittr", "rmarkdown", "roxygen2", "stringr", "testthat")
  for (i in to_install) {
    message(paste("looking for ", i))
    if (!requireNamespace(i)) {
      message(paste("     installing", i))
      install.packages(i)
    }
  }

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(attachment)

## ----examples-create_dependencies_file----------------------------------------
# Create a fake package
tmpdir <- tempfile(pattern = "depsfile")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
          recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

# Create the dependencies commands but no file
create_dependencies_file(
  path = file.path(dummypackage,"DESCRIPTION"),
  to = NULL,
  open_file = FALSE)

# Create the dependencies files in the package
create_dependencies_file(
  path = file.path(dummypackage,"DESCRIPTION"),
  to = file.path(dummypackage, "inst/dependencies.R"),
  open_file = FALSE)
list.files(file.path(dummypackage, "inst"))
# browseURL(dummypackage)

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)

#
# This is the server logic of a Shiny web application. You can run the
# application by clicking 'Run App' above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#

# library(shiny)

# Define server logic required to draw a histogram
function(input, output, session) {

    output$distPlot <- renderPlot({

        # generate bins based on input$bins from ui.R
        x    <- faithful[, 2]
        bins <- seq(min(x), max(x), length.out = input$bins + 1)

        # draw the histogram with the specified number of bins
        hist(x, breaks = bins, col = 'darkgray', border = 'white',
             xlab = 'Waiting time to next eruption (in mins)',
             main = 'Histogram of waiting times')

    })

}

## ----setuprmd, include = FALSE------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(attachment)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc()

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(pkg_ignore = c("fakepackage.to_ignore", "other.package"), update.config = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(extra.suggests = c("suggested.package.not.detected"), update.config = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(pkg_ignore = c("package.to.move"), extra.suggests = c("package.to.move"), update.config = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(extra.suggests = c("bookdown"), update.config = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# #' Knit my internal template
# #'
# #' @importFrom bookdown html_document2
# #' @export
# #'
# my_knit <- function() {
#   rmarkdown::render(system.file("my_template.Rmd", package = "my.package"))
# }

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(dir.r = c("R", "inst"))

## -----------------------------------------------------------------------------
# Copy package in a temporary directory
tmpdir <- tempfile(pattern = "insidermd")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)
att_amend_desc(path = dummypackage, inside_rmd = TRUE, update.config = TRUE)

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(dummypackage) %>%
#   set_remotes_to_desc()

## ----eval=TRUE----------------------------------------------------------------
find_remotes(pkg = c("attachment", "desc", "glue"))

## ----eval=FALSE---------------------------------------------------------------
# att_from_description() %>%
#   find_remotes()

## ----eval=FALSE---------------------------------------------------------------
# find_remotes(list.dirs(.libPaths(), full.names = FALSE, recursive = FALSE))

## ----eval=FALSE---------------------------------------------------------------
# # From GitHub
# remotes::install_github("ThinkR-open/fusen",
#                         quiet = TRUE, upgrade = "never")
# attachment::find_remotes("fusen")
# #> $fusen
# #> [1] "ThinkR-open/fusen"
# 
# # From r-universe as default repos
# install.packages("fusen", repos = "https://thinkr-open.r-universe.dev")
# attachment::find_remotes("fusen")
# #> r-universe: need to set options to repos="https://thinkr-open.r-universe.dev"

## ----eval=FALSE---------------------------------------------------------------
# create_dependencies_file()

## ----eval=FALSE---------------------------------------------------------------
# # No Remotes ----
# # remotes::install_github("ThinkR-open/fcuk")
# # Attachments ----
# to_install <- c("covr", "desc", "devtools", "glue", "knitr", "magrittr", "rmarkdown", "stats", "stringr", "testthat", "utils")
# for (i in to_install) {
#   message(paste("looking for ", i))
#   if (!requireNamespace(i)) {
#     message(paste("     installing", i))
#     install.packages(i)
#   }
# }

## ----eval=TRUE----------------------------------------------------------------
dummypackage <- system.file("dummypackage", package = "attachment")

att_from_rscripts(path = file.path(dummypackage, "R"))
att_from_rmds(path = file.path(dummypackage, "vignettes"), inside_rmd = TRUE)
att_from_examples(dir.r = file.path(dummypackage, "R"))

## ----setuprmd, include = FALSE------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------
library(attachment)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc()

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(pkg_ignore = c("fakepackage.to_ignore", "other.package"), update.config = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(extra.suggests = c("suggested.package.not.detected"), update.config = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(pkg_ignore = c("package.to.move"), extra.suggests = c("package.to.move"), update.config = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(extra.suggests = c("bookdown"), update.config = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# #' Knit my internal template
# #'
# #' @importFrom bookdown html_document2
# #' @export
# #'
# my_knit <- function() {
#   rmarkdown::render(system.file("my_template.Rmd", package = "my.package"))
# }

## -----------------------------------------------------------------------------
# Copy package in a temporary directory
tmpdir <- tempfile(pattern = "insidermd")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)
att_amend_desc(path = dummypackage, inside_rmd = TRUE, update.config = TRUE)

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# att_amend_desc(dummypackage) %>%
#   set_remotes_to_desc()

## ----eval=TRUE----------------------------------------------------------------
find_remotes(pkg = c("attachment", "desc", "glue"))

## ----eval=FALSE---------------------------------------------------------------
# att_from_description() %>%
#   find_remotes()

## ----eval=FALSE---------------------------------------------------------------
# find_remotes(list.dirs(.libPaths(), full.names = FALSE, recursive = FALSE))

## ----eval=FALSE---------------------------------------------------------------
# # From GitHub
# remotes::install_github("ThinkR-open/fusen",
#                         quiet = TRUE, upgrade = "never")
# attachment::find_remotes("fusen")
# #> $fusen
# #> [1] "ThinkR-open/fusen"
# 
# # From r-universe as default repos
# install.packages("fusen", repos = "https://thinkr-open.r-universe.dev")
# attachment::find_remotes("fusen")
# #> r-universe: need to set options to repos="https://thinkr-open.r-universe.dev"

## ----eval=FALSE---------------------------------------------------------------
# create_dependencies_file()

## ----eval=FALSE---------------------------------------------------------------
# # No Remotes ----
# # remotes::install_github("ThinkR-open/fcuk")
# # Attachments ----
# to_install <- c("covr", "desc", "devtools", "glue", "knitr", "magrittr", "rmarkdown", "stats", "stringr", "testthat", "utils")
# for (i in to_install) {
#   message(paste("looking for ", i))
#   if (!requireNamespace(i)) {
#     message(paste("     installing", i))
#     install.packages(i)
#   }
# }

## ----eval=TRUE----------------------------------------------------------------
dummypackage <- system.file("dummypackage", package = "attachment")

att_from_rscripts(path = file.path(dummypackage, "R"))
att_from_rmds(path = file.path(dummypackage, "vignettes"), inside_rmd = TRUE)
att_from_examples(dir.r = file.path(dummypackage, "R"))

#' my_mean
#'
#' @param x a vector
#'
#' @export
#' @importFrom magrittr %>%
#' @examples
#' # example code
#' library(utils)
#' data("fruit", package = "stringr")
my_mean <- function(x){
  x <- x %>% stats::na.omit()
  1+1
  sum(x)/base::length(x)
}

#' Amend DESCRIPTION with dependencies read from package code parsing
#'
#' Amend package DESCRIPTION file with the list of dependencies extracted from
#' R, examples, tests, vignettes files.
#' att_to_desc_from_pkg() is an alias of att_amend_desc(),
#' for the correspondence with [att_to_desc_from_is()].
#'
#' @param path path to the root of the package directory. Default to current directory.
#' @param path.n path to namespace file.
#' @param dir.r path to directory with R scripts.
#' @param dir.v path to vignettes directory. Set to empty (dir.v = "") to ignore.
#' @param dir.t path to tests directory. Set to empty (dir.t = "") to ignore.
#' @param extra.suggests vector of other packages that should be added in Suggests (pkgdown, covr for instance)
#' @param pkg_ignore vector of packages names to ignore.
#' @param update.config logical. Should the parameters used in this call be saved in the config file of the package
#' @param use.config logical. Should the command use the parameters from the config file to run
#' @param path.c character Path to the yaml config file where parameters are saved
#'
#' @inheritParams att_from_namespace
#' @inheritParams att_to_desc_from_is
#' @inheritParams att_from_rmds
#' @inheritParams att_from_examples
#'
#' @importFrom desc description
#'
#' @return Update DESCRIPTION file.
#'
#' @details
#'
#' Your daily use is to run `att_amend_desc()`, as is.
#' You will want to run this function sometimes with some extra information like
#' `att_amend_desc(pkg_ignore = "x", update.config = TRUE)` if you have to update
#' the configuration file.
#' Next time `att_amend_desc()` will use these parameters from the configuration
#' file directly.
#'
#'
#' @export
#' @examples
#'
#' # Run on an external "dummypackage" as an example
#' # For your local use, you do not have to specify the `path` as below
#' # By default, `att_amend_desc()` will run on the current working directory
#'
#' # Create a fake package for the example
#' tmpdir <- tempfile(pattern = "description")
#' dir.create(tmpdir)
#' file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
#'  recursive = TRUE)
#' dummypackage <- file.path(tmpdir, "dummypackage")
#'
#' # Update documentation and dependencies
#' att_amend_desc(path = dummypackage)
#'
#' # You can look at the content of this external package
#' #' # browseURL(dummypackage)
#'
#' # Update the config file with extra parameters
#' # We recommend that you store this code in a file in your "dev/" directory
#' # to run it when needed
#' att_amend_desc(path = dummypackage, extra.suggests = "testthat", update.config = TRUE)
#'
#' # Next time, in your daily development
#' att_amend_desc(path = dummypackage)
#'
#' # Clean after examples
#' unlink(tmpdir, recursive = TRUE)

att_amend_desc <- function(path = ".",
                           path.n = "NAMESPACE",
                           path.d = "DESCRIPTION",
                           dir.r = "R",
                           dir.v = "vignettes",
                           dir.t = "tests",
                           extra.suggests = NULL,
                           pkg_ignore = NULL,
                           document = TRUE,
                           normalize = TRUE,
                           inside_rmd = NULL,
                           must.exist = TRUE,
                           check_if_suggests_is_installed = TRUE,
                           update.config = FALSE,
                           use.config = TRUE,
                           path.c = "dev/config_attachment.yaml"
) {

  if (path != ".") {
    old <- setwd(normalizePath(path))
    on.exit(setwd(old))
  }



  path <- normalizePath(path)

  # decide whether to use or update config file ----
  if (isTRUE(update.config) & isTRUE(use.config)) {
    use.config <- FALSE
    message("'update.config' was set to TRUE, hence, 'use.config' was forced to FALSE")
  }

  # extract all current parameter values - ignore config parameters - save also default
  att_params <- names(formals(att_amend_desc))
  att_params <- att_params[!att_params %in% c("path", "update.config", "use.config", "path.c")]
  local_att_params <- mget(att_params)

  params_to_load <- compare_inputs_load_or_save(
    path.c = path.c,
    local_att_params = local_att_params,
    use.config = use.config,
    update.config = update.config)

  if (!is.null(params_to_load)) {
    for (param_name in names(params_to_load)){
      assign(param_name, params_to_load[[param_name]])
    }
    message(c("Documentation parameters were restored from attachment config file."))
  }

  # Save all open files ----
  save_all()

  # Update description ----
  if (!file.exists(path.d)) {
    x3 <- description$new("!new")
    x3$set("Package", basename(path))
    x3$write("DESCRIPTION")
    message("An new path.d =", path.d, " was added to the directory. ",
            "Please fill it. ",
            "\nNext time, you may want to use 'usethis::use_description()'")
  }
  if (path.d == "DESCRIPTION") {path.d <- file.path(normalizePath(path), path.d)}

  # Remove non-existing directories in path.n for Imports
  if (!file.exists(path.n)) {
    if (isTRUE(document)) {
      roxygen2::roxygenise(path, roclets = NULL)
      path.n <- file.path(path, "NAMESPACE")
      message("A new path.n =", path.n, " was added to the directory. ")
    } else {
      message("There is no directory named: ",
              path.n,
              ". This is removed from the Imports exploration")
      path.n <- ""
    }
  }

  # Remove non-existing directories in dir.r for Imports
  dir.r.test <- dir.exists(dir.r)
  if (any(dir.r.test)) {
    if (any(!dir.r.test)) {
      message("There is no directory named: ",
              paste(dir.r[!dir.r.test], collapse = ", "),
              ". This is removed from the Imports exploration")
    }
    dir.r <- dir.r[dir.r.test]
  } else {
    dir.r <- ""
  }

  # Remove non-existing directories for Suggests
  dir.v.test <- dir.exists(dir.v)
  if (any(dir.v.test)) {
    if (any(!dir.v.test)) {
      message("There is no directory named: ",
              paste(dir.v[!dir.v.test], collapse = ", "),
              ". This is removed from the Suggests exploration")
    }
    dir.v <- dir.v[dir.v.test]
  } else {
    dir.v <- ""
  }

  # Remove non-existing directories for Suggests
  dir.t.test <- dir.exists(dir.t)
  if (any(dir.t.test)) {
    if (any(!dir.t.test)) {
      message("There is no directory named: ",
              paste(dir.t[!dir.t.test], collapse = ", "),
              ". This is removed from the Suggests exploration")
    }
    dir.t <- dir.t[dir.t.test]
  } else {
    dir.t <- ""
  }

  # Imports ----
  # Find dependencies in namespace and scripts
  imports <- NULL
  if (path.n != "") {
    imports <- unique(c(imports, att_from_namespace(path.n, document = document)))
  }
  if (dir.r != "") {
    # Look for R scripts
    imports <- unique(c(imports, att_from_rscripts(dir.r)))
    # Look for Rmd, in case in a bookdown
    # imports <- unique(c(imports, att_from_rmds(dir.r, inside_rmd = inside_rmd)))
  }

  # Suggests ----
  suggests <- NULL

  # Get suggests in examples and remove if already in imports
  if (dir.r != "") {
    ex <- att_from_examples(dir.r = dir.r)
    suggests <- c(suggests, ex[!ex %in% imports])
  }

  # Get suggests in vignettes and remove if already in imports
  if (!grepl("^$|^\\s+$$", dir.v)) {
    vg <- att_from_rmds(dir.v, inside_rmd = inside_rmd)
    suggests <- c(suggests, vg[!vg %in% imports])
  }

  # Get suggests in tests and remove if already in imports
  if (!grepl("^$|^\\s+$$", dir.t)) {
    tt <- att_from_rscripts(dir.t)
    suggests <- c(suggests, tt[!tt %in% imports])
  }

  # Ignore packages ----
  if (!is.null(pkg_ignore)) {
    imports <- imports[!imports %in% pkg_ignore]
    suggests <- suggests[!suggests %in% pkg_ignore]
    # suggests_keep <- suggests_keep[!suggests_keep %in% pkg_ignore]
  }
  if (!is.null(extra.suggests)) {
    suggests <- unique(c(suggests, extra.suggests))
  }

  # Ignore `base` from imports and suggests
  imports <- imports[imports != "base"]
  suggests <- suggests[suggests != "base"]

  # Build DESCRIPTION ----
  att_to_desc_from_is(path.d, imports, suggests, normalize, must.exist, check_if_suggests_is_installed = check_if_suggests_is_installed)
}

#' @rdname att_amend_desc
#' @export
att_to_desc_from_pkg <- att_amend_desc

#' Amend DESCRIPTION with dependencies from imports and suggests package list
#'
#' @param path.d path to description file.
#' @param imports character vector of package names to add in Imports section
#' @param suggests character vector of package names to add in Suggests section
#' @param check_if_suggests_is_installed Logical. Whether to require that packages in the Suggests section are installed.
#' @param normalize Logical. Whether to normalize the DESCRIPTION file. See [desc::desc_normalize()]
#' @param must.exist Logical. If TRUE then an error is given if packages do not exist
#' within installed packages. If NA, a warning.
#'
#' @importFrom desc description
#' @importFrom glue glue glue_collapse
#'
#' @export
#'
#' @return Fill in Description file
#'
#' @details
#' `must.exist` is better set to `TRUE` during package development.
#' This stops the process when a package does not exists on your system.
#' This avoids check errors with typos in package names in DESCRIPTION.
#' When used in CI to discover dependencies, for a bookdown for instance,
#' you may want to set to `FALSE` (no message at all) or `NA` (warning for not installed).
#'
#' @examples
#' tmpdir <- tempfile(pattern = "descfromis")
#' dir.create(tmpdir)
#' file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
#'  recursive = TRUE)
#' dummypackage <- file.path(tmpdir, "dummypackage")
#' # browseURL(dummypackage)
#' att_to_desc_from_is(path.d = file.path(dummypackage, "DESCRIPTION"),
#' imports = c("magrittr", "attachment"), suggests = c("knitr"))
#'
#' # In combination with other functions
#' att_to_desc_from_is(path.d = file.path(dummypackage, "DESCRIPTION"),
#' imports = att_from_rscripts(file.path(dummypackage, "R")),
#' suggests = att_from_rmds(file.path(dummypackage, "vignettes")))
#'
#' # Clean temp files after this example
#' unlink(tmpdir, recursive = TRUE)

att_to_desc_from_is <- function(path.d = "DESCRIPTION", imports = NULL,
                                suggests = NULL, check_if_suggests_is_installed = TRUE, normalize = TRUE,
                                must.exist = TRUE) {

  if (!file.exists(path.d)) {
    x3 <- description$new("!new")
    if (path.d == "DESCRIPTION") {path.d <- file.path(normalizePath("."), path.d)}
    x3$set("Package", basename(dirname(path.d)))
    x3$write(path.d)
    message("An new path.d =", path.d, " was added to the directory. ",
            "Please fill it. ",
            "\nNext time, you may want to use 'usethis::use_description()'")
  }

  desc <- description$new(path.d)
  pkg_name <- desc$get("Package")
  # Remove pkg name from imports
  imports <- imports[imports != pkg_name]
  # Remove pkg name from suggests
  suggests <- suggests[suggests != pkg_name]
  # check that packages are installed

  # rlang::check_installed("pkg")
  # imports
  check_installed <- c(imports)
  if (check_if_suggests_is_installed){
    check_installed <- c(check_installed, suggests)
  }
  suppressWarnings(
    res <- vapply(
      check_installed, FUN = requireNamespace,
      FUN.VALUE = logical(1), quietly = TRUE)
  )
  missing_packages <- names(res[!res])

  if (length(missing_packages) > 0) {
    if (length(missing_packages) == 1) {
      msg <-
        glue::glue(
          "The package {missing_packages} is missing or misspelled.
             Please correct your typo or install it."
        )
    } else {
      msg <-
        glue::glue(
          "Packages {pkgs} are missing or misspelled.
             Please correct your typos or do the proper installations.",
          pkgs = glue::glue_collapse(missing_packages, sep = ", ", last = " & ")
        )
    }
    if (isTRUE(must.exist)) {
      stop(msg)
    } else if (is.na(must.exist)) {
      warning(msg)
    }
  }

  # Get previous dependencies in Description in case version is set
  deps_desc <- desc$get_deps()
  deps_orig <- deps_desc[deps_desc$type != "Depends",]
  deps_depends_orig <- deps_desc[deps_desc$type == "Depends",]
  deps_linkingto_orig <- deps_desc[deps_desc$type == "LinkingTo",]

  remotes_orig <- desc$get_remotes()
  if (length(remotes_orig) != 0) {
    remotes_orig_pkg <- gsub("^.*/|[.]git|@.*$", "", remotes_orig)
  } else {
    remotes_orig_pkg <- NULL
  }

  # If remotes: remove orig remotes not anymore in imports or suggests
  if (!is.null(remotes_orig_pkg)) {
    remotes_keep_pkg <- remotes_orig_pkg[remotes_orig_pkg %in% c(imports, suggests)] #suggests_keep
    remotes_keep <- remotes_orig[remotes_orig_pkg %in% remotes_keep_pkg]
  } else {
    remotes_keep_pkg <- remotes_keep <- NULL
  }

  # Create new deps dataframe
  all_packages <- c(imports, suggests)
  if (is.null(all_packages)) {all_packages <- character()}

  deps_new <- data.frame(
    type = c(rep("Imports", length(imports)), rep("Suggests", length(suggests))),
    package = all_packages, stringsAsFactors = FALSE) %>%
    merge(deps_orig[,c("package", "version")],
          by = "package", sort = TRUE, all.x = TRUE, all.y = FALSE) %>%
    .[,c("type", "package", "version")] %>%
    .[order(.$type, .$package), , drop = FALSE] %>%
    .[!duplicated(.$package),]

  # Test if package had Depends category
  if (nrow(deps_depends_orig) != 0) {
    # _Keep depends to specific R version (added below to be first)
    R_depends <- deps_depends_orig[deps_depends_orig$package == "R",]

    # _Test if other Depends still in dependencies
    Other_depends <- deps_depends_orig[deps_depends_orig$package != "R",] %>%
      .[order(.$package),]
    if (nrow(Other_depends) != 0) {
      Other_depends_keep <- Other_depends[Other_depends$package %in% deps_new$package, ]
      if (length(Other_depends_keep) != 0) {
        message("Package(s) ",
                paste(Other_depends_keep$package, collapse = ", "),
                " is(are) in category 'Depends'. Check your Description file",
                " to be sure it is really what you want."
        )
        # If in Depends, not in Imports
        deps_new <- rbind(Other_depends_keep,
                          deps_new[-which(deps_new$package %in% Other_depends_keep$package),])
      }
    }

    if (nrow(R_depends) != 0) {
      deps_new <- rbind(R_depends, deps_new)
    }
  }

  if (nrow(deps_linkingto_orig) != 0) {
    deps_linkingto <- deps_linkingto_orig
    message("Package(s) ",
            deps_linkingto$package,
            " is(are) in category 'LinkingTo'. Check your Description file",
            " to be sure it is really what you want."
    )
    deps_new <- rbind(deps_new, deps_linkingto)
  }

  deps_new$version[is.na(deps_new$version)] <- "*"

  # Compare old and new
  removed <- deps_desc$package[!deps_desc$package %in% deps_new$package]
  if (length(removed) > 0) {
    message("[-] ", length(removed), " package(s) removed: ",
            paste(removed, collapse = ", "), ".")
  }
  added <- deps_new$package[!deps_new$package %in% deps_desc$package]
  if (length(added) > 0) {
    message("[+] ", length(added), " package(s) added: ",
            paste(added, collapse = ", "), ".")
  }

  # Remove previous deps
  desc$del_deps()
  # Set new deps
  desc$set_deps(deps_new)

  # remotes <- desc$get_remotes()
  if (length(remotes_keep) != 0) {
    desc$set_remotes(sort(remotes_keep))
  }
  # Reorder sections
  if (isTRUE(normalize)) {
    desc$normalize()
  }
  # Write Description file
  desc$write(file = path.d)

  return(invisible(path.d))
}

# test on dummy package
tmpdir <- tempfile(pattern = "pkg")
dir.create(tmpdir)
file.copy(
  system.file("dummypackage", package = "attachment"), tmpdir,
  recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
deps <- att_from_namespace(path = file.path(dummypackage, "NAMESPACE"))

test_that("att_from_namespace works", {
  expect_true(setequal(deps, c("magrittr")))
})

unlink(tmpdir, recursive = TRUE)

# Do not work when NAMESPACE does not exists
tmpdir <- tempfile(pattern = "pkgnamespace")
dir.create(tmpdir)
file.copy(
  system.file("dummypackage", package = "attachment"), tmpdir,
  recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
file.remove(file.path(dummypackage, "NAMESPACE"))

test_that("att_from_namespace works", {
  expect_error(
    att_from_namespace(path = file.path(dummypackage, "NAMESPACE")),
    regexp = "does not exists"
  )

  expect_true(setequal(deps, c("magrittr")))
})

unlink(tmpdir, recursive = TRUE)

# Works when used twice
tmpdir <- tempfile(pattern = "pkgtwice")
dir.create(tmpdir)
file.copy(
  system.file("dummypackage", package = "attachment"), tmpdir,
  recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

test_that("att_from_namespace works", {
  deps <- expect_error(
    att_from_namespace(path = file.path(dummypackage, "NAMESPACE")),
    regexp = NA
  )
  expect_true(setequal(deps, c("magrittr")))

  deps <- expect_error(
    att_from_namespace(path = file.path(dummypackage, "NAMESPACE")),
    regexp = NA
  )
  expect_true(setequal(deps, c("magrittr")))

  deps <- expect_error(
    att_from_namespace(path = file.path(dummypackage, "NAMESPACE")),
    regexp = NA
  )
  expect_true(setequal(deps, c("magrittr")))
})

unlink(tmpdir, recursive = TRUE)

# File that failed in the past ----
deps <- att_from_namespace("fake_namespace", document = FALSE)

test_that("att_from_namespace works", {
  expect_true(setequal(deps, c("Rcpp", "magrittr")))
})

# File that failed in the past with bad NAMESPACE ----
# test on dummy package
tmpdir <- tempfile(pattern = "pkgbadnamespace")
dir.create(tmpdir)
file.copy(
  system.file("dummypackage", package = "attachment"), tmpdir,
  recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# Replace R script to introduce error in NAMESPACE
r_file <- file.path(dummypackage, "R", "my_mean.R")
lines_orig <- lines <- readLines(r_file)
lines[6] <- "#' @importFrom magrittr\n"
writeLines(enc2utf8(lines), r_file)

test_that("bad namespace can be corrected", {
  # expect_warning(cli::cli_warn("toto"), regexp = "tota")

  # Check that the information transmitted by roxygen2 is correctly retransmitted by attachment

  if (packageVersion("roxygen2") >= "7.3.0") {
    # roxygen > 7.3.0 generates a message

    expect_message(att_amend_desc(dummypackage),
                   "must have at least 2 words, not 1")

    # Still message because of r_file but no error
    expect_message(att_from_namespace(path = file.path(dummypackage, "NAMESPACE"),
                                      document = TRUE, clean = TRUE))
  } else {
    # roxygen > 7.1.2 generates a warning
    expect_warning(att_amend_desc(dummypackage),
                   "must have at least 2 words, not 1")
    # Hence no error
    expect_error(
      suppressWarnings(
        att_from_namespace(
          path = file.path(dummypackage, "NAMESPACE"),
          document = TRUE, clean = FALSE)
      ),
      regexp = NA)

    # Still warning because of r_file but no error
    expect_warning(att_from_namespace(path = file.path(dummypackage, "NAMESPACE"),
                                      document = TRUE, clean = TRUE))
  }


  # Correct R function, no warning, no error
  writeLines(enc2utf8(lines_orig), r_file)
  deps <- att_from_namespace(path = file.path(dummypackage, "NAMESPACE"),
                                    document = TRUE, clean = TRUE)
  expect_true(setequal(deps, c("magrittr")))
})

unlink(dummypackage, recursive = TRUE)
unlink(tmpdir, recursive = TRUE)

# att_amend_desc ----
# Copy package in a temporary directory
tmpdir <- tempfile("dummyamend")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)

user_warn <- getOption("warn")
att_amend_desc(path = dummypackage)

test_that("att_amend_desc add dev/ in .Rbuildignore if needed", {
  rbuildignore_file <- readLines(file.path(tmpdir, "dummypackage", ".Rbuildignore"))
  expect_true(any(grepl("\\^dev\\$",rbuildignore_file)))
})


test_that("att_amend_desc does not change warn level", {
  new_warn <- getOption("warn")
  expect_equal(new_warn, user_warn)
})


desc_file <- readLines(file.path(tmpdir, "dummypackage", "DESCRIPTION"))
namespace_file <- readLines(file.path(tmpdir, "dummypackage", "NAMESPACE"))

test_that("att_amend_desc updates namespace", {
  expect_length(namespace_file, 4)
})

test_that("att_amend_desc updates description", {
  # person() can be 1 or 4 lines depending on {desc} version
  w.depends <- grep("Depends:", desc_file)
  expect_length(w.depends, 1)
  expect_equal(desc_file[w.depends + 1], "    R (>= 3.5.0)")
  expect_equal(desc_file[w.depends + 2], "Imports: ")
  expect_equal(desc_file[w.depends + 3], "    magrittr,")
  expect_equal(desc_file[w.depends + 4], "    stats")
  expect_equal(desc_file[w.depends + 5], "Suggests: ")
  expect_equal(desc_file[w.depends + 6], "    glue,")
  expect_equal(desc_file[w.depends + 7], "    knitr,")
  expect_equal(desc_file[w.depends + 8], "    rmarkdown,")
  expect_equal(desc_file[w.depends + 9], "    stringr,")
  expect_equal(desc_file[w.depends + 10], "    testthat,")
  expect_equal(desc_file[w.depends + 11], "    utils")
  expect_equal(desc_file[w.depends + 12], "LinkingTo:" )
  expect_equal(desc_file[w.depends + 13], "    Rcpp")
  # base does not appear
  expect_false(all(grepl("base", desc_file)))
  # utils is removed
  expect_false(all(grepl("utils", desc_file)))
})
unlink(dummypackage, recursive = TRUE)

# Remotes stays here if exists ----
tmpdir <- tempfile("dummyremotes")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
path.d <- file.path(dummypackage, "DESCRIPTION")
cat(
  "Remotes:
    thinkr-open/fusen,
    tidyverse/magrittr,
    rstudio/rmarkdown@branch,
    git::https://github.com/yihui/knitr@main
", append = TRUE,
  file = path.d)

test_that("Remotes stays here if exists and package in imports/suggests", {
  suppressMessages(
    expect_error(att_amend_desc(path = dummypackage), regexp = NA))
  desc_file <- readLines(file.path(dummypackage, "DESCRIPTION"))

  expect_false(any(grepl("thinkr-open/fusen", desc_file))) # not in deps
  w.remotes <- grep('Remotes:', desc_file)
  expect_length(w.remotes, 1)
  expect_equal(desc_file[w.remotes + 1], "    git::https://github.com/yihui/knitr@main,") #imports
  expect_equal(desc_file[w.remotes + 2], "    rstudio/rmarkdown@branch,") #suggest
  expect_equal(desc_file[w.remotes + 3], "    tidyverse/magrittr") #imports
})
unlink(dummypackage, recursive = TRUE)

# pkg_ignore and extra.suggests are used ----
tmpdir <- tempfile("dummyextra")
dir.create(tmpdir)
file.copy(system.file("dummypackage", package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

test_that("pkg_ignore and extra.suggests are used", {
  suppressMessages(
    expect_error(
      att_amend_desc(path = dummypackage,
                     pkg_ignore = "glue",
                     extra.suggests = "roxygen2"),
      regexp = NA)
  )
  desc_file <- readLines(file.path(dummypackage, "DESCRIPTION"))

  expect_false(any(grepl("glue", desc_file))) # ignored totally
  w.suggests <- grep('Suggests:', desc_file)
  expect_length(w.suggests, 1)
  expect_equal(desc_file[w.suggests + 3], "    roxygen2,") #suggest
})
unlink(dummypackage, recursive = TRUE)

# Test fails if dir.t do not exists
tmpdir <- tempfile("dummydirt")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

test_that("fails if dir.t do not exists", {
  expect_message(
    att_amend_desc(path = dummypackage,
                   dir.r = c("R", "rara")),
    regexp = "There is no directory named: rara")

  expect_error(
    att_amend_desc(path = dummypackage,
                   dir.r = "rara", # do not exist, nothing fails
                   use.config = FALSE),
    regexp = NA)

  expect_message(
    att_amend_desc(path = dummypackage,
                   dir.v = c("vignettes", "vava"),
                   use.config = FALSE),
    regexp = "There is no directory named: vava")

  expect_error(
    att_amend_desc(path = dummypackage,
                   dir.v = "vava",
                   update.config = TRUE), # do not exist, nothing fails
    regexp = NA)

  expect_message(
    att_amend_desc(path = dummypackage,
                   dir.t = c("tests", "tata"),
                   update.config = TRUE),
    regexp = "There is no directory named: tata")

  expect_error(
    att_amend_desc(path = dummypackage,
                   dir.t = "tata",
                   update.config = TRUE), # do not exist, nothing fails
    regexp = NA)
})
unlink(dummypackage, recursive = TRUE)

# Test Deprecated att_to_description ----
# suppressWarnings()
# Copy package in a temporary directory
tmpdir <- tempfile(pattern = "deprecated")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)
suppressWarnings(att_to_description(path = dummypackage))
desc_file <- readLines(file.path(tmpdir, "dummypackage", "DESCRIPTION"))
namespace_file <- readLines(file.path(tmpdir, "dummypackage", "NAMESPACE"))

test_that("att_to_description still updates namespace", {
  expect_length(namespace_file, 4)
})
unlink(tmpdir, recursive = TRUE)


# att_to_desc_from_is ----
# Copy package in a temporary directory
tmpdir <- tempfile(pattern = "pkg")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)
att_to_desc_from_is(path.d = file.path(dummypackage, "DESCRIPTION"),
                    imports = c("magrittr", "attachment"), suggests = c("knitr"))

desc_file <- readLines(file.path(tmpdir, "dummypackage", "DESCRIPTION"))

test_that("att_to_desc_from_is updates description", {
  # person() can be 1 or 4 lines depending on {desc} version
  w.depends <- grep("Depends:", desc_file)
  expect_length(w.depends, 1)
  expect_equal(desc_file[w.depends], "Depends: ")
  expect_equal(desc_file[w.depends + 1], "    R (>= 3.5.0)")
  expect_equal(desc_file[w.depends + 2], "Imports: ")
  expect_equal(desc_file[w.depends + 3], "    attachment,")
  expect_equal(desc_file[w.depends + 4], "    magrittr")
  expect_equal(desc_file[w.depends + 5], "Suggests: ")
  expect_equal(desc_file[w.depends + 6], "    knitr")
  expect_equal(desc_file[w.depends + 7], "LinkingTo:" )
  expect_equal(desc_file[w.depends + 8], "    Rcpp")
})
unlink(tmpdir, recursive = TRUE)



# Test missing DESCRIPTION works ----
# Copy package in a temporary directory
tmpdir <- tempfile("dummydesc")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)
file.remove(file.path(dummypackage, "DESCRIPTION"))
test_that("Works with missing DESCRIPTION", {
  expect_false(file.exists(file.path(dummypackage, "DESCRIPTION")))
  expect_message(att_amend_desc(path = dummypackage), "use_description")
  expect_true(file.exists(file.path(dummypackage, "DESCRIPTION")))
  desc_file <- readLines(file.path(tmpdir, "dummypackage", "DESCRIPTION"))
  expect_true(grepl("dummypackage", desc_file[1]))
  expect_false(any(grepl("dummypackage", desc_file[-1])))

  file.remove(file.path(dummypackage, "DESCRIPTION"))
  expect_false(file.exists(file.path(dummypackage, "DESCRIPTION")))
  expect_message(att_to_desc_from_is(path = file.path(dummypackage, "DESCRIPTION"),
                                     imports = c("magrittr")), "use_description")
  expect_true(file.exists(file.path(dummypackage, "DESCRIPTION")))
  desc_file <- readLines(file.path(tmpdir, "dummypackage", "DESCRIPTION"))
  expect_true(grepl("dummypackage", desc_file[1]))
  expect_false(any(grepl("dummypackage", desc_file[-1])))
})

unlink(tmpdir, recursive = TRUE)

# Test missing NAMESPACE works ----
# Copy package in a temporary directory
tmpdir <- tempfile("dummyname")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)
file.remove(file.path(dummypackage, "NAMESPACE"))

test_that("Works with missing DESCRIPTION", {
  expect_false(file.exists(file.path(dummypackage, "NAMESPACE")))
  expect_error(att_from_namespace(file.path(dummypackage, "NAMESPACE")), "attachment::att_amend_desc()")

  expect_message(att_amend_desc(path = dummypackage, document = FALSE),
                 "no directory named: NAMESPACE")
  expect_false(file.exists(file.path(dummypackage, "NAMESPACE")))

  expect_message(att_amend_desc(path = dummypackage, document = TRUE, update.config = TRUE), "new path.n")
  expect_true(file.exists(file.path(dummypackage, "NAMESPACE")))

})

unlink(tmpdir, recursive = TRUE)
# Note: glue is necessary for {attachment}, ggplot2 is not even in suggests.

# missing pkg in R is not installed ----
test_that("missing pkg is not installed", {
  # Copy package in a temporary directory
  tmpdir <- tempfile("dummymissing")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")
  # browseURL(dummypackage)

  cat("
#' Function
#' @importFrom ggplot ggplot
#' @export
my_fun <- function() {
data %>%
filter(col == 3) %>%
mutate(new_col = 1) %>%
ggplot() +
  aes(x, y, colour = new_col) +
  geom_point()
}
", file = file.path(dummypackage, "R", "function.R"))

  expect_error(attachment::att_amend_desc(path = dummypackage))#,
  # "The package ggplot is missing or misspelled.")

  # Clean after
  unlink(dummypackage, recursive = TRUE)
})

# missing pkgS are not installed ----
test_that("missing pkgS are not installed", {

  # Copy package in a temporary directory
  tmpdir <- tempfile("dummypkgs")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")

  cat("
#' Function
#' @importFrom ggplot ggplot
#' @importFrom ggplot3 ggplot
# @importFrom ggplot4 ggplot # Only comment, not used
#' @export
my_fun <- function() {
data %>%
filter(col == 3) %>%
mutate(new_col = 1) %>%
ggplot() +
  aes(x, y, colour = new_col) +
  geom_point()
}
", file = file.path(dummypackage, "R", "function.R"))

  expect_error(attachment::att_amend_desc(path = dummypackage))#,
  # "Packages ggplot & ggplot3 are missing or misspelled.")

  # Clean after
  unlink(dummypackage, recursive = TRUE)
})

# missing pkg in vignette (Suggests) is not installed ----
test_that("missing pkg is not installed", {
  # Copy package in a temporary directory
  tmpdir <- tempfile("dummysuggestinstal")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")
  # browseURL(dummypackage)

  cat("
## The vignette
```{r}
library(glue)
library(ggplot3)
```
", file = file.path(dummypackage, "vignettes", "vignette.Rmd"))

  expect_error(attachment::att_amend_desc(path = dummypackage,check_if_suggests_is_installed = TRUE),
               "The package ggplot3 is missing or misspelled.")
  # Clean after
  unlink(dummypackage, recursive = TRUE)
})

# Test update desc when missing packages for books ----
tmpdir <- tempfile("miss")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"),
          tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

test_that("att_to_desc_from_is can update DESCRIPTION w/o uninstalled packages", {
  imports <- unique(
    c("fakeinstalled", # Not exists
      "bookdown", "knitr",
      "pagedown", # Not installed on CI usually
      att_from_rmds(file.path(dummypackage, "vignettes"), recursive = FALSE))
  )
  # Default to error
  expect_error(att_to_desc_from_is(
    path.d = file.path(dummypackage, "DESCRIPTION"),
    imports = imports
  ), regexp = "missing or misspelled")

  # must.exist = FALSE
  expect_warning(
    att_to_desc_from_is(
      path.d = file.path(dummypackage, "DESCRIPTION"),
      imports = imports,
      must.exist = NA # then warning
    ),
    regexp = "missing or misspelled")

  expect_warning(
    expect_error(
      att_to_desc_from_is(
        path.d = file.path(dummypackage, "DESCRIPTION"),
        imports = imports,
        must.exist = FALSE # then no error, no warning
      ), regexp = NA),
    regexp = NA)

  desc <- readLines(file.path(file.path(dummypackage, "DESCRIPTION")))
  expect_true(any(grepl("fakeinstalled", desc)))
  expect_true(any(grepl("bookdown", desc)))
  expect_true(any(grepl("knitr", desc)))
  expect_true(any(grepl("pagedown", desc)))
})
unlink(tmpdir, recursive = TRUE)

# Test manual
if (interactive()) {
  skip_if_not(requireNamespace("rstudioapi"))
  skip_if_not(rstudioapi::isAvailable())

  # This requires a manual test in another session
  # dummypackage <- tempfile('dummypackage')
  # suppressMessages(
  #   usethis::create_package(
  #     path = dummypackage,
  #     open = FALSE,
  #   )
  # )

  answer <- rstudioapi::showQuestion(
    "Run interactive test?",
    paste("Do you want to open a new RStudio session for the test?\n",
          "If yes, you will have to run: source('dev/test_to_run.R')"),
    cancel = "no", ok = "yes")

  if (isTRUE(answer)) {

  tmpdir <- tempfile("manual")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"),
            tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")


  # Add a dummy function using an external dependency
  # namely the {magrittr} pipe %>%
  r_file <- file.path(dummypackage, "R", "fun_manual.R")
  file.create(r_file)
  #> [1] TRUE
  writeLines(
    text = "#' @importFrom magrittr %>%
#' @export
my_length <- function(x) {
  x %>% length()
}",
    con = r_file
  )

  dir.create(file.path(dummypackage, "dev"))
  writeLines(
    text = paste0(
      "pkgload::load_all('", getwd(), "')\n",
      "att_amend_desc()
# This should work without error
my_length(1)


"),
    con = file.path(dummypackage, "dev", "test_to_run.R")
  )

    rstudioapi::openProject(path = dummypackage, newSession = TRUE)

    answer2 <- rstudioapi::showQuestion(
      "Did the code in 'dev/test_to_run.R' worked correctly ?",
      paste("Did the code in 'dev/test_to_run.R' worked correctly ?",
      "\n\n(You can close the RStudio session opened, the project will be deleted)"),
      cancel = "no", ok = "yes")

    test_that("if check_if_suggests_is_installed = FALSE no test is done on Suggests packages", {
      expect_true(answer2)
    })

    unlink(tmpdir, recursive = TRUE)

  }


}


# missing pkg in vignette (Suggests) is not installed but we dont check it ----
test_that("if check_if_suggests_is_installed = FALSE no test is done on Suggests packages", {
  # Copy package in a temporary directory
  tmpdir <- tempfile("dummysuggestcheck")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")
  # browseURL(dummypackage)
  cat("
## The vignette
```{r}
library(glue)
library(ggplot3)
```
", file = file.path(dummypackage, "vignettes", "vignette.Rmd"))
  attachment::att_amend_desc(path = dummypackage,check_if_suggests_is_installed = FALSE)


  desc_file <- readLines(file.path(dummypackage, "DESCRIPTION"))

  w.depends <- grep("Depends:", desc_file)
  expect_length(w.depends, 1)
  expect_equal(desc_file[w.depends + 1], "    R (>= 3.5.0)")
  expect_equal(desc_file[w.depends + 2], "Imports: ")
  expect_equal(desc_file[w.depends + 3], "    magrittr,")
  expect_equal(desc_file[w.depends + 4], "    stats")
  expect_equal(desc_file[w.depends + 5], "Suggests: ")
  expect_equal(desc_file[w.depends + 6], "    ggplot3,")
  expect_equal(desc_file[w.depends + 7], "    glue,")
  expect_equal(desc_file[w.depends + 8], "    knitr,")
  expect_equal(desc_file[w.depends + 9], "    rmarkdown,")
  expect_equal(desc_file[w.depends + 10], "    stringr,")
  expect_equal(desc_file[w.depends + 11], "    testthat,")
  expect_equal(desc_file[w.depends + 12], "    utils")
  expect_equal(desc_file[w.depends + 13], "LinkingTo:" )
  expect_equal(desc_file[w.depends + 14], "    Rcpp")


  # Clean after
  unlink(dummypackage, recursive = TRUE)
})


# Test update desc when packages in examples ----
test_that("if a package is used in an example, it is added to Suggests packages", {
  # Copy package in a temporary directory
  tmpdir <- tempfile("dummysuggestexamples")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")

  r_file <- file.path(dummypackage, "R", "fun_manual.R")
  file.create(r_file)
  #> [1] TRUE
  writeLines(
    text = "#' @importFrom magrittr %>%
#' @examples
#' library(pkgfake)
#' fakepkg::fun()
#' @export
my_length <- function(x) {
  x %>% length()
}",
    con = r_file
  )

  attachment::att_amend_desc(path = dummypackage, check_if_suggests_is_installed = FALSE)


  desc_file <- readLines(file.path(dummypackage, "DESCRIPTION"))

  w.depends <- grep("Depends:", desc_file)
  expect_length(w.depends, 1)
  expect_equal(desc_file[w.depends + 5], "Suggests: ")
  expect_equal(desc_file[w.depends + 6], "    fakepkg,")
  expect_equal(desc_file[w.depends + 7], "    glue,")
  expect_equal(desc_file[w.depends + 8], "    knitr,")
  expect_equal(desc_file[w.depends + 9], "    pkgfake,")
  expect_equal(desc_file[w.depends + 10], "    rmarkdown,")
  expect_equal(desc_file[w.depends + 11], "    stringr,")
  expect_equal(desc_file[w.depends + 12], "    testthat,")
  expect_equal(desc_file[w.depends + 13], "    utils")
  # Clean after
  unlink(dummypackage, recursive = TRUE)

})


test_that("if a package is used in an example has already been added to IMPORTS, it is not added to the suggests, it is added to Suggests packages", {
  # Copy package in a temporary directory
  tmpdir <- tempfile("dummysuggestexamples")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")

  r_file <- file.path(dummypackage, "R", "fun_manual.R")
  file.create(r_file)
  #> [1] TRUE
  writeLines(
    text = "#' @importFrom magrittr %>%
#' @examples
#' library(magrittr)
#' fakepkg::fun()
#' @export
my_length <- function(x) {
  x %>% length()
}",
    con = r_file
  )

  attachment::att_amend_desc(path = dummypackage, check_if_suggests_is_installed = FALSE)


  desc_file <- readLines(file.path(dummypackage, "DESCRIPTION"))

  w.depends <- grep("Depends:", desc_file)
  expect_length(w.depends, 1)
  expect_equal(desc_file[w.depends + 1], "    R (>= 3.5.0)")
  expect_equal(desc_file[w.depends + 2], "Imports: ")
  expect_equal(desc_file[w.depends + 3], "    magrittr,")
  expect_equal(desc_file[w.depends + 4], "    stats")
  expect_equal(desc_file[w.depends + 5], "Suggests: ")
  expect_equal(desc_file[w.depends + 6], "    fakepkg,")
  expect_equal(desc_file[w.depends + 7], "    glue,")
  expect_equal(desc_file[w.depends + 8], "    knitr,")
  expect_equal(desc_file[w.depends + 9], "    rmarkdown,")
  expect_equal(desc_file[w.depends + 10], "    stringr,")
  expect_equal(desc_file[w.depends + 11], "    testthat,")
  expect_equal(desc_file[w.depends + 12], "    utils")
  expect_equal(desc_file[w.depends + 13], "LinkingTo:" )
  expect_equal(desc_file[w.depends + 14], "    Rcpp")

  # Clean after
  unlink(dummypackage, recursive = TRUE)

})

# These tests can not run on CRAN
skip_on_cran()

if (length(find.package("extrapackage", quiet = TRUE)) != 0) {
  unlink(find.package("extrapackage", quiet = TRUE), recursive = TRUE)
}

# test on dummy package
tmpdir <- tempfile(pattern = "pkgrenv2")
dir.create(tmpdir)
file.copy(
  system.file("dummyfolder", package = "attachment"), tmpdir,
  recursive = TRUE
)
dummyfolder <- file.path(tmpdir, "dummyfolder")







# Create a second dummy package that will be a dependance of dummy
# So that, we are sure it does not exists on CRAN for extra checks
extra_path <- file.path(tmpdir, "dummy.extra")
dir.create(extra_path, recursive = TRUE)
file.copy(
  system.file("dummypackage", package = "attachment"),
  extra_path,
  recursive = TRUE
)
extrapackage <- file.path(extra_path, "extrapackage")
file.rename(file.path(extra_path, "dummypackage"), extrapackage)
# Rename package and remove 'Rcpp'
desc_lines <- readLines(file.path(extrapackage, "DESCRIPTION"))
desc_lines <- gsub("dummypackage", "extrapackage", desc_lines)
desc_lines <- desc_lines[-grep("LinkingTo|Rcpp", desc_lines)]
cat(desc_lines, sep = "\n", file = file.path(extrapackage, "DESCRIPTION"))
# Remove calls to 'dummypackage' and 'Rcpp'
unlink(file.path(extrapackage, "tests"), recursive = TRUE)
# document
# inuse <- search()
att_amend_desc(path = extrapackage)
unloadNamespace("extrapackage") # for windows mainly
# Install package to make it available to {renv}
install.packages(extrapackage, repos = NULL, type = "source")












lock_ <- file.path(tmpdir, "my_lock.lock")


  expect_message({

    my_renv_ <- create_renv_for_dev(
      path = dummyfolder,
      install_if_missing = FALSE,
      output = lock_,
      # force generation of a lockfile even when pre-flight validation checks have failed?
      force = TRUE)}
  )





test_that("create_renv_for_dev creates lock files even without DESCRIPTION file", {
  expect_true(file.exists(lock_))
})

# print(my_renv_extra)


renv_content <- getFromNamespace("lockfile", "renv")(my_renv_)


test_that("lockfile are correct renv files", {
expect_s3_class(renv_content, "renv_lockfile_api")
expect_true("glue" %in% names(renv_content$data()$Packages))
expect_true("extrapackage" %in% names(renv_content$data()$Packages))
})

  # Copy package in a temporary directory
  tmpdir <- tempfile("dummyadd_build_ignore")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")
  # browseURL(dummypackage)
  # rstudioapi::filesPaneNavigate(dummypackage)
  cat("truc\nbidul\ngg\n",file = file.path(tmpdir, "dummypackage", ".Rbuildignore"))
  user_warn <- getOption("warn")
  att_amend_desc(path = dummypackage)

    rbuildignore_file <- readLines(file.path(tmpdir, "dummypackage", ".Rbuildignore"))
  test_that("att_amend_desc add dev/ in .Rbuildignore if needed", {
    expect_true(any(grepl("\\^dev\\$",rbuildignore_file)))
  })
  test_that("att_amend_desc dont remove existing ligne", {
    expect_true(any(grepl("truc",rbuildignore_file)))
    expect_true(any(grepl("bidul",rbuildignore_file)))
    expect_true(any(grepl("gg",rbuildignore_file)))
  })
  att_amend_desc(path = dummypackage)
  rbuildignore_file <- readLines(file.path(tmpdir, "dummypackage", ".Rbuildignore"))

  test_that("att_amend_desc dont add dev/ in .Rbuildignore if already here", {
    expect_equal(c("truc", "bidul", "gg", "^dev$"),rbuildignore_file)
  })

#
# This is the user-interface definition of a Shiny web application. You can
# run the application by clicking 'Run App' above.
#
# Find out more about building applications with Shiny here:
#
#    http://shiny.rstudio.com/
#

# library(shiny)

# Define UI for application that draws a histogram
fluidPage(

    # Application title
    titlePanel("Old Faithful Geyser Data"),

    # Sidebar with a slider input for number of bins
    sidebarLayout(
        sidebarPanel(
            sliderInput("bins",
                        "Number of bins:",
                        min = 1,
                        max = 50,
                        value = 30)
        ),

        # Show a plot of the generated distribution
        mainPanel(
            plotOutput("distPlot")
        )
    )
)

#' Add files to Rbuildignore
#'
#' @param x vector of filenames
#' @param directory path where to Rbuildignore should be written
#' @noRd
add_build_ignore <- function(x, directory = ".") {
  build_file <- file.path(directory, ".Rbuildignore")

  if (!file.exists(build_file)) {
    writeLines("", build_file)
  }

  build_ignore <- readLines(build_file)

  to_ignore <- paste0("^", gsub("[.]", "\\\\.", x), "$")

  to_ignore <- to_ignore[!to_ignore %in% build_ignore]

  if (length(to_ignore) != 0) {
    writeLines(enc2utf8(c(build_ignore, to_ignore)), build_file)
  }
}

#' Get all packages called in examples from R files
#'
#' @param dir.r path to directory with R scripts.
#'
#' @return Character vector of packages called with library or require.
#' @importFrom roxygen2 parse_file block_get_tag_value
#' @examples
#' dummypackage <- system.file("dummypackage",package = "attachment")
#'
#' # browseURL(dummypackage)
#' att_from_examples(dir.r = file.path(dummypackage,"R"))

#' @export
att_from_examples <- function(dir.r = "R") {
  rfiles <- list.files(dir.r, full.names = TRUE, pattern = "\\.r$", ignore.case = TRUE,recursive = FALSE)

  roxy_file <- tempfile("roxy.examples", fileext = ".R")

  all_examples <- unlist(lapply(rfiles, function(the_file) {
    file_roxytags <- roxygen2::parse_file(the_file)
    res <- unlist(
      lapply(file_roxytags,
             function(x) roxygen2::block_get_tag_value(block = x, tag = "examples"))
    )
    res
  }))
  # Clean \dontrun and \donttest, and replace with '{' on next line
  all_examples_clean <-
    gsub(pattern = "\\\\dontrun\\s*\\{|\\\\donttest\\s*\\{", replacement = "#ICI\n{", x = all_examples)

# Clean escape characters
  all_examples_clean <- gsub(pattern = "\\\\", replacement = "", x = all_examples_clean)

  cat(all_examples_clean, file = roxy_file, sep = "\n")

  all_deps_examples_data <- attachment::att_from_data(all_examples_clean)

  all_deps_examples <- attachment::att_from_rscript(roxy_file)

  file.remove(roxy_file)

  all_deps <- unique(c(all_deps_examples, all_deps_examples_data))

  return(all_deps)
}

test_that("att_from_examples works", {
  tmpdir <- tempfile("suggestexamples")

  dir.create(file.path(tmpdir, "R"), recursive = TRUE)

  r_file <- file.path(tmpdir, "R", "fun_manual.R")
  file.create(r_file)

  writeLines(
    text = "#' @importFrom magrittr %>%
#' @examples
#' library(magrittr)
#' fakepkg::fun()
#' @export
my_length <- function(x) {
  x %>% length()
}",
    con = r_file
  )

  expect_equal(att_from_examples(dir.r = file.path(tmpdir, "R")),
               c("magrittr", "fakepkg"))

})


test_that("att_from_examples works with escape characters", {
  tmpdir <- tempfile("suggestexamples")

  dir.create(file.path(tmpdir, "R"), recursive = TRUE)

  r_file <- file.path(tmpdir, "R", "fun_manual.R")
  file.create(r_file)

  writeLines(
    text = "#' @importFrom magrittr %>%
#' @examples
#' library(magrittr)
#' fakepkg::fun()
#' 1 %not_in% 1:10
#' not_null(NULL)
`%not_in%` <- Negate(`%in%`)
",
    con = r_file
  )

  expect_equal(att_from_examples(dir.r = file.path(tmpdir, "R")),
               c("magrittr", "fakepkg"))

})


test_that("att_from_examples works even with sysdata.rda", {
  tmpdir <- tempfile("suggestexamples")

  dir.create(file.path(tmpdir, "R"), recursive = TRUE)

  r_file <- file.path(tmpdir, "R", "fun_manual.R")
  file.create(r_file)

  writeLines(
    text = "#' @importFrom magrittr %>%
#' @examples
#' library(magrittr)
#' fakepkg::fun()
#' @export
my_length <- function(x) {
  x %>% length()
}",
    con = r_file
  )

  save(iris,file = file.path(tmpdir,"R","sysdata.rda"),compress = "bzip2",version = 3,ascii = FALSE)

  expect_equal(att_from_examples(dir.r = file.path(tmpdir, "R")),
               c("magrittr", "fakepkg"))

})



test_that("att_from_examples works with dontrun", {
  tmpdir <- tempfile("suggestexamples")

  dir.create(file.path(tmpdir, "R"), recursive = TRUE)

  file.copy(test_path("fake_function.R"),file.path(tmpdir, "R"))

  expect_equal(att_from_examples(dir.r = file.path(tmpdir, "R")),
               c("magrittr", "fakepkg"))

})

#' Look for functions called in data loading code
#'
#' @param chr A character vector containing the code as a string. The code should follow the pattern used for loading data with `data()`, specifying the dataset and package.
#' @return A character vector containing the names of the packages from which datasets are being loaded.
#' @importFrom stringr str_extract_all
#' @export
#' @examples
#'
#' vec_char <- 'data("starwars", package = "dplyr")'
#' att_from_data(vec_char)
att_from_data <- function(chr) {
  pkg <-
    str_extract_all(
      chr,
      '(?<=data\\(\\".{1,100}\\"\\,\\s{0,5}package\\s{0,5}\\=\\s{0,5}\\\")[[:alnum:]\\.]+(?=\\\"\\))'
    ) %>%
    unlist()
  return(pkg)

}

test_that("att_from_data works with a single package", {

  input1 <- 'data("starwars", package = "dplyr")'
  expect_equal(att_from_data(input1), "dplyr")

})

test_that("att_from_data works with multi package", {

  input1 <- 'data("starwars", package = "dplyr")data("dataset", package = "pkgfake")'
  expect_equal(att_from_data(input1), c("dplyr", "pkgfake"))

})


test_that("att_from_data works with extra spaces", {

  input1 <- 'data("starwars",package    = "dplyr")'
  expect_equal(att_from_data(input1), "dplyr")

})

# Fixture for issues #120, #128, #129, #132
# Each line below tests one specific detection case.

# --- Must detect (true positives) ---
library(findedge0)
require(findedge1)
requireNamespace("findedge2")
loadNamespace("findedge3")
glue::glue("ok")
stringr:::str_trim("ok")

# --- Named `package=` arg ---
library(lib.loc = "/tmp", package = "findedge4")
require(package = "findedge5")
requireNamespace(lib.loc = "/tmp", "findedge6")
require(lib.loc = "/tmp", package = "findedge7")

# --- R 4.4 use() --- #128
use("findedge8")
use("findedge9", c("fn_a", "fn_b"))
use(package = "findedge10")

# --- getFromNamespace --- #129
getFromNamespace("fn_x", "findedge11")
getFromNamespace(x = "fn_x", ns = "findedge12")

# --- Must NOT detect (false positives) ---
# This comment with blabla::dontfind_a shouldn't trigger
x <- "following-sibling::dontfind_b"                       # #120 xpath-like
y <- "label::after"                                        # #132 CSS
z <- sprintf('%s::plot()', 'dontfind_c')                   # #129 '%s'
"library(dontfind_d)"                                      # string literal
"require(dontfind_e)"                                      # string literal
# library(dontfind_f)                                      # comment

#' Look for functions called with `::` and library/requires in one script
#'
#' @param path path to R script file
#' @param encoding Encoding passed to [readLines()] when reading `path`. Defaults
#'  to `getOption("encoding")` so the system locale is respected (important on
#'  Windows where scripts are often Latin-1 / Windows-1252).
#'
#' @importFrom stringr str_extract_all str_replace_all
#'
#' @return a vector
#' @export
#'
#' @details
#' Uses the R parser to walk the syntax tree so that occurrences of `pkg::fun`
#' or `library()/require()/requireNamespace()/loadNamespace()/use()/getFromNamespace()`
#' inside string literals or comments are ignored.
#' Named arguments such as `library(package = "pkg")` are supported, as are
#' fully-qualified forms like `base::library(pkg)` or
#' `methods::getFromNamespace(fn, "pkg")`.
#' Introspection helpers such as `packageVersion()`, `getNamespace()`,
#' `asNamespace()`, and `attachNamespace()` are **not** treated as dependency
#' introducers, because they are commonly used for version or feature checks
#' on packages that may or may not be required at runtime.
#'
#' If the file cannot be parsed as valid R (syntax error, corrupt encoding,
#' etc.), the function falls back to a regex-based detector and emits a
#' `warning()` naming the file so users can investigate.
#'
#' @examples
#' dummypackage <- system.file("dummypackage",package = "attachment")
#' # browseURL(dummypackage)
#'
#' att_from_rscript(path = file.path(dummypackage,"R","my_mean.R"))

att_from_rscript <- function(path, encoding = getOption("encoding")) {

  lines <- tryCatch(
    readLines(path, warn = FALSE, encoding = encoding),
    error = function(e) {
      warning(
        sprintf("Could not read R script '%s': %s", path, conditionMessage(e)),
        call. = FALSE
      )
      character(0)
    }
  )
  if (length(lines) == 0) return(character(0))

  out <- tryCatch(
    parse_pkgs_from_r_code(lines),
    error = function(e) {
      warning(
        sprintf(
          "Could not parse R script '%s' as valid R code (%s); falling back to text-based detection, which may return false positives.",
          path, conditionMessage(e)
        ),
        call. = FALSE
      )
      legacy_pkgs_from_r_code(lines)
    }
  )
  out <- unique(stats::na.omit(out))
  out <- out[nzchar(out) & out != "base"]
  attributes(out) <- NULL
  out
}

# Known dependency-introducing function calls and where the package arg lives.
# Each entry: `arg_name` = canonical named arg, `arg_index` = positional fallback.
# Kept intentionally narrow: only calls that *load/attach* a package are treated
# as introducing a dependency. `packageVersion()`, `getNamespace()`, and friends
# are often used for feature-detection and should not silently expand Imports.
pkg_intro_calls <- list(
  library          = list(arg_name = "package", arg_index = 1L),
  require          = list(arg_name = "package", arg_index = 1L),
  requireNamespace = list(arg_name = "package", arg_index = 1L),
  loadNamespace    = list(arg_name = "package", arg_index = 1L),
  use              = list(arg_name = "package", arg_index = 1L),
  getFromNamespace = list(arg_name = "ns",      arg_index = 2L)
)

match_call_arg <- function(call_args, arg_name, arg_index) {
  if (length(call_args) == 0) return(NULL)
  nms <- names(call_args)
  if (is.null(nms)) nms <- rep("", length(call_args))
  hit <- which(nms == arg_name)
  if (length(hit) > 0) return(call_args[[hit[1]]])
  positional <- call_args[nms == ""]
  if (length(positional) >= arg_index) return(positional[[arg_index]])
  NULL
}

arg_as_string <- function(arg) {
  if (is.null(arg)) return(NA_character_)
  if (is.character(arg) && length(arg) == 1) return(arg)
  if (is.name(arg) || is.symbol(arg)) return(as.character(arg))
  NA_character_
}

is_empty_symbol <- function(x) {
  is.symbol(x) && !nzchar(as.character(x))
}

# Skip "empty" AST nodes such as the first subscript in `x[, 1]` or the
# missing `else` branch of `if (cond) a`. The element is received as a
# function argument (never bound to a local), which avoids triggering
# "argument is missing, with no default" errors on empty-symbol values.
walk_elem <- function(el, walker) {
  if (is.null(el) || is_empty_symbol(el)) return(invisible())
  walker(el)
}

parse_pkgs_from_r_code <- function(lines) {
  exprs <- parse(text = lines, keep.source = FALSE)
  pkgs <- character(0)

  walk <- function(x) {
    if (is_empty_symbol(x) || is.null(x)) return(invisible())
    if (is.call(x)) {
      head <- x[[1]]
      if (is.call(head) && length(head) >= 2) {
        op <- tryCatch(as.character(head[[1]]), error = function(e) "")
        if (length(op) == 1 && op %in% c("::", ":::")) {
          ns <- tryCatch(as.character(head[[2]]), error = function(e) NA_character_)
          if (!is.na(ns) && nzchar(ns)) pkgs[[length(pkgs) + 1L]] <<- ns
          # Also honour fully-qualified dependency-introducing calls such as
          # `base::library(pkg)`, `base::require(pkg)`, `methods::getFromNamespace(fn, "pkg")`.
          fn_name <- tryCatch(as.character(head[[3]]), error = function(e) NA_character_)
          if (length(fn_name) == 1 && !is.na(fn_name) &&
              fn_name %in% names(pkg_intro_calls)) {
            spec <- pkg_intro_calls[[fn_name]]
            call_args <- as.list(x)[-1]
            arg <- match_call_arg(call_args, spec$arg_name, spec$arg_index)
            pkg <- arg_as_string(arg)
            if (!is.na(pkg) && nzchar(pkg)) pkgs[[length(pkgs) + 1L]] <<- pkg
          }
        }
      } else if (is.name(head)) {
        fn <- as.character(head)
        if (fn %in% c("::", ":::")) {
          if (length(x) >= 2) {
            ns <- tryCatch(as.character(x[[2]]), error = function(e) NA_character_)
            if (!is.na(ns) && nzchar(ns)) pkgs[[length(pkgs) + 1L]] <<- ns
          }
        } else if (fn %in% names(pkg_intro_calls)) {
          spec <- pkg_intro_calls[[fn]]
          call_args <- as.list(x)[-1]
          arg <- match_call_arg(call_args, spec$arg_name, spec$arg_index)
          pkg <- arg_as_string(arg)
          if (!is.na(pkg) && nzchar(pkg)) pkgs[[length(pkgs) + 1L]] <<- pkg
        }
      }
      args <- as.list(x)[-1]
      for (i in seq_along(args)) walk_elem(args[[i]], walk)
      if (is.call(head)) walk(head)
    } else if (is.expression(x) || is.pairlist(x)) {
      xs <- as.list(x)
      for (i in seq_along(xs)) walk_elem(xs[[i]], walk)
    }
  }

  exprs_list <- as.list(exprs)
  for (i in seq_along(exprs_list)) walk(exprs_list[[i]])
  unique(pkgs)
}

# Fallback used only when parse() fails on the input (e.g. non-R snippets).
# Keeps the historical regex-based behaviour so we never silently drop a file.
# Character class includes `_` so that package names like `my_pkg` are not
# truncated to `pkg` — CRAN forbids `_` but many dev / GitHub packages use it.
legacy_pkgs_from_r_code <- function(lines) {
  file <- gsub("\\\\n", " ", lines)
  file <- file[grep("^\\s*#", file, invert = TRUE)]
  pkg_points <- unlist(str_extract_all(file, "[[:alnum:]\\._]+(?=::)"))
  w.lib <- grep("library|require", file)
  if (length(w.lib)) {
    pkg_lib <- file[w.lib]
    pkg_lib <- unlist(str_extract_all(
      pkg_lib,
      "(?<=library\\()[[:alnum:]\\._\\\"]+(?=\\))|(?<=require\\()[[:alnum:]\\._\\\"]+(?=\\))|(?<=requireNamespace\\()[[:alnum:]\\._\\\"]+(?=\\))"
    ))
    pkg_lib <- str_replace_all(pkg_lib, "\\\"$|^\\\"", "")
  } else {
    pkg_lib <- character(0)
  }
  unique(c(pkg_lib, pkg_points))
}


#' Look for functions called with `::` and library/requires in folder of scripts
#'
#' @param path directory with R scripts inside or vector of R scripts
#' @param pattern pattern to detect R script files
#' @param recursive logical. Should the listing recurse into directories?
#' @param folder_to_exclude Folder to exclude during scan to detect packages. 'renv' by default.
#' @inheritParams att_from_rscript
#' @return vector of character of packages names found in the R script
#'
#' @export
#' @examples
#' dummypackage <- system.file("dummypackage",package = "attachment")
#' # browseURL(dummypackage)
#'
#' att_from_rscripts(path = file.path(dummypackage, "R"))
#' att_from_rscripts(path = list.files(file.path(dummypackage, "R"), full.names = TRUE))

att_from_rscripts <- function(path = "R", pattern = "*.[.](r|R)$", recursive = TRUE,
                              folder_to_exclude = "renv",
                              encoding = getOption("encoding")) {

  if (isTRUE(all(dir.exists(path)))) {
    all_f <- list.files(path, full.names = TRUE, pattern = pattern, recursive = recursive)
  } else if (isTRUE(all(file.exists(path)))) {
    all_f <- normalizePath(path[grepl(pattern, path)])
  } else {
    stop("Some files/directories do not exist")
  }

  if (!is.null(folder_to_exclude) && length(folder_to_exclude) > 0) {
    exclude_files <- unlist(lapply(folder_to_exclude, function(folder) {
      list.files(path = file.path(path, folder), full.names = TRUE, pattern = pattern, recursive = recursive)
    }))

    # Exclure les fichiers
    all_f <- setdiff(all_f, exclude_files)
  }


  lapply(all_f, att_from_rscript, encoding = encoding) %>%
    unlist() %>%
    unique() %>%
    na.omit()
}

pkgs_to_find <- c("find.me",
                      # "attachment",
                      # "knitr",
                      "findme1",
                      "findme2",
                      "findme3",
                      "findme4",
                      "findme5",
                      "findme6",
                      "findme1a",
                      "findme2a",
                      "findme3a",
                      "findme4a",
                      "findme5a",
                      "findme6a"
)


test_that("rscript well parsed", {

  res <- att_from_rscript(path = "f2.R")

  expect_equal(sort(res), sort(pkgs_to_find))

  # {base} not listed
  expect_false("base" %in% res)

})

# att_from_rscripts accept a vector of files or directory
dir_with_R <- tempfile(pattern = "rscripts")
dir.create(dir_with_R)
file.copy("f2.R", to = file.path(dir_with_R, "f2.R"))
path_copy <- file.path(dir_with_R, "f2b.R")
file.copy("f2.R", to = path_copy)
file.copy("f1.Rmd", to = file.path(dir_with_R, "f1.Rmd"))
# Modify second file to add a dep
lines2 <- readLines(path_copy)
lines2[1] <- paste(lines2[1], "; library(find.rscript)")
writeLines(lines2, path_copy)

res_dir <- att_from_rscripts(path = dir_with_R)

all_R_files <- list.files(dir_with_R, full.names = TRUE)
res_files <- att_from_rscripts(path = all_R_files)

test_that("att_from_rscripts well parsed", {
  expect_equal(sort(res_dir), sort(c(pkgs_to_find, "find.rscript")))
  expect_equal(sort(res_files), sort(c(pkgs_to_find, "find.rscript")))

  # {base} not listed
  expect_false("base" %in% res_dir)
  expect_false("base" %in% res_files)

})

unlink(dir_with_R, recursive = TRUE)

# Test escape code not used ----
newline_script <- att_from_rscript(path = "escape_newline.R")

test_that("newline correctly escaped", {
  # Only `glue` is a real R call here (via `glue::glue_collapse`).
  # `rmarkdown` and `knitr` only appear inside glue-template string literals
  # and must not be detected (previous regex-based detection produced these
  # as false positives).
  expect_equal(sort(newline_script), "glue")
  expect_false("nknitr" %in% newline_script)
  expect_false("knitr" %in% newline_script)
  expect_false("rmarkdown" %in% newline_script)
})



test_that("folder_to_exclude works in att_from_rscripts", {


  dir_with_R <- tempfile(pattern = "rscripts")
  dir.create(dir_with_R)
  file.copy("f2.R", to = file.path(dir_with_R, "f2.R"))
  dir.create(file.path(dir_with_R,"renv"))
  dir.create(file.path(dir_with_R,"avoid"))
  dir.create(file.path(dir_with_R,"keep"))
  file.copy("f3.R", to = file.path(dir_with_R,"renv", "f3.R"))
  file.copy("f4.R", to = file.path(dir_with_R,"avoid", "f4.R"))
  file.copy("f5.R", to = file.path(dir_with_R,"keep", "f5.R"))
  file.copy("f1.Rmd", to = file.path(dir_with_R, "f1.Rmd"))

  res_dir <- att_from_rscripts(path = dir_with_R,folder_to_exclude = c("renv","avoid"))

  expect_true("find.me5.from.keep" %in% res_dir)
  expect_false("dont.find.me3" %in% res_dir)
  expect_false("dont.find.me4" %in% res_dir)




})

# Detection edge cases: issues #120, #128, #129, #132
# See tests/testthat/f_edge_cases.R for the fixture.

res_edge <- att_from_rscript(path = "f_edge_cases.R")

must_find <- c(
  "findedge0", "findedge1", "findedge2", "findedge3", "glue", "stringr",
  "findedge4", "findedge5", "findedge6", "findedge7",
  "findedge8", "findedge9", "findedge10",
  "findedge11", "findedge12"
)
must_not_find <- c(
  "dontfind_a", "dontfind_b", "dontfind_c", "dontfind_d", "dontfind_e", "dontfind_f",
  "sibling", "label", "s"
)

test_that("edge cases: all true positives detected", {
  for (pkg in must_find) {
    expect_true(pkg %in% res_edge, info = paste("should detect", pkg))
  }
})

test_that("edge cases: no false positives from strings or comments", {
  for (pkg in must_not_find) {
    expect_false(pkg %in% res_edge, info = paste("should NOT detect", pkg))
  }
})

test_that("edge cases: base still filtered", {
  expect_false("base" %in% res_edge)
})

test_that("fully-qualified intro calls detect the inner package", {
  tf <- tempfile(fileext = ".R")
  on.exit(unlink(tf))
  writeLines(c(
    'base::library(findqual1)',
    'base::require(findqual2)',
    'base::requireNamespace("findqual3")',
    'base::loadNamespace("findqual4")',
    'methods::getFromNamespace("fn", "findqual5")'
  ), tf)
  res <- att_from_rscript(tf)
  for (pkg in paste0("findqual", 1:5)) {
    expect_true(pkg %in% res, info = paste("should detect", pkg))
  }
  expect_false("base" %in% res)
})

test_that("legacy fallback preserves underscores in package names", {
  # `a::b::c` is not valid R, so parse() fails and the legacy regex
  # detector is invoked with a warning. Underscored names must survive.
  tf <- tempfile(fileext = ".R")
  on.exit(unlink(tf))
  writeLines(c(
    'my_pkg_with_under_score::fn()',
    'library(pkg_one_two)',
    'a::b::c'   # parse error triggers fallback
  ), tf)
  res <- suppressWarnings(att_from_rscript(tf))
  expect_true("my_pkg_with_under_score" %in% res)
  expect_true("pkg_one_two" %in% res)
})

#' return package dependencies from NAMESPACE file
#'
#' @param path path to NAMESPACE file
#' @param document Run function roxygenise of roxygen2 package
#' @param clean Logical. Whether to remove the original NAMESPACE before updating
#'
#' @return a vector
#' @export
#' @importFrom stringr str_match
#' @importFrom stats setNames na.omit
#' @importFrom utils read.table
#' @importFrom roxygen2 roxygenise
#'
#' @examples
#' tmpdir <- tempfile(pattern = "namespace")
#' dir.create(tmpdir)
#' file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
#'  recursive = TRUE)
#' dummypackage <- file.path(tmpdir, "dummypackage")
#' # browseURL(dummypackage)
#' att_from_namespace(path = file.path(dummypackage, "NAMESPACE"))
#'
#' # Clean temp files after this example
#' unlink(tmpdir, recursive = TRUE)

att_from_namespace <- function(path = "NAMESPACE", document = TRUE, clean = TRUE) {
  if (!file.exists(path)) {
    stop(path, " does not exists. ",
         "You may want to use 'attachment::att_amend_desc()' or 'roxygen2::roxygenise()' first.")}

  path <- normalizePath(path)
  if (isTRUE(document)) {
    message("Updating ", basename(dirname(path)), " documentation")
    # devtools::document(dirname(path))
    if (isTRUE(clean)) {
        file.remove(path)
        cat("# Generated by roxygen2: do not edit by hand
\n", file = path)
    }
    withr::with_dir(dirname(path), {
      roxygenise(dirname(path), roclets = NULL)
      roxygen2::load_pkgload(path = path)
    })
  }
  base <- try(readLines(path), silent = TRUE)
  base <- try(base[!grepl("^#|^$", base)], silent = TRUE)
  # base <- try(read.table(path)[["V1"]], silent = TRUE)
  if (!isTRUE(inherits(base, "try-error"))) {
    out <- na.omit(unique(c(
      unique(str_match(base, "importFrom\\(([[:alnum:]\\.]+),.*")[, 2]),
      unique(str_match(base, "import\\(([[:alnum:]\\.]+).*")[, 2])
  )))
  } else {
    message("att_from_namespace() failed,",
    " package were not retrieved from NAMESPACE")
    out <- NULL
  }
  c(out, NULL)
}

#' Get all dependencies from a Rmd file
#'
#' @param path Path to a Rmd file
#' @param temp_dir Path to temporary script from purl vignette
#' @param warn -1 for quiet warnings with purl, 0 to see warnings
#' @param inside_rmd Logical or `NULL`. Whether the function is being called
#'  from inside a knit session, in which case the actual purl step must be
#'  delegated to an external R process. When `NULL` (the default), this is
#'  auto-detected via `knitr::opts_knit$get("out.format")`.
#' @param inline Logical. Default TRUE. Whether to explore inline code for dependencies.
#' @inheritParams knitr::purl
#'
#' @importFrom stringr str_extract
#' @importFrom knitr purl
#'
#' @return vector of character of packages names found in the Rmd
#'
#' @examples
#'
#' dummypackage <- system.file("dummypackage",package = "attachment")
#' # browseURL(dummypackage)
#' att_from_rmd(path = file.path(dummypackage,"vignettes/demo.Rmd"))
#'
#' @export
att_from_rmd <- function(path, temp_dir = tempdir(), warn = -1,
                         encoding = getOption("encoding"),
                         inside_rmd = NULL, inline = TRUE) {
  if (missing(path)) {stop("argument 'path' is missing, with no default")}

  if (is.null(inside_rmd)) {
    inside_rmd <- !is.null(knitr::opts_knit$get("out.format"))
  }

  op <- options(knitr.purl.inline = inline)
  on.exit(options(op))

  r_file <- normalizePath(file.path(temp_dir, basename(gsub("[.]([[:alnum:]])*$", ".R", path))), mustWork = FALSE, winslash = "\\")
  path <- normalizePath(path, winslash = "\\")

  # Need an external script to run on windows because of \\ path
  runR <- tempfile(fileext = "run.R")

  cat(
    paste0('warn_user <- getOption("warn");',
           'options(warn=', warn, ');',
           'invisible(knitr::purl("', gsub("\\", "\\\\", path, fixed = TRUE), '"',
           ', output = "', gsub("\\", "\\\\", r_file, fixed = TRUE), '"',
           ', encoding = "', encoding, '"',
           ', documentation = 0, quiet = TRUE));',
           'options(warn=warn_user)'
    )
    , file = runR)


  if (isTRUE(inside_rmd)) {
    # Purl in a new environment to avoid knit inside knit if function is inside Rmd file
    file <- system(
      paste(normalizePath(file.path(Sys.getenv("R_HOME"), "bin", "Rscript"), mustWork = FALSE), runR)
    )
  } else {
    source(runR)
  }

  # Add yaml to the file
  the_outputs <- rmarkdown::yaml_front_matter(path)$output
  if (is.null(names(the_outputs))) {
    yaml_pkg <- unlist(the_outputs)
  } else {
    yaml_pkg <- names(the_outputs)
  }
  yaml <- c("\n# yaml to parse \n",
            paste(yaml_pkg, "\n"))
  cat(yaml, file = r_file, append = TRUE)
  res <- att_from_rscript(r_file)

  # clean tempdir
  file.remove(runR)
  file.remove(r_file)

  return(res)
}

#' Get all packages called in vignettes folder
#'
#' @param path path to directory with Rmds or vector of Rmd files
#' @param pattern pattern to detect Rmd files
#' @param recursive logical. Should the listing recurse into directories?
#' @param folder_to_exclude Folder to exclude during scan to detect packages 'renv' by default
#' @inheritParams att_from_rmd
#'
#' @return Character vector of packages called with library or require.
#' When the directory contains "vignettes" in its path, `knitr` is always added
#' and the vignette engine is inferred from the files actually present:
#' `rmarkdown` is added when `.Rmd` files are found, `quarto` when `.qmd`
#' files are found (both when the directory mixes the two). If the directory
#' is empty, `rmarkdown` is added as a safe default.
#'
#' @examples
#' dummypackage <- system.file("dummypackage",package = "attachment")
#' # browseURL(dummypackage)
#' att_from_rmds(path = file.path(dummypackage,"vignettes"))

#' @export
att_from_rmds <- function(path = "vignettes",
                          pattern = "*.[.](Rmd|rmd|qmd)$",
                          recursive = TRUE, warn = -1,
                          inside_rmd = NULL, inline = TRUE,folder_to_exclude = "renv") {

  if (isTRUE(all(dir.exists(path)))) {
    all_f <- list.files(path, full.names = TRUE, pattern = pattern, recursive = recursive)
  } else if (isTRUE(all(file.exists(path)))) {
    all_f <- normalizePath(path[grepl(pattern, path)])
  } else {
    stop("Some files/directories do not exist")
  }

  if (!is.null(folder_to_exclude) && length(folder_to_exclude) > 0) {
    exclude_files <- unlist(lapply(folder_to_exclude, function(folder) {
      list.files(path = file.path(path, folder), full.names = TRUE, pattern = pattern, recursive = recursive)
    }))

    # Exclure les fichiers
    all_f <- setdiff(all_f, exclude_files)
  }



  res <- lapply(all_f,
                function(x) att_from_rmd(
                  x, warn = warn,
                  inside_rmd = inside_rmd, inline = inline)
  ) %>%
    unlist() %>%
    unique() %>%
    na.omit()

  if (isTRUE(any(grepl("vignettes", path)))) {
    engine <- character(0)
    if (length(all_f)) {
      if (any(grepl("\\.qmd$", all_f, ignore.case = TRUE))) engine <- c(engine, "quarto")
      if (any(grepl("\\.rmd$", all_f, ignore.case = TRUE))) engine <- c(engine, "rmarkdown")
    }
    if (length(engine) == 0) engine <- "rmarkdown"
    unique(c("knitr", engine, res))
  } else {
    res
  }
}

#' @rdname att_from_rmds
#' @export
att_from_qmds <- att_from_rmds

#' @rdname att_from_rmd
#' @export
att_from_qmd <- att_from_rmd

# Issue #131: quarto .qmd vignettes should not force `rmarkdown` into Suggests.

make_vignettes_dir <- function(exts) {
  d <- tempfile(pattern = "vig")
  dir.create(file.path(d, "vignettes"), recursive = TRUE)
  for (ext in exts) {
    f <- file.path(d, "vignettes", paste0("demo_", ext, ".", ext))
    writeLines(c(
      "---",
      "title: demo",
      "---",
      "",
      "```{r}",
      "library(findme.quarto)",
      "```"
    ), f)
  }
  file.path(d, "vignettes")
}

test_that("qmd-only vignettes dir yields quarto but not rmarkdown", {
  p <- make_vignettes_dir("qmd")
  res <- attachment::att_from_rmds(path = p)
  expect_true("knitr" %in% res)
  expect_true("quarto" %in% res)
  expect_false("rmarkdown" %in% res)
  unlink(dirname(p), recursive = TRUE)
})

test_that("Rmd-only vignettes dir still yields rmarkdown (backwards compat)", {
  p <- make_vignettes_dir("Rmd")
  res <- attachment::att_from_rmds(path = p)
  expect_true("knitr" %in% res)
  expect_true("rmarkdown" %in% res)
  expect_false("quarto" %in% res)
  unlink(dirname(p), recursive = TRUE)
})

test_that("mixed qmd + Rmd vignettes dir yields both engines", {
  p <- make_vignettes_dir(c("qmd", "Rmd"))
  res <- attachment::att_from_rmds(path = p)
  expect_true("knitr" %in% res)
  expect_true("quarto" %in% res)
  expect_true("rmarkdown" %in% res)
  unlink(dirname(p), recursive = TRUE)
})

# Issue #106: att_from_rmd() should auto-detect whether it is called
# from inside a knit session, so the user does not have to pass inside_rmd.

test_that("inside_rmd defaults to NULL (auto-detect)", {
  default_val <- formals(attachment::att_from_rmd)$inside_rmd
  expect_null(default_val)
})

test_that("inside_rmd = NULL outside knit returns package deps", {
  skip_if_not(file.exists("f1.Rmd"))
  res <- suppressMessages(suppressWarnings(
    attachment::att_from_rmd(path = "f1.Rmd", inside_rmd = NULL)
  ))
  expect_type(res, "character")
  expect_gt(length(res), 0)
})

test_that("auto-detect reads knitr::opts_knit$get('out.format')", {
  # Unit-level check of the detection expression used by att_from_rmd():
  # outside a knit session it must be NULL, and once out.format is set it
  # must be non-NULL. We do NOT re-run the purl pipeline here because that
  # spawns an external Rscript via system() and would make the test slow
  # and OS-dependent.
  expect_null(knitr::opts_knit$get("out.format"))

  withr::defer(knitr::opts_knit$restore())
  knitr::opts_knit$set(out.format = "markdown")
  expect_false(is.null(knitr::opts_knit$get("out.format")))
})

# All code including inline code ----
# Rmd packages listed to be found
all_to_be_found <- c(
  "find.me",
  "knitr",
  "findme1",
  "findme2",
  "findme3",
  "findme4",
  "findme5",
  "findme6",
  "findme1a",
  "findme2a",
  "findme3a",
  "findme4a",
  "findme5a",
  "findme6a",
  "find.inline",
  "bookdown",
  "rmarkdown",
  "emo",
  "findme.quarto",
  "utils"
)

# One Rmd with YAML ---
test_that("rmd well parsed", {
  # Multiple outputs
  res <- sort(attachment::att_from_rmd(path = "yaml_rmd.Rmd"))
  expect_equal(sort(res), sort(c( "emo", "bookdown", "rmarkdown", "knitr")))

  # One output
  res <- sort(attachment::att_from_rmd(path = "f1.Rmd"))
  expect_equal(sort(res),
               sort(sort(setdiff(
                 all_to_be_found, c("emo", "rmarkdown", "findme.quarto", "utils")))))
})

# A directory ----
test_that("rmds well parsed", {
  res <- sort(attachment::att_from_rmds(path = "."))
  expect_equal(sort(res), sort(all_to_be_found))
})

# Without inline code ----
test_that("rmd well parsed", {
  res <- sort(attachment::att_from_rmds(path = ".", inline = FALSE))
  # attachment::att_from_rmd("f1.Rmd")
  expect_equal(sort(res), sort(setdiff(all_to_be_found, c("find.inline", "emo"))))
})

# Test inside ----
test_that("test inside rmd works", {
  if (isTRUE(rmarkdown::pandoc_available("1.12.3"))) {
    success_file <- rmarkdown::render("insidermd.Rmd", quiet = TRUE)
    expect_equal(basename(success_file), "insidermd.html")
    # clean
    file.remove(success_file)
  } else {
    expect_error(rmarkdown::render("insidermd.Rmd", quiet = TRUE))
  }
})

# Test quarto and new way of chunk options with knitr>=1.35 ----
if (utils::packageVersion("knitr") >= "1.35") {
  test_that("att_from_rmds with qmd works", {
    res <- attachment::att_from_rmds(path = "quarto.qmd")
    expect_true("findme.quarto" %in% res)
    expect_true(!"dontfindme.quarto" %in% res)
  })

  test_that("att_from_rmd with qmd works", {
    res <- attachment::att_from_rmd(path = "quarto.qmd")
    expect_true("findme.quarto" %in% res)
    expect_true(!"dontfindme.quarto" %in% res)
  })

  test_that("att_from_qmds works", {
    res <- attachment::att_from_qmds(path = "quarto.qmd")
    expect_true("findme.quarto" %in% res)
    expect_true(!"dontfindme.quarto" %in% res)
  })

  test_that("att_from_qmd qmd works", {
    res <- attachment::att_from_qmd(path = "quarto.qmd")
    expect_true("findme.quarto" %in% res)
    expect_true(!"dontfindme.quarto" %in% res)
  })
}

# Test that warn level is set back to user's one after ----
test_that("rmds well parsed", {

  withr::with_options(list(warn = 0), {

    res <- sort(attachment::att_from_rmds(path = "."))
    expect_equal(sort(res), sort(all_to_be_found))

    expect_equal(getOption("warn"), 0)
  })

  withr::with_options(list(warn = -1), {

    res <- sort(attachment::att_from_rmds(path = "."))
    expect_equal(sort(res), sort(all_to_be_found))

    expect_equal(getOption("warn"), -1)
  })
})




test_that("folder_to_exclude works in att_from_rmds", {


  dir_with_Rmd <- tempfile(pattern = "rmds")
  dir.create(dir_with_Rmd)
  file.copy("f1.Rmd", to = file.path(dir_with_Rmd, "f1.Rmd"))
  dir.create(file.path(dir_with_Rmd,"renv"))
  dir.create(file.path(dir_with_Rmd,"avoid"))
  dir.create(file.path(dir_with_Rmd,"keep"))
  file.copy("f1.Rmd", to = file.path(dir_with_Rmd,"renv", "f3.Rmd")) #library(dont.find.me3)
  file.copy("f1.Rmd", to = file.path(dir_with_Rmd,"avoid", "f4.Rmd")) #library(dont.find.me4)
  file.copy("f1.Rmd", to = file.path(dir_with_Rmd,"keep", "f5.Rmd")) #library(find.me5.from.keep)

cat("
```{r}
library(find.me5.from.keep)
```
    ",append = TRUE,file = file.path(dir_with_Rmd,"keep", "f5.Rmd"))


cat("
```{r}
library(dont.find.me3)
```
    ",append = TRUE,file = file.path(dir_with_Rmd,"renv", "f3.Rmd"))

cat("
```{r}
library(dont.find.me4)
```
    ",append = TRUE,file = file.path(dir_with_Rmd,"avoid", "f4.Rmd"))

  res_dir <- att_from_rmds(path = dir_with_Rmd,folder_to_exclude = c("renv","avoid"),recursive = TRUE)

  expect_true("find.me5.from.keep" %in% res_dir)
  expect_false("dont.find.me3" %in% res_dir)
  expect_false("dont.find.me4" %in% res_dir)




})

#' dependencies_file_text
#' 
#' Create the text of create_dependencies_file()
#' 
#' @param ll a vector of all packages
#' @param remotes_orig a vector of remotes 
#' @param install_only_if_missing Logical Modify the installation instructions to check, beforehand, if the packages are missing . (default FALSE)
#' @importFrom glue glue
#' @return a list
#' 
#' @noRd
dependencies_file_text <- function(ll, remotes_orig, install_only_if_missing = FALSE){
    
  if (length(remotes_orig) != 0) {

    remotes_orig_pkg <- gsub("^.*/|^local::|.git$", "", remotes_orig)
    remotes_without_orig <- gsub("^.*::/{0,1}", "", remotes_orig)
    # Remove remotes from ll
    ll <- ll[!ll %in% remotes_orig_pkg]
    # Install script
    inst_remotes <- remotes_orig
    # _If no (), then bioc
    w.bioc <- grepl("bioc::", remotes_orig)
    inst_remotes[w.bioc] <- glue("remotes::install_bioc('{remotes_without_orig[w.bioc]}')")
    # _If no (), then local
    w.local <- grepl("local::", remotes_orig)
    inst_remotes[w.local] <- glue("remotes::install_local('{remotes_without_orig[w.local]}')")
    # _If no (), then git
    w.git <- grepl("git::", remotes_orig)
    inst_remotes[w.git] <- glue("remotes::install_git('{remotes_without_orig[w.git]}')")
    # _If no (), then git
    w.gitlab <- grepl("gitlab::", remotes_orig)
    inst_remotes[w.gitlab] <- glue("remotes::install_gitlab('{remotes_without_orig[w.gitlab]}')")
    # _If no (), then github
    w.github <- !grepl("\\(", remotes_orig) & !grepl("remotes::", inst_remotes)
    inst_remotes[w.github] <- glue("remotes::install_github('{remotes_without_orig[w.github]}')")

    
    # Install only if missing
    if (isTRUE(install_only_if_missing)) {
      inst_remotes <-
        paste0(
          "if(isFALSE(requireNamespace('",
          remotes_orig_pkg,
          "', quietly = TRUE))) {","message('installation of " ,remotes_orig_pkg,"');", inst_remotes,
          "}"
        )
      
      install_pkg_remote <-
        "if(isFALSE(requireNamespace('remotes', quietly = TRUE))) {install.packages(\"remotes\")}"
    } else {
       install_pkg_remote <- "install.packages(\"remotes\")"
      }
    
    # _Others (WIP...)
    inst_remotes[!(w.github | w.local | w.bioc | w.git | w.gitlab)] <- remotes_orig[!(w.github | w.local | w.bioc | w.git | w.gitlab)]

    # Store content
    remotes_content <- paste("# Remotes ----",
                             install_pkg_remote,
                             paste(inst_remotes, collapse = "\n"),
                             sep = "\n")
  } else {
    remotes_content <- "# No Remotes ----"
  }

  

  if (length(ll) != 0) {

    attachment_content <- glue::glue(
      '
# Attachments ----
to_install <- c("*{glue::glue_collapse(as.character(ll), sep="\\", \\"")}*")
  for (i in to_install) {
    message(paste("looking for ", i))
    if (!requireNamespace(i, quietly = TRUE)) {
      message(paste("     installing", i))
      install.packages(i)
    }
  }\n\n', .open = "*{", .close = "}*")
  } else {
    attachment_content <- glue::glue(
      '
# No attachments ----
      \n\n', .open = "*{", .close = "}*")
  }
    content <- list("remotes_content" = remotes_content,
                    "attachment_content" = attachment_content)
  return(content)
}

test_that("dependencies_file_text works", {
  expect_true(inherits(dependencies_file_text, "function")) 
  
  remo <- c("ThinkR-open/attachment", "local::/path/fakelocal", "gitlab::statnmap/fakepkg", 
"bioc::3.3/fakepkgbioc", "git::https://github.com/fakepkggit.git", 
"git::https://MyForge.com/fakepkggit2r", "ThinkR-open/fusen")
ll <- c("knitr", "magrittr", "rmarkdown", "testthat")

thetext <- dependencies_file_text(ll, remo, FALSE)
  expect_equal(class(thetext), "list")
  expect_length(thetext, 2)
  expect_equal(names(thetext), c("remotes_content","attachment_content"))
  
  

  expect_equal(thetext[1], list(remotes_content = "# Remotes ----\ninstall.packages(\"remotes\")\nremotes::install_github('ThinkR-open/attachment')\nremotes::install_local('path/fakelocal')\nremotes::install_gitlab('statnmap/fakepkg')\nremotes::install_bioc('3.3/fakepkgbioc')\nremotes::install_git('https://github.com/fakepkggit.git')\nremotes::install_git('https://MyForge.com/fakepkggit2r')\nremotes::install_github('ThinkR-open/fusen')"))
  
  expect_equal(thetext[2], list(attachment_content = structure("# Attachments ----\nto_install <- c(\"knitr\", \"magrittr\", \"rmarkdown\", \"testthat\")\n  for (i in to_install) {\n    message(paste(\"looking for \", i))\n    if (!requireNamespace(i, quietly = TRUE)) {\n      message(paste(\"     installing\", i))\n      install.packages(i)\n    }\n  }\n", class = c("glue", 
"character"))))



remo <- NULL
ll <- NULL

thetext2 <- dependencies_file_text(ll, remo, FALSE)
expect_equal(class(thetext2), "list")
expect_length(thetext2, 2)
expect_equal(names(thetext2), c("remotes_content", "attachment_content"))

expect_equal(thetext2[1],list(remotes_content = "# No Remotes ----"))

expect_equal(thetext2[2], list(attachment_content = structure("# No attachments ----\n      \n", class = c("glue", 
"character"))))

})
  

#' Deprecated functions
#'
#' @description
#' List of functions deprecated. They will be removed in a future release.
#'
#'
#' @name attachment-deprecated
#' @keywords internal
#' @return List of functions used for deprecation side effects.
#' Output depends on the deprecated function.
#' @section Details:
#' \tabular{rl}{
#'    `att_to_description` \tab is now called `att_amend_desc`
#'    so that it is the first function proposed when using auto-completion\cr
#' }
NULL

#' @rdname attachment-deprecated
#' @export
#' @inheritParams att_amend_desc
att_to_description <- function(path = ".",
                               path.n = "NAMESPACE",
                               path.d = "DESCRIPTION",
                               dir.r = "R",
                               dir.v = "vignettes",
                               dir.t = "tests",
                               extra.suggests = NULL,
                               pkg_ignore = NULL,
                               document = TRUE,
                               normalize = TRUE,
                               inside_rmd = NULL) {
  .Deprecated("att_amend_desc")
  att_amend_desc(path = path,
                 path.n = path.n,
                 path.d = path.d,
                 dir.r = dir.r,
                 dir.v = dir.v,
                 dir.t = dir.t,
                 extra.suggests = extra.suggests,
                 pkg_ignore = pkg_ignore,
                 document = document,
                 normalize = normalize,
                 inside_rmd = inside_rmd)
}

# This code contains knitr on a newline and should not be read with `\nknitr`

create_vignette_head <- function(pkg, vignette_name, yaml_options = NULL) {
  pkgname <- basename(pkg)

  # Get all yaml options except Title, output, editor_options
  yaml_options <- yaml_options[
    !names(yaml_options) %in% c("output", "title", "editor_options")]

  enc2utf8(
    glue(
      '---
title: ".{vignette_name}."
output: rmarkdown::html_vignette',
      ifelse(length(yaml_options) != 0,
             glue::glue_collapse(
               c("",
                 glue("{names(yaml_options)}: \"{yaml_options}\""), ""),
               sep = "\n"),
             "\n"),
      'vignette: >
  %\\VignetteIndexEntry{.{vignette_name}.}
  %\\VignetteEngine{knitr::rmarkdown}
  %\\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(.{pkgname}.)
```
    ',
    .open = ".{", .close = "}."
    )
  )
}

#' Create the list of instructions to install dependencies from a DESCRIPTION file
#' 
#' Outputs the list of instructions and a "dependencies.R" file with instructions in the "inst/" directory
#'
#' @param path path to the DESCRIPTION file
#' @param field DESCRIPTION field to parse, "Import" and "Depends" by default. Can add "Suggests"
#' @param to path where to save the dependencies file. Set to "inst/dependencies.R" by default. Set to `NULL` if you do not want the file, but only the instructions as a list of character.
#' @param open_file Logical. Open the file created in an editor
#' @param ignore_base Logical. Whether to ignore package coming with base, as they cannot be installed (default TRUE)
#' @param install_only_if_missing Logical Modify the installation instructions to check, beforehand, if the packages are missing . (default FALSE)
#' @export
#' @return List of R instructions to install all dependencies from a DESCRIPTION file. Side effect: creates a R file containing these instructions.
#' @importFrom glue glue glue_collapse
#' @importFrom desc description
#' @importFrom utils packageDescription
#'
#' @examples
#' # Create a fake package
#' tmpdir <- tempfile(pattern = "depsfile")
#' dir.create(tmpdir)
#' file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
#'           recursive = TRUE)
#' dummypackage <- file.path(tmpdir, "dummypackage")
#'
#' # Create the dependencies commands but no file
#' create_dependencies_file(
#'   path = file.path(dummypackage,"DESCRIPTION"),
#'   to = NULL,
#'   open_file = FALSE)
#'
#' # Create the dependencies files in the package
#' create_dependencies_file(
#'   path = file.path(dummypackage,"DESCRIPTION"),
#'   to = file.path(dummypackage, "inst/dependencies.R"),
#'   open_file = FALSE)
#' list.files(file.path(dummypackage, "inst"))
#' # browseURL(dummypackage)
#'
#' # Clean temp files after this example
#' unlink(tmpdir, recursive = TRUE)
create_dependencies_file <- function(path = "DESCRIPTION",
                                     field = c("Depends", "Imports"),
                                     to = "inst/dependencies.R", 
                                     open_file = TRUE,
                                     ignore_base = TRUE,
                                     install_only_if_missing = FALSE) {

  # get all packages
  ll <- att_from_description(path = path, field = field)
  # get pkg in remotes
  if (isTRUE(ignore_base)) {
    to_remove <-
      which(lapply(ll , packageDescription, field = "Priority") == "base")
    if (length(to_remove) > 0) {
      ll <- ll[-to_remove]
    }

  }

  desc <- description$new(path)
  # Get previous dependencies in Description in case version is set
  remotes_orig <- desc$get_remotes()

  content <- dependencies_file_text(ll = ll,
                         remotes_orig = remotes_orig,
                         install_only_if_missing = install_only_if_missing)

  if (!is.null(to)) {
    if (!dir.exists(dirname(to))) {
      dir.create(dirname(to), recursive = TRUE, showWarnings = FALSE)
      dir_to <- normalizePath(dirname(to))
    } else {
      dir_to <- normalizePath(dirname(to))
    }
    
    the_file <- file.path(dir_to, basename(to))
    # file.create(the_file)
    cat(unlist(content), sep  = "\n", file = the_file)

    if (interactive() && open_file) {
      utils::file.edit(file, editor = "internal")
    }
    
    return(invisible(content))
  } else {
    return(content)
  }
  

}

# Copy package in a temporary directory
tmpdir <- tempfile(pattern = "pkgdeps")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)

# Create the dependencies commands but no file
test_that("create_dependencies_file does not create file with NULL", {
  
  out_list <- create_dependencies_file(
    path = file.path(dummypackage, "DESCRIPTION"),
    to = NULL,
    field = c("Depends", "Imports", "Suggests"),
    open_file = FALSE)
  
  expect_false(file.exists(
    file.path(dummypackage, "inst", "dependencies.R")))
})

create_dependencies_file(
  path = file.path(dummypackage, "DESCRIPTION"),
  to = file.path(dummypackage, "inst", "dependencies.R"),
  field = c("Depends", "Imports", "Suggests"),
  open_file = FALSE)

  expect_true(file.exists(
    file.path(dummypackage, "inst", "dependencies.R")))
  
dep_file_without_remotes <- readLines(file.path(dummypackage, "inst", "dependencies.R"))

test_that("create-dependencies-file works without remotes", {
  expect_equal(dep_file_without_remotes[1], "# No Remotes ----")
  expect_equal(dep_file_without_remotes[3], "to_install <- c(\"glue\", \"knitr\", \"magrittr\", \"rmarkdown\", \"stringr\", \"testthat\")")
})
  

  # Add remotes in DESCRIPTION

  # Test internal_remotes_to_desc ----
  tmpdir <- tempfile(pattern = "pkgwithremotes")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")

  path.d <- file.path(dummypackage, "DESCRIPTION")

  cat(c("Remotes:\n     ThinkR-open/attachment,\n     local::/path/fakelocal,\n     gitlab::statnmap/fakepkg,\n     bioc::3.3/fakepkgbioc,\n     git::https://github.com/fakepkggit.git,\n     git::https://MyForge.com/fakepkggit2r,\n     ThinkR-open/fusen\n"), append = TRUE,
      file = path.d)

  new_desc <- readLines(path.d)

create_dependencies_file(path = file.path(dummypackage,"DESCRIPTION"),
                         to = file.path(dummypackage, "inst/dependencies.R"),
                         field = c("Depends", "Imports", "Suggests"),
                         open_file = FALSE)

dep_file_with_remotes <- readLines(file.path(tmpdir, "dummypackage", "inst/dependencies.R"))

test_that("create-dependencies-file works with remotes", {
  expect_equal(dep_file_with_remotes[1], "# Remotes ----")
  expect_equal(dep_file_with_remotes[3], "remotes::install_github('ThinkR-open/attachment')")
  expect_equal(dep_file_with_remotes[4], "remotes::install_local('path/fakelocal')")
  expect_equal(dep_file_with_remotes[5], "remotes::install_gitlab('statnmap/fakepkg')")
  expect_equal(dep_file_with_remotes[6], "remotes::install_bioc('3.3/fakepkgbioc')")
  expect_equal(dep_file_with_remotes[7], "remotes::install_git('https://github.com/fakepkggit.git')")
  expect_equal(dep_file_with_remotes[8], "remotes::install_git('https://MyForge.com/fakepkggit2r')")
  expect_equal(dep_file_with_remotes[9], "remotes::install_github('ThinkR-open/fusen')")
  expect_equal(dep_file_with_remotes[11], "to_install <- c(\"glue\", \"knitr\", \"magrittr\", \"rmarkdown\", \"stringr\", \"testthat\")")
})


create_dependencies_file(path = file.path(dummypackage,"DESCRIPTION"),
                         to = file.path(dummypackage, "inst/dependencies.R"),
                         field = c("Depends", "Imports", "Suggests"),
                         open_file = FALSE,
                         install_only_if_missing = TRUE)

dep_file_with_remotes_install_only_if_missing <- readLines(file.path(tmpdir, "dummypackage", "inst/dependencies.R"))

test_that("create-dependencies-file works with remotes install_only_if_missing", {
  expect_equal(dep_file_with_remotes_install_only_if_missing[1], "# Remotes ----")
  expect_equal(dep_file_with_remotes_install_only_if_missing[3], "if(isFALSE(requireNamespace('attachment', quietly = TRUE))) {message('installation of attachment');remotes::install_github('ThinkR-open/attachment')}")
  expect_equal(dep_file_with_remotes_install_only_if_missing[4], "if(isFALSE(requireNamespace('fakelocal', quietly = TRUE))) {message('installation of fakelocal');remotes::install_local('path/fakelocal')}")
  expect_equal(dep_file_with_remotes_install_only_if_missing[5], "if(isFALSE(requireNamespace('fakepkg', quietly = TRUE))) {message('installation of fakepkg');remotes::install_gitlab('statnmap/fakepkg')}")
  expect_equal(dep_file_with_remotes_install_only_if_missing[6], "if(isFALSE(requireNamespace('fakepkgbioc', quietly = TRUE))) {message('installation of fakepkgbioc');remotes::install_bioc('3.3/fakepkgbioc')}")
  expect_equal(dep_file_with_remotes_install_only_if_missing[7], "if(isFALSE(requireNamespace('fakepkggit', quietly = TRUE))) {message('installation of fakepkggit');remotes::install_git('https://github.com/fakepkggit.git')}")
  expect_equal(dep_file_with_remotes_install_only_if_missing[8],  "if(isFALSE(requireNamespace('fakepkggit2r', quietly = TRUE))) {message('installation of fakepkggit2r');remotes::install_git('https://MyForge.com/fakepkggit2r')}")
  expect_equal(dep_file_with_remotes_install_only_if_missing[9], "if(isFALSE(requireNamespace('fusen', quietly = TRUE))) {message('installation of fusen');remotes::install_github('ThinkR-open/fusen')}")
  expect_equal(dep_file_with_remotes_install_only_if_missing[11], "to_install <- c(\"glue\", \"knitr\", \"magrittr\", \"rmarkdown\", \"stringr\", \"testthat\")")
})

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)

#' Compare input from the user to config file or default inputs
#'
#' @param local_att_params List of parameters called by the user
#' @inheritParams att_amend_desc
#'
#' @noRd

compare_inputs_load_or_save <- function(path.c, local_att_params, use.config, update.config) {

    if (isTRUE(use.config) & file.exists(path.c)) {
      # reassign input value to saved parameters
      saved_att_params <- load_att_params(path_to_yaml = path.c)
      # Check if different from config and as defaults
      diff_config <- lapply(names(local_att_params), function(x) {setdiff(local_att_params[x], saved_att_params[x])})
      diff_default <- lapply(names(local_att_params), function(x) {setdiff(local_att_params[x], formals(att_amend_desc)[x])})

      if (any(lengths(diff_config) != 0) & any(lengths(diff_default) != 0)) {
        stop(c("Params in your `att_amend_desc()` and the one in the config file are different. ",
             "Please choose among `update.config = TRUE` or `use.config = FALSE`"))
      }

      params_to_load <- saved_att_params
      # for (param_name in names(saved_att_params)){
      #   assign(param_name, saved_att_params[[param_name]])
      # }
      # message(c("Documentation parameters were restored from attachment config file.\n"))
    } else if (isTRUE(update.config) | !file.exists(path.c)) {
      # save current parameters to yaml config - overwrite if already exist
      save_att_params(
        param_list = local_att_params,
        path_to_yaml = path.c,
        overwrite = TRUE)

      params_to_load <- NULL
    } else {
      message("attachment config file was not updated. Parameters used this time won't be stored.")

      params_to_load <- local_att_params
    }

    return(params_to_load)
  }

#' load_att_params
#'
#' @param path_to_yaml character The path to the yaml file
#'
#' @importFrom yaml read_yaml
#' @importFrom glue glue
#'
#' @return list A named list of att_amend_desc parameters
#'
#' @noRd
#' @examples
#' # create a list of parameters and tmp file name
#' parameter_list <- list(
#'   pkg_ignore = c("remotes", "i"),
#'   extra.suggests = c("testthat", "rstudioapi")
#' )
#' yaml_path <- paste0(tempfile(pattern = "save_att"), ".yaml")
#'
#' # save params
#' save_att_params(param_list = parameter_list,
#'                 path_to_yaml = yaml_path)
#'
#' # read yaml file
#' config_params <- load_att_params(path_to_yaml = yaml_path)
#'
#' # clear created yaml file
#' unlink(yaml_path)
#'
load_att_params <- function(
    path_to_yaml = "dev/config_attachment.yaml",
    verbose = FALSE
    ){

  # check yaml file exist
  if (isFALSE(file.exists(path_to_yaml))){
    stop(glue("The att_amend_desc() config file {path_to_yaml} does not exist"))
  }

  # read yaml
  param_list <- read_yaml(file = path_to_yaml)

  # Check each name in list corresponds to a parameter name
  att_param_names <- names(formals(att_amend_desc))
  input_names <- names(param_list)
  all_inputs_are_params <- all(input_names %in% att_param_names)
  if (isFALSE(all_inputs_are_params)){
    bad_names <- input_names[!input_names %in% att_param_names]
    stop(paste0("Unexpected parameters in config : ", paste0(bad_names, collapse = " ; ")))
  }

  # Show parameters used from config
  if (isTRUE(verbose)) {
    message("att_amend_desc() parameter loaded are : \n",
            paste0(
              glue("{names(param_list)} = {param_list}"),
              collapse = "\n"
            )
    )
  }

  # return parameters
  return(param_list)

}

#' save_att_params
#'
#' @param param_list list A named list of all parameters to save
#' @param path_to_yaml character The path to the yaml file
#' @param overwrite logical Whether to overwrite the yaml file if it already exists
#'
#' @importFrom yaml write_yaml
#' @return character The path to the yaml file
#' @noRd
#' @examples
#' # create a list of parameters and tmp file name
#' parameter_list <- list(
#'   pkg_ignore = c("remotes", "i"),
#'   extra.suggests = c("testthat", "rstudioapi")
#' )
#' yaml_path <- paste0(tempfile(pattern = "save_att"), ".yaml")
#'
#' # save params
#' save_att_params(param_list = parameter_list,
#'                 path_to_yaml = yaml_path)
#'
#' yaml::read_yaml(yaml_path)
#' # rstudioapi::navigateToFile(yaml_path)
#'
#' # clear created yaml file
#' unlink(yaml_path)
save_att_params <- function(
    param_list,
    path_to_yaml = "dev/config_attachment.yaml",
    overwrite = FALSE
    ) {

  # Check each name in list corresponds to a parameter name
  att_param_names <- names(formals(att_amend_desc))
  input_names <- names(param_list)
  all_inputs_are_params <- all(input_names %in% att_param_names)
  if (isFALSE(all_inputs_are_params)){
    bad_names <- input_names[!input_names %in% att_param_names]
    stop(paste0("Unexpected parameters to save : ", paste0(bad_names, collapse = " ; ")))
  }

  # Create dir if missing
  dir_yaml <- normalizePath(dirname(path_to_yaml), mustWork = FALSE)
  if (!dir.exists(dir_yaml)) {
    dir.create(dir_yaml)
    add_build_ignore(basename(dir_yaml))
    }

  # Write params to yaml
  yaml_exists <- file.exists(path_to_yaml)

  if (isTRUE(yaml_exists & !overwrite)) {
    stop("yaml file already exists and overwriting is not permitted")
  } else {
    write_yaml(
      x = param_list,
      file = path_to_yaml,
      indent.mapping.sequence = TRUE
    )
    message("Saving attachment parameters to yaml config file")
  }

  return(path_to_yaml)
}

test_that("save_att_params works", {

  # Test error if incorrect list is provided
  expect_error(object = save_att_params(param_list =
                                          list(notanattparam = 3,
                                               pkg_ignore = c("remotes"),
                                               anotherbadparam = c("testthat")
                                               )
                                        ),
               regexp = "Unexpected parameters to save : notanattparam ; anotherbadparam")
  
  # Create tmp dir
  tmpdir <- tempfile(pattern = "att")
  dir.create(tmpdir)
  path_to_yaml <- file.path(tmpdir, "config_attachment.yaml")
  
  # Setup list of att_amend_desc() params
  param_list <- list(
    pkg_ignore = c("remotes", "i", "usethis", "rstudioapi", "renv",
                   "gitlab", "git", "local", "find.rscript", "bioc"),
    extra.suggests = c("testthat", "rstudioapi", "renv", "lifecycle"),
    dir.t = "",
    normalize = FALSE
    )
  
  # Run function to save params in yaml in tmp folder
  expect_message(
    yaml_config <- save_att_params(
      param_list = param_list,
      path_to_yaml = path_to_yaml
    ), regexp = "Saving attachment parameters to yaml config file"
  )

  # Test that yaml file is created
  expect_true(file.exists(yaml_config))
  
  # Test that reading yaml file restore the correct list of parameters
  config_data <- yaml::read_yaml(yaml_config)
  expect_equal(
    object = config_data,
    expected = param_list
    )
  
  # Test overwriting error
  expect_error(object = save_att_params(param_list = param_list,
                               path_to_yaml = path_to_yaml,
                               overwrite = FALSE),
               regexp = "yaml file already exists and overwriting is not permitted"
               )

  # Test that yaml file is updated after overwriting
  new_parameter_list <- list(
    pkg_ignore = c("remotes", "i", "usethis", "rstudioapi"),
    extra.suggests = c("testthat", "rstudioapi", "lifecycle", "git", "renv")
  )
  
  yaml_config <- save_att_params(
    param_list = new_parameter_list,
    path_to_yaml = path_to_yaml,
    overwrite = TRUE
  )
  
  config_data <- yaml::read_yaml(yaml_config)
  expect_equal(
    object = config_data,
    expected = new_parameter_list
    )
  
  # Test clearing yaml with no params
  empty_parameter_list <- list()
  
  yaml_config <- save_att_params(
    param_list = empty_parameter_list,
    path_to_yaml = path_to_yaml,
    overwrite = TRUE
  )
  
  config_data <- yaml::read_yaml(yaml_config)
  expect_equal(
    object = config_data,
    expected = empty_parameter_list
    )

  # clean
  unlink(tmpdir, recursive = TRUE)
  
})

test_that("load_att_params works", {
  
  # create a list of parameters and tmp file name
  parameter_list <- list(
    pkg_ignore = c("remotes", "i"),
    extra.suggests = c("testthat", "rstudioapi")
  )
  yaml_path <- paste0(tempfile(pattern = "save_att"), ".yaml")
  
  # test error for non-existing file
  expect_error(object = load_att_params(yaml_path),
              regexp = glue::glue("The .* does not exist"))

  
  # save params
  save_att_params(param_list = parameter_list,
                  path_to_yaml = yaml_path)
  
  # test correct list is returned
  result <- load_att_params(yaml_path)
  expect_equal(
    object = result,
    expected = parameter_list
    )
  
  # test message is returned
  expect_message(object = load_att_params(yaml_path, verbose = TRUE),
                 regexp = "att_amend_desc\\(\\) parameter loaded are : \npkg_ignore = c\\(\"remotes\", \"i\"\\)\nextra.suggests = c\\(\"testthat\", \"rstudioapi\"\\)")
  
  # add wrong param to yaml
  write(x = "randomparam:\n- randomvalue\n",
        file = yaml_path,
        append=TRUE
        )
  
  # test error for incorrect param names
  expect_error(object = load_att_params(yaml_path),
               regexp = "Unexpected parameters in config : randomparam")
  
  # clear created file
  unlink(yaml_path)

})


# att_amend_desc use saved config ----
test_that("att_amend_desc can create, use and update config file", {
  # Copy package in a temporary directory
  tmpdir <- tempfile("dummyamend")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")

  expect_false(dir.exists(file.path(dummypackage, "dev")))
  
  withr::with_dir(dummypackage, {
    # create a first config file
    att_amend_desc(
      # path = dummypackage, # default to "."
      # update.config = FALSE, # default
      # path.c = "dev/config_attachment.yaml" # default
    )
  })
  
  # Create if not exists
  expect_true(dir.exists(file.path(dummypackage, "dev")))
  expect_true(file.exists(file.path(dummypackage, "dev", "config_attachment.yaml")))
  # with default params
    
  yaml_config <- readLines(file.path(dummypackage, "dev", "config_attachment.yaml"))
  expect_equal(object = yaml_config,
               expected = c(
                 #paste0("path: ", dummypackage), # no path stored
                 "path.n: NAMESPACE", "path.d: DESCRIPTION", "dir.r: R", "dir.v: vignettes",
                 "dir.t: tests", "extra.suggests: ~", "pkg_ignore: ~", "document: yes",
                 "normalize: yes", "inside_rmd: ~", "must.exist: yes", "check_if_suggests_is_installed: yes"
               ))

  # Do not overwrite if not update.config and not default params
  withr::with_dir(dummypackage, {
    expect_error(
      att_amend_desc(
        # path = dummypackage, # default to "."
        extra.suggests = c("ggplot2"),
        document = FALSE,
        check_if_suggests_is_installed = FALSE,
        update.config = FALSE, # default
        # path.c = file.path(dummypackage, "dev", "config_attachment.yaml") " default
      ), 
      regexp = "Params in your `att_amend_desc\\(\\)` and the one in the config file are different. Please choose among `update.config = TRUE` or `use.config = FALSE`"
    )
  })
  
    # Do not overwrite if not update.config
  withr::with_dir(dummypackage, {
    expect_message(
      att_amend_desc(
        # path = dummypackage, # default to "."
        extra.suggests = c("ggplot2"),
        document = FALSE,
        check_if_suggests_is_installed = FALSE,
        use.config = FALSE,
        update.config = FALSE
        # path.c = file.path(dummypackage, "dev", "config_attachment.yaml") " default
      ), 
      regexp = "Parameters used this time won't be stored"
    )
  })
  # config not changed, parameters not used
  yaml_config <- readLines(file.path(dummypackage, "dev", "config_attachment.yaml"))
  expect_equal(object = yaml_config,
               expected = c(
                 #paste0("path: ", dummypackage), # no path stored
                 "path.n: NAMESPACE", "path.d: DESCRIPTION", "dir.r: R", "dir.v: vignettes",
                 "dir.t: tests", "extra.suggests: ~", "pkg_ignore: ~", "document: yes",
                 "normalize: yes", "inside_rmd: ~", "must.exist: yes", "check_if_suggests_is_installed: yes"
               ))
  
  # overwrite config file with new non-default parameters
  withr::with_dir(dummypackage, {
    expect_message(
      att_amend_desc(
        # path = dummypackage, # default to "."
        extra.suggests = c("ggplot2"),
        document = FALSE,
        check_if_suggests_is_installed = FALSE,
        update.config = TRUE,
        # path.c = file.path(dummypackage, "dev", "config_attachment.yaml") " default
      ),
      regexp = "'update.config' was set to TRUE, hence, 'use.config' was forced to FALSE"
    )
  })
  yaml_config <- readLines(file.path(dummypackage, "dev", "config_attachment.yaml"))
  expect_equal(object = yaml_config,
               expected = c(
                 "path.n: NAMESPACE", "path.d: DESCRIPTION", "dir.r: R", "dir.v: vignettes",
                 "dir.t: tests", "extra.suggests: ggplot2", "pkg_ignore: ~", "document: no",
                 "normalize: yes", "inside_rmd: ~", "must.exist: yes", "check_if_suggests_is_installed: no"
               ))
  
  # remove non-default edits, without updating config
  expect_message(object = att_amend_desc(path = dummypackage, use.config = FALSE),
                 regexp = "1 package\\(s\\) removed: ggplot2.")

  # re-run with saved config
  expect_message(
    object = att_amend_desc(
      path = dummypackage,
      use.config = TRUE
    ),
    regexp = "1 package\\(s\\) added: ggplot2")
  desc_file <- readLines(file.path(dummypackage, "DESCRIPTION"))
  expect_equal(desc_file[grep("Suggests: ", desc_file) + 1], "    ggplot2,")

  # error when trying to use and update config at the same time
  expect_message(
    object =att_amend_desc(
      path = dummypackage,
      use.config = TRUE,
      update.config = TRUE),
    regexp = "'update.config' was set to TRUE, hence, 'use.config' was forced to FALSE"
  )

  # Clean after
  unlink(dummypackage, recursive = TRUE)
})

#' Return all package dependencies from current package
#'
#' @param path path to the DESCRIPTION file
#' @param dput if FALSE return a vector instead of dput output
#' @param field DESCRIPTION field to parse, Import, Suggests and Depends by default
#'
#' @return A character vector with packages names
#'
#' @export
#' @importFrom stringr str_replace_all str_trim
#'
#' @examples
#'
#' dummypackage <- system.file("dummypackage", package = "attachment")
#' # browseURL(dummypackage)
#' att_from_description(path = file.path(dummypackage, "DESCRIPTION"))
#'
att_from_description <- function(path = "DESCRIPTION", dput = FALSE,
                             field = c("Depends", "Imports", "Suggests")) {
  out <- read.dcf(path)
  out <- out[, intersect(colnames(out), field)] %>%
    gsub(pattern = "\n", replacement = "") %>%
    strsplit(",") %>%
    unlist() %>%
    setNames(NULL)
  out <- out[!grepl("^R [(]", out)] %>%
    str_replace_all("\\(.+\\)","") %>%
    str_trim() %>%
    unique() %>%
    sort()



  if (!dput) {
    return(out)
  }
  dput(out)
}

extra_dev_pkg <- c(
  "renv", "fusen", "devtools",
  "roxygen2", "usethis", "pkgload",
  "testthat", "remotes", "covr",
  "attachment", "pak", "dockerfiler",
  "pkgdown"
)

#' Create reproducible environments for your R projects with \emph{renv}
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Tool to create and maintain renv.lock files.
#' The idea is to have 2 distinct files, one for development and the other for deployment.
#' Indeed, although packages like \emph{attachment} or \emph{pkgload} must be installed to develop,
#' they are not necessary in your project, package or Shiny application.
#'
#'
#' @param path Path to your current package source folder
#' @param dev_pkg Vector of packages you need for development. Use `_default`
#' (with underscore before to avoid confusing with a package name), to
#' use the default list. Use `NULL` for no extra package.
#' Use `attachment:::extra_dev_pkg` for the list.
#' @param folder_to_include Folder to scan to detect development packages
#' @param folder_to_exclude Folder to exclude during scan to detect packages.'renv' by default
#' @param output Path and name of the file created, default is `./renv.lock`
#' @param install_if_missing Logical. Install missing packages. `TRUE` by default
#' @param document Logical. Whether to run [att_amend_desc()] before
#' detecting packages in DESCRIPTION.
#' @param pkg_ignore Vector of packages to ignore from being discovered in your files.
#' This does not prevent them to be in "renv.lock" if they are recursive dependencies.
#' @inheritParams att_to_desc_from_is
#' @param ... Other arguments to pass to [renv::snapshot()]
#'
#' @return a renv.lock file
#'
#'
#' @importFrom cli cat_bullet
#' @export
#'
#' @examples
#' \dontrun{
#' # Writes a renv.lock a file in the user directory
#' create_renv_for_dev()
#' create_renv_for_dev(dev_pkg = "attachment")
#' create_renv_for_prod()
#' }
create_renv_for_dev <- function(path = ".",
                                dev_pkg = "_default",
                                folder_to_include = c("dev", "data-raw"),
                                folder_to_exclude = c("renv"),
                                output = "renv.lock",
                                install_if_missing = TRUE,
                                document = TRUE,
                                pkg_ignore = NULL,
                                check_if_suggests_is_installed = TRUE,
                                ...) {

  if (!requireNamespace("renv")) {
    stop("'renv' is required. Please install it before.")
  }

  path <- normalizePath(path)

  if (!is.null(dev_pkg) && "_default" %in% dev_pkg) {
    cli::cli_alert_info(
      paste('`dev_pkg = _default` includes: ',
            paste(extra_dev_pkg, collapse = ", ")))
    dev_pkg <- c(extra_dev_pkg, dev_pkg[dev_pkg != "_default"])
  }


# s'il n y pas de fichier DESCRIPTION on va chercher l'information ailleurs

  if (file.exists(file.path(path, "DESCRIPTION"))){

  if (isTRUE(document)) {
    # Use a temporary config_file for renv
    config_file <- file.path(path, "dev", "config_attachment.yaml")
    if (file.exists(config_file)) {
      yaml_params <- load_att_params(path_to_yaml = config_file)
      yaml_params[["check_if_suggests_is_installed"]] <- check_if_suggests_is_installed
      yamlfile <- tempfile(fileext = ".yaml")
      save_att_params(path_to_yaml = yamlfile, param_list = yaml_params)
      att_amend_desc(path,
                     use.config = TRUE,
                     path.c = yamlfile)
    } else {
      att_amend_desc(path,
                     check_if_suggests_is_installed = check_if_suggests_is_installed,
                     use.config = FALSE)
    }
  }

  if ( isTRUE(check_if_suggests_is_installed)){

  fields <- c("Depends", "Imports", "Suggests")

  } else {

  fields <- c("Depends", "Imports")

  }

  pkg_list <- unique(
    c(
      att_from_description(path = file.path(path, "DESCRIPTION"),field = fields),
      dev_pkg
    )
  )} else {
    cli::cli_alert_info("No DESCRIPTION file found")
    cli::cli_alert_info(      paste(
      "we wil parse qmd files,Rmd files and R scripts from ",path,
      "."
    ))


    pkg_list <- unique(
      c(
        att_from_rmds(path = path,recursive = TRUE, folder_to_exclude = folder_to_exclude),
        att_from_rscripts(path = path,recursive = TRUE, folder_to_exclude = folder_to_exclude),
        dev_pkg
      )
    )
  }

  # Extra folders
  folder_to_include_relative <- folder_to_include
  folder_to_include <- file.path(path, folder_to_include)
  folder_exists <- dir.exists(folder_to_include)

  if (any(!folder_exists)) {
    cli::cli_alert_info(
      paste(
        "There is no directory named: ",
        paste(folder_to_include_relative[!folder_exists], collapse = ", "),
        ". This is removed from the exploration."
      )
    )
  }

  if (any(folder_exists)) {
    folder_to_include <- folder_to_include[folder_exists]

    # folder_to_include <- folder_to_include[dir.exists(file.path(path, folder_to_include))]

    from_r_script <- att_from_rscripts(folder_to_include,, folder_to_exclude = folder_to_exclude)
    from_rmd <- att_from_rmds(folder_to_include,, folder_to_exclude = folder_to_exclude)

    pkg_list <- unique(c(pkg_list, from_r_script, from_rmd))
  }

  # Ignore
  if (!is.null(pkg_ignore)) {
    pkg_list <- pkg_list[!pkg_list %in% pkg_ignore]
  }

  # Install
  if (install_if_missing) {
    install_if_missing(pkg_list)
  }

  cli::cat_bullet(
    sprintf("create renv.lock at %s", output),
    bullet = "tick",
    bullet_col = "green"
  )

  renv::snapshot(
    packages = pkg_list,
    lockfile = output,
    prompt = FALSE,
    ...
    # type = "packages"
  )


  if (!file.exists(output)) {
    stop("error during renv.lock creation")
  }

  output
}

#' @export
#' @rdname create_renv_for_dev
create_renv_for_prod <- function(path = ".",
                                 output = "renv.lock.prod",
                                 dev_pkg = "remotes",
                                 check_if_suggests_is_installed = FALSE,
                                 folder_to_include = NULL,
                                 ...) {
  create_renv_for_dev(
    path = path,
    dev_pkg = dev_pkg,
    folder_to_include = folder_to_include,
    output = output,
    check_if_suggests_is_installed = check_if_suggests_is_installed,
    ...
  )
}

# These tests can not run on CRAN
skip_on_cran()

if (length(find.package("extrapackage", quiet = TRUE)) != 0) {
  unlink(find.package("extrapackage", quiet = TRUE), recursive = TRUE)
}

# test on dummy package
tmpdir <- tempfile(pattern = "pkgrenv")
dir.create(tmpdir)
file.copy(
  system.file("dummypackage", package = "attachment"), tmpdir,
  recursive = TRUE
)
dummypackage <- file.path(tmpdir, "dummypackage")

# Create a second dummy package that will be a dependance of dummy
# So that, we are sure it does not exists on CRAN for extra checks
extra_path <- file.path(tmpdir, "dummy.extra")
dir.create(extra_path, recursive = TRUE)
file.copy(
  system.file("dummypackage", package = "attachment"),
  extra_path,
  recursive = TRUE
)
extrapackage <- file.path(extra_path, "extrapackage")
file.rename(file.path(extra_path, "dummypackage"), extrapackage)
# Rename package and remove 'Rcpp'
desc_lines <- readLines(file.path(extrapackage, "DESCRIPTION"))
desc_lines <- gsub("dummypackage", "extrapackage", desc_lines)
desc_lines <- desc_lines[-grep("LinkingTo|Rcpp", desc_lines)]
cat(desc_lines, sep = "\n", file = file.path(extrapackage, "DESCRIPTION"))
# Remove calls to 'dummypackage' and 'Rcpp'
unlink(file.path(extrapackage, "tests"), recursive = TRUE)
# document
# inuse <- search()
att_amend_desc(path = extrapackage)
unloadNamespace("extrapackage") # for windows mainly
# Install package to make it available to {renv}
install.packages(extrapackage, repos = NULL, type = "source")

# Add a development package not required in DESCRIPTION: dummy.extra ----
lock_includes_extra_dev <- file.path(tmpdir, "for_extra_dev.lock")
lock_without_extra_dev <- file.path(tmpdir, "blank_dev.lock")
lock_includes_extra_prod <- file.path(tmpdir, "for_extra_prod.lock")
lock_without_extra_prod <- file.path(tmpdir, "blank_prod.lock")

# if (interactive()) {
#   expect_message({my_renv_extra <-
#     create_renv_for_dev(
#       path = dummypackage,
#       install_if_missing = FALSE,
#       dev_pkg = "extrapackage",
#       output = lock_includes_extra_dev)}
#     # "There is no directory named: dev, data-raw" # cli
#   )
#   expect_message({
#     # message, but not missing directories as they are skipped
#     my_renv_blank <-
#       create_renv_for_dev(
#         path = dummypackage,
#         dev_pkg = NULL,
#         install_if_missing = FALSE,
#         output = lock_without_extra_dev)}
#   )
# } else {

  expect_message({

    my_renv_extra_dev <- create_renv_for_dev(
      path = dummypackage,
      install_if_missing = FALSE,
      dev_pkg = "extrapackage",
      output = lock_includes_extra_dev,
      # force generation of a lockfile even when pre-flight validation checks have failed?
      force = TRUE)}#,
    # "There is no directory named: dev, data-raw" # cli
  )
  expect_message({
    # message, but not missing directories as they are skipped
    my_renv_blank_dev <- create_renv_for_dev(
      path = dummypackage,
      install_if_missing = FALSE,
      dev_pkg = NULL,
      output = lock_without_extra_dev,
      # force generation of a lockfile even when pre-flight validation checks have failed?
      force = TRUE)})

  # renv for prod


  expect_message({
    my_renv_extra_prod <- create_renv_for_prod(
      path = dummypackage,
      install_if_missing = FALSE,
      dev_pkg = "extrapackage",
      output = lock_includes_extra_prod,
      # force generation of a lockfile even when pre-flight validation checks have failed?
      force = TRUE)}#,
    # "There is no directory named: dev, data-raw" # cli
  )
  expect_message({
    # message, but not missing directories as they are skipped
    my_renv_blank_prod <- create_renv_for_prod(
      path = dummypackage,
      install_if_missing = FALSE,
      dev_pkg = NULL,
      output = lock_without_extra_prod,
      # force generation of a lockfile even when pre-flight validation checks have failed?
      force = TRUE)})
# }


test_that("create_renv_for_dev creates lock files", {
  expect_true(file.exists(lock_includes_extra_dev))
  expect_true(file.exists(my_renv_extra_dev))
  expect_true(file.exists(lock_without_extra_dev))
  expect_true(file.exists(my_renv_blank_dev))
  expect_true(file.exists(lock_includes_extra_prod))
  expect_true(file.exists(my_renv_extra_prod))
  expect_true(file.exists(lock_without_extra_prod))
  expect_true(file.exists(my_renv_blank_prod))
})

# print(my_renv_extra)


local_renv_extra_dev <- getFromNamespace("lockfile", "renv")(my_renv_extra_dev)
local_renv_blank_dev <- getFromNamespace("lockfile", "renv")(my_renv_blank_dev)
local_renv_extra_prod <- getFromNamespace("lockfile", "renv")(my_renv_extra_prod)
local_renv_blank_prod <- getFromNamespace("lockfile", "renv")(my_renv_blank_prod)

test_that("lockfile are renv files", {
  expect_s3_class(local_renv_extra_dev, "renv_lockfile_api")
  expect_s3_class(local_renv_blank_dev, "renv_lockfile_api")
  expect_s3_class(local_renv_extra_prod, "renv_lockfile_api")
  expect_s3_class(local_renv_blank_prod, "renv_lockfile_api")
})

pkg_extra_dev <- names(local_renv_extra_dev$data()$Packages)
pkg_blank_dev <- names(local_renv_blank_dev$data()$Packages)
pkg_extra_prod <- names(local_renv_extra_prod$data()$Packages)
pkg_blank_prod <- names(local_renv_blank_prod$data()$Packages)

test_that("extrapackage is present thanks to dev_pkg", {

  #dev

  expect_true("extrapackage" %in% pkg_extra_dev)
  expect_false("extrapackage" %in% pkg_blank_dev)
  # all blank are in extra
  expect_true(all(pkg_blank_dev %in% pkg_extra_dev))
  # there are extra not in blank
  expect_equal(setdiff(pkg_extra_dev, pkg_blank_dev), c("extrapackage"))

  # prod
  expect_true("extrapackage" %in% pkg_extra_prod)
  expect_false("extrapackage" %in% pkg_blank_prod)
  # all blank are in extra
  expect_true(all(pkg_blank_prod %in% pkg_extra_prod))
  # there are extra not in blank
  expect_equal(setdiff(pkg_extra_prod, pkg_blank_prod), c("extrapackage"))
})

# reference cannot work because it is system and R version dependent
# test_that("create_renv_for_dev works", {
#   expect_equal_to_reference(names(pkg_extra),"my_renv.test")
#   expect_equal_to_reference(names(pkg_blank),"my_renv_blank.test")
# })

# Test for extra and "_default" in interactive ----
test_that("_default works", {
  skip_if_not(interactive()) # to pass devtools::check()

  lock_includes_extra_default<- file.path(tmpdir, "for_extra_default.lock")
  expect_message({my_renv_extra_default <-
    create_renv_for_dev(
      path = dummypackage,
      install_if_missing = FALSE,
      dev_pkg = c("_default", "glue"),
      output = lock_includes_extra_default)}
    # "There is no directory named: dev, data-raw" # cli
  )

  expect_true(file.exists(lock_includes_extra_default))
  expect_true(file.exists(my_renv_extra_default))

  local_renv_extra_default <- getFromNamespace("lockfile", "renv")(my_renv_extra_default)
  expect_s3_class(local_renv_extra_default, "renv_lockfile_api")

  pkg_extra_default <- names(local_renv_extra_default$data()$Packages)
  expect_true("glue" %in% pkg_extra_default)
  # all extra are in extra_default
  # expect_true(all(pkg_extra_dev %in% pkg_extra_default))
  # devtools and fusen are in extra_default
  expect_true(all(c("devtools", "fusen") %in% pkg_extra_default))
})

# Test for "folder_to_include" in dummypackage ----
dev_dir <- file.path(dummypackage, "dev")
if (!dir.exists(dev_dir)) {dir.create(dev_dir)}
cat("library(glue)", file = file.path(dummypackage, "dev", "my_r.R"))
cat("```{r}\nlibrary(\"extrapackage\")\n```", file = file.path(dummypackage, "dev", "my_rmd.Rmd"))

test_that("folder_to_include works", {

  lock_includes_devdir <- file.path(tmpdir, "for_devdir.lock")
  expect_message({my_renv_devdir <-
    create_renv_for_dev(
      path = dummypackage,
      install_if_missing = FALSE,
      output = lock_includes_devdir,
      force = TRUE)}
    # "There is no directory named: data-raw" # cli
  )

  expect_true(file.exists(lock_includes_devdir))
  expect_true(file.exists(my_renv_devdir))

  local_renv_devdir <- getFromNamespace("lockfile", "renv")(my_renv_devdir)
  expect_s3_class(local_renv_devdir, "renv_lockfile_api")

  pkg_devdir <- names(local_renv_devdir$data()$Packages)
  # glue and extrapackage in dev/ are there
  expect_true(all(c("glue", "extrapackage") %in% pkg_devdir))
})

# Test pkg_ignore works ----
# extrapackage is in "dev/" but I want it to be ignored
test_that("DEV create_renv_(pkg_ignore) works", {

  lock_includes_ignore <- file.path(tmpdir, "for_ignore.lock")
  expect_message({my_renv_ignore <-
    create_renv_for_dev(
      path = dummypackage,
      install_if_missing = FALSE,
      output = lock_includes_ignore,
      pkg_ignore = "extrapackage",
      force = TRUE)}
  )

  expect_true(file.exists(lock_includes_ignore))
  expect_true(file.exists(my_renv_ignore))

  local_renv_ignore <- getFromNamespace("lockfile", "renv")(my_renv_ignore)
  expect_s3_class(local_renv_ignore, "renv_lockfile_api")

  pkg_ignore <- names(local_renv_ignore$data()$Packages)
  # glue and  in dev/ are there
  expect_true(all(c("glue") %in% pkg_ignore))
  expect_true(all(c("magrittr") %in% pkg_ignore))
  # extrapackage is ignored in dev/
  expect_false(all(c("extrapackage") %in% pkg_ignore))
})


# Test pkg_ignore works ----
# extrapackage is in "dev/" but I want it to be ignored
test_that("PROD create_renv_(pkg_ignore) works", {

  lock_includes_ignore <- file.path(tmpdir, "for_ignore.lock")
  expect_message({my_renv_ignore <-
    create_renv_for_prod(
      path = dummypackage,
      install_if_missing = FALSE,
      output = lock_includes_ignore,
      pkg_ignore = "extrapackage",
      force = TRUE)}
  )

  expect_true(file.exists(lock_includes_ignore))
  expect_true(file.exists(my_renv_ignore))

  local_renv_ignore <- getFromNamespace("lockfile", "renv")(my_renv_ignore)
  expect_s3_class(local_renv_ignore, "renv_lockfile_api")

  pkg_ignore <- names(local_renv_ignore$data()$Packages)
  # glue and  in dev/ are there
  expect_false(all(c("glue") %in% pkg_ignore)) # in suggests only so not present
  expect_true(all(c("magrittr") %in% pkg_ignore))
  # extrapackage is ignored in dev/
  expect_false(all(c("extrapackage") %in% pkg_ignore))
})

# Clean userspace
remove.packages("extrapackage")
unlink(extrapackage, recursive = TRUE)
unlink(tmpdir, recursive = TRUE)
unlink(dummypackage, recursive = TRUE)


tmpdir <- tempfile("dummyrenv")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
desc_file <- file.path(dummypackage, "DESCRIPTION")
desc_lines <- readLines(desc_file)
# desc_lines <- c(desc_lines,"Suggests: \n    idontexist")
desc_lines[desc_lines == "Suggests: "] <- "Suggests: \n    idontexist,\n    withr,"
writeLines(desc_lines,desc_file)

test_that("suggested package are not in renv prod", {

  out_renv_file <- tempfile(pattern = "renv.lock.prod")

  create_renv_for_prod(
    document = FALSE,# to use the DESCRIPTION file we have created
    path = dummypackage,
    install_if_missing = FALSE,
    output = out_renv_file,
    # check_if_suggests_is_installed = FALSE,
    force = TRUE)

  base <- paste(readLines(out_renv_file),collapse = " ")
  expect_false(grepl(pattern = "idontexist",x = base))
  expect_false(grepl(pattern = "withr",x = base))
  expect_true(grepl(pattern = "magrittr",x = base))

  unlink(dummypackage, recursive = TRUE)
}
)













tmpdir <- tempfile("dummyrenvdev")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
desc_file <- file.path(dummypackage, "DESCRIPTION")
desc_lines <- readLines(desc_file)
# desc_lines <- c(desc_lines,"Suggests: \n    idontexist")
desc_lines[desc_lines == "Suggests: "] <- "Suggests: \n    idontexist,\n    withr,"
writeLines(desc_lines,desc_file)

test_that("suggested package are in renv dev", {

  lock_temp<- file.path(tmpdir, "temp.lock")

  my_renv <- create_renv_for_dev(
    document = FALSE, # to use the DESCRIPTION file we have created
    path = dummypackage,
    install_if_missing = FALSE,
    output =  lock_temp,
    # check_if_suggests_is_installed = TRUE,
    force = TRUE)

  expect_true(file.exists(lock_temp))
  expect_true(file.exists(my_renv))
  local_renv <- getFromNamespace("lockfile", "renv")(my_renv)
  expect_s3_class(local_renv, "renv_lockfile_api")
  pkg_local_renv <- names(local_renv$data()$Packages)

  base <- paste(pkg_local_renv,collapse = " ")
  # expect_true(grepl(pattern = "idontexist",x = base)) #renv dont install unistalled package
  expect_true(grepl(pattern = "withr",x = base)) # ici ca coince dans le check, mais ok dans le test
  expect_true(grepl(pattern = "magrittr",x = base))

  unlink(dummypackage, recursive = TRUE)

}
)






unlink(tmpdir, recursive = TRUE)


unlink(dummypackage, recursive = TRUE)


tmpdir <- tempfile("dummyrenvsuggest")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
desc_file <- file.path(dummypackage, "DESCRIPTION")
desc_lines <- readLines(desc_file)
# desc_lines <- c(desc_lines,"Suggests: \n    idontexist")
desc_lines[desc_lines == "Suggests: "] <- "Suggests: \n    idontexist,\n    withr,"
writeLines(desc_lines,desc_file)
cat("
## The vignette
```{r}
library(glue)
library(ggplot3)
```
", file = file.path(dummypackage, "vignettes", "vignette.Rmd"))


test_that("suggested package are not in renv prod even from vignettes", {

  out_renv_file <- tempfile(pattern = "renv.lock.prod")

  create_renv_for_prod(
    document = FALSE,# to use the DESCRIPTION file we have created
    path = dummypackage,
    install_if_missing = FALSE,
    output = out_renv_file,
    # check_if_suggests_is_installed = FALSE,
    force = TRUE)

  base <- paste(readLines(out_renv_file),collapse = " ")
  expect_false(grepl(pattern = "idontexist",x = base))
  expect_false(grepl(pattern = "withr",x = base))
  expect_false(grepl(pattern = "ggplot3",x = base))
  expect_false(grepl(pattern = "glue",x = base))
  expect_true(grepl(pattern = "magrittr",x = base))

  unlink(dummypackage, recursive = TRUE)
  file.remove(out_renv_file)
}
)




unlink(tmpdir, recursive = TRUE)


unlink(dummypackage, recursive = TRUE)


tmpdir <- tempfile("dummyrenvprod")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
desc_file <- file.path(dummypackage, "DESCRIPTION")
desc_lines <- readLines(desc_file)
# desc_lines <- c(desc_lines,"Suggests: \n    idontexist")
desc_lines[desc_lines == "Suggests: "] <- "Suggests: \n    idontexist,\n    withr,"
writeLines(desc_lines,desc_file)
cat("
## The vignette
```{r}
library(glue)
library(ggplot3)
```
", file = file.path(dummypackage, "vignettes", "vignette.Rmd"))


test_that("suggested package are not in renv prod even from vignettes", {

  out_renv_file <- tempfile(pattern = "renv.lock.prod")

  create_renv_for_prod(
    document = TRUE,
    path = dummypackage,
    install_if_missing = FALSE,
    output = out_renv_file,
    # check_if_suggests_is_installed = FALSE,
    force = TRUE)

  base <- paste(readLines(out_renv_file),collapse = " ")
  expect_false(grepl(pattern = "idontexist",x = base))
  expect_false(grepl(pattern = "withr",x = base))
  expect_false(grepl(pattern = "ggplot3",x = base))
  expect_false(grepl(pattern = "glue",x = base))
  expect_true(grepl(pattern = "magrittr",x = base))

  unlink(dummypackage, recursive = TRUE)
  file.remove(out_renv_file)
}
)

#' Install missing package from DESCRIPTION
#'
#' @param path path to the DESCRIPTION file
#' @param field DESCRIPTION fields to parse, "Depends", "Imports", "Suggests" by default
#' @param ...  Arguments to be passed to [utils::install.packages()]
#' @export
#' @return Used for side effect. Installs R packages from DESCRIPTION file if missing.
#' @examples
#' \dontrun{
#' # This will install packages on your system
#' dummypackage <- system.file("dummypackage", package = "attachment")
#' # browseURL(dummypackage)
#'
#' install_from_description(path = file.path(dummypackage, "DESCRIPTION"))
#' }

install_from_description <- function(path = "DESCRIPTION", field = c("Depends", "Imports", "Suggests"), ...) {

  to_be_installed <- att_from_description(path = path, field = field)
  install_if_missing(to_be_installed = to_be_installed, ...)
}

#' install  packages if missing
#'
#' @param to_be_installed a character vector containing required packages names
#' @param ...  Arguments to be passed to [utils::install.packages()]
#'
#' @importFrom utils install.packages
#'
#' @export
#' @return Used for side effect. Install missing packages from the character vector input.
#' @examples
#' \dontrun{
#' # This will install packages on your system
#' install_if_missing(c("dplyr", "fcuk", "rusk"))
#' }
#'
install_if_missing <- function(to_be_installed, ...) {
  already_installed <- basename(try(find.package(to_be_installed, quiet = TRUE,lib.loc = .libPaths()), silent = TRUE))
  will_be_installed <- setdiff(to_be_installed, already_installed)

  if ( length(will_be_installed) == 0 ) {
    message("All required packages are installed")
    return(invisible(NULL))
  }
  message("Installation of: ", paste0(will_be_installed, collapse = ", "))

  install.packages(will_be_installed, ...)
}

dummypackage <- system.file("dummypackage", package = "attachment")
# Copy to tmpdir
pkg_dir <- tempfile(pattern = "fromdesc")
dir.create(pkg_dir)
file.copy(dummypackage, pkg_dir, recursive = TRUE)
# browseURL(file.path(pkg_dir, "dummypackage"))
# install_from_description(path = file.path(dummypackage,"DESCRIPTION"), field = c("Imports", "Depends"))

# This test need "magrittr" to be correctly tested
if (length(find.package("magrittr", quiet = TRUE)) == 0) {
  install.packages("magrittr", repos = "https://cloud.r-project.org")
}

# install_from_description ----
test_that("install_from_description works", {
  expect_message(
    install_from_description(
      path = file.path(pkg_dir, "dummypackage","DESCRIPTION"), field = c("Imports", "Depends")),
    'All required packages are installed'
  )
  lines <- readLines(file.path(pkg_dir, "dummypackage","DESCRIPTION"))
  w.magrittr <- grep("magrittr", lines)
  lines[w.magrittr] <- c("    magrittr,\n    toto,\n    tata, ")
  writeLines(lines, file.path(pkg_dir, "dummypackage","DESCRIPTION"))
  # Add dummy package name to try to install
  # test message
  # There is a warning to capture because tata and toto do not exist
  expect_message(
    capture_warning(
      install_from_description(
        path = file.path(pkg_dir, "dummypackage","DESCRIPTION"), field = c("Imports", "Depends"),
        repos = "https://cloud.r-project.org")
    ),
    "Installation of: tata, toto"
  )
  # There is a warning because tata and toto do not exist
  # This is expected warning from install.package()
  expect_warning(
    install_from_description(
      path = file.path(pkg_dir, "dummypackage","DESCRIPTION"), field = c("Imports", "Depends"),
      repos = "https://cloud.r-project.org")
  )
})

unlink(pkg_dir, recursive = TRUE)

#' Proposes values for Remotes field for DESCRIPTION file based on your installation
#'
#' @param pkg Character. Packages to test for potential non-CRAN installation
#'
#' @return
#' List of all non-CRAN packages and code to add in Remotes field in DESCRIPTION.
#' NULL otherwise.
#' @importFrom utils packageDescription
#' @export
#'
#' @examples
#' # Find from vector of packages
#' find_remotes(pkg = c("attachment", "desc", "glue"))
#'
#' # Find from Description file
#' dummypackage <- system.file("dummypackage", package = "attachment")
#' att_from_description(
#' path = file.path(dummypackage, "DESCRIPTION")) %>%
#' find_remotes()
#'
#' \dontrun{
#' # For the current package directory
#' att_from_description() %>% find_remotes()
#' }
#'
#' \donttest{
#' # For a specific package name
#' find_remotes("attachment")
#'
#' # Find remotes from all installed packages
#' find_remotes(list.dirs(.libPaths(), full.names = FALSE, recursive = FALSE))
#' }
find_remotes <- function(pkg) {

  pkgdesc <- lapply(pkg, function(x) {
    packageDescription(x)
  }) %>%
    setNames(pkg)

  # Keep only thos that are packages
  w.notpkg <- which(unlist(lapply(pkgdesc, function(x) all(is.na(x)))))
  if (length(w.notpkg) != 0) {
      message(glue::glue_collapse(names(pkgdesc)[w.notpkg], sep = ", ", last = " & "),
              ifelse(length(w.notpkg) == 1,
                     " does not seem to be a package. It is removed from exploration.",
                     " do not seem to be packages. They are removed from exploration."
              )
      )
    pkgdesc <- pkgdesc[-w.notpkg]
    if (length(pkgdesc) == 0) {return(NULL)}
  }

  extract_pkg_info(pkgdesc)
}

#' Add Remotes field to DESCRIPTION based on your local installation
#'
#' @param stop.local Logical. Whether to stop if package was installed from local source.
#' Message otherwise.
#' @param clean Logical. Whether to clean all existing remotes before run.
#' @inheritParams att_to_desc_from_is
#'
#' @return Used for side effect. Adds Remotes field in DESCRIPTION file.
#' @export
#' @examples
#' tmpdir <- tempfile(pattern = "setremotes")
#' dir.create(tmpdir)
#' file.copy(system.file("dummypackage", package = "attachment"), tmpdir,
#'  recursive = TRUE)
#' dummypackage <- file.path(tmpdir, "dummypackage")
#' # Add remotes field if there are Remotes locally
#' att_amend_desc(dummypackage) %>%
#'   set_remotes_to_desc()
#'
#' # Clean temp files after this example
#' unlink(tmpdir, recursive = TRUE)
#'
#' \dontrun{
#' # For your current package
#' att_amend_desc() %>%
#'   set_remotes_to_desc()
#' }
set_remotes_to_desc <- function(path.d = "DESCRIPTION", stop.local = FALSE,
                                clean = TRUE) {
  pkgs <- att_from_description(path.d)
  remotes <- find_remotes(pkgs)
  if (is.null(remotes)) {
    message("There are no remote packages installed on your computer to add to description")
    return(NULL)
  } else {
    internal_remotes_to_desc(remotes, path.d, stop.local, clean)
  }
}

#' (internal) Add Remotes field to DESCRIPTION based on your local installation
#'
#' @inheritParams att_from_description
#'
#' @importFrom desc description
#' @noRd
internal_remotes_to_desc <- function(remotes, path.d = "DESCRIPTION",
                                     stop.local = FALSE, clean = TRUE) {
  desc <- description$new(path.d)

  if (!isTRUE(clean)) {
    remotes_orig <- desc$get_remotes()
    names(remotes_orig) <- basename(gsub("^.*/|[.]git", "", remotes_orig))
    new_remotes <- c(
      remotes_orig,
      remotes
    )
  } else if (is.null(remotes) & isTRUE(clean)) {
    desc$clear_remotes()
    # Write Description file
    desc$write(file = path.d)
    message("Remotes were cleaned, no new remote to add.")
    return(NULL)
  } else {
    new_remotes <- remotes
  }
  are.na <- which(unlist(lapply(new_remotes, is.na)))
  pkgs_names <- names(new_remotes[are.na])

  if (length(pkgs_names) != 0) {
    plural <- ifelse(length(pkgs_names) > 1, TRUE, FALSE)
    msg <-
      glue::glue(
        "Package{ifelse(plural, 's', '')} {pkgs} {ifelse(plural, 'were', 'was')} probably installed from source locally.
             Please re-install {ifelse(plural, 'them', 'it')} from CRAN or remote repository if Remotes field is needed.",
        pkgs = glue::glue_collapse(glue("'{pkgs_names}'"), sep = ", ", last = " & ")
      )

    if (isTRUE(stop.local)) {
      stop(msg)
    } else {
      message(msg)
    }
    new_remotes <- new_remotes[-are.na]
  }

  w.unique <- !duplicated(names(new_remotes))
  new_remotes <- unlist(new_remotes)[w.unique]
  new_remotes <- new_remotes[sort(names(new_remotes))]

  if (length(new_remotes) != 0) {
    desc$set_remotes(new_remotes)
    # Write Description file
    desc$write(file = path.d)
    # Message
    plural <- ifelse(length(new_remotes) > 1, TRUE, FALSE)
    message(
      glue::glue(
        "Remotes for {pkgs} {ifelse(plural, 'were', 'was')} added to DESCRIPTION.",
        pkgs = glue::glue_collapse(glue("'{names(new_remotes)}'"), sep = ", ", last = " & ")
      )
    )
  }
}

#' Internal. Core of find_remotes separated for unit tests
#' @param pkgdesc Named list of PackageDescriptions
#' @noRd
extract_pkg_info <- function(pkgdesc) {
  is_cran <- lapply(pkgdesc, function(x) {
    (!is.null(x[["Repository"]]) && !grepl("r-universe", x[["Repository"]])) |
      (!is.null(x[["Priority"]]) && x[["Priority"]] == "base")
  }) %>% unlist()

  pkg_not_cran <- names(is_cran[!is_cran])
  # cran_pkg <- names(cran_or_not[!cran_or_not])

  if (length(pkg_not_cran) == 0) {
    return(NULL)
  } else {
    guess_repo <- lapply(pkg_not_cran, function(x) {
      desc <- pkgdesc[[x]]
      if (!is.null(desc[["Repository"]]) && grepl("r-universe", desc[["Repository"]])) {
        # For when {remotes} and/or {pak} can read universe from Remotes: field ?
        # paste0("universe::", gsub(".r-universe.dev", "", desc[["Repository"]]), "/", desc[["Package"]])
        setNames(NA, paste0("r-universe: need to set options to repos=\"", desc[["Repository"]], "\""))
      } else if (!is.null(desc$RemoteType) && desc$RemoteType == "github") {

        attachment::complete_remote_branch(
          remote = paste(desc$RemoteUsername, desc$RemoteRepo, sep = "/"),
          remoteref = desc$RemoteRef
        )

      } else if (!is.null(desc$RemoteType) && desc$RemoteType %in% c("gitlab", "bitbucket")) {

        attachment::complete_remote_branch(remote = paste0(
          desc$RemoteType,
          "::",
          paste(desc$RemoteUsername, desc$RemoteRepo, sep = "/")
        ),
        remoteref = desc$RemoteRef)

      } else if (!is.null(desc$RemoteType) && desc$RemoteType == "local" && !is.null(desc$RemoteUrl)  && is.null(desc$RemoteHost)) {

        attachment::complete_remote_branch(
          remote = paste0(desc$RemoteType, "::", desc$RemoteUrl),
          remoteref = desc$RemoteRef
        )

      } else if (!is.null(desc$RemoteType) &&
                 !(desc$RemoteType %in% c("github","gitlab","bitbucket","local")) &&
                 grepl("git",x = desc$RemoteType) ){

        attachment::complete_remote_branch(remote = paste0("git::", desc$RemoteUrl),
                               remoteref = desc$RemoteRef)

      } else if (is.null(desc$RemoteType) && isTRUE(grepl("bioconductor",x = desc$URL))) {
        biocversion <-  desc$git_branch %>%
          gsub(pattern = "RELEASE_", replacement = "") %>%
          gsub(pattern = "_", replacement = ".")
        paste0("bioc::",biocversion,"/",desc$Package)
      } else if (!is.null(desc$RemoteType) && !is.null(desc$RemoteHost)) {
        c("Maybe ?" = paste0(desc$RemoteType, "::", desc$RemoteHost, ":",
                                     paste(desc$RemoteUsername, desc$RemoteRepo, sep = "/")))
      } else {
        c("local maybe ?" = NA)
      }
    }) %>%
      setNames(pkg_not_cran)
  }
  guess_repo

}


#' Construct a Remote Branch Reference String
#'
#' This function constructs a reference string for a remote branch in a version control system like Git.
#' It returns a formatted string combining the remote name and a specified branch or reference.
#'
#' @param remote A character string representing the name of the remote repository (e.g., `"origin"`).
#' @param remoteref A character string representing the branch or reference within the remote repository.
#'        Default is `"HEAD"`.
#'
#' @return A character string. If `remoteref` is `"HEAD"`, it returns just the `remote` string. If
#'         `remoteref` is not `"HEAD"`, it returns a string in the format `"remote@remoteref"`.
#' @keywords internal
#' @export
complete_remote_branch <- function(remote, remoteref = "HEAD"){

  if (isTRUE(remoteref != "HEAD")) {
    return(
      paste(remote, remoteref, sep = "@")
    )
  } else {
    return(remote)
  }

}

# Test extract Remotes ----
test_that("find_remotes works with no error", {
  # Cannot verify directly if this works because it depends on user installation
  # From CRAN or Github. At least there should be no error.
  expect_true(length(find_remotes("desc")) == 0 |
                length(find_remotes("desc") == 1))
  # base package avoided
  expect_true(length(find_remotes("stats")) == 0)
})

# find_remotes does not fail with packages installation errors ----

test_that("find_remotes does not fail with packages installation errors", {
  tmplibpath <- tempfile("tmplibpath")
  dir.create(tmplibpath)
  dir.create(file.path(tmplibpath, "emptydir"))

  withr::with_libpaths(tmplibpath, {
    expect_warning(packageDescription("zazazaz"), "no package 'zazazaz' was found")

    expect_message(
      suppressWarnings(
        find_remotes(
          c(
            list.dirs(tmplibpath, full.names = FALSE, recursive = FALSE),
            "dontexistspkg"
          )
        )),
      regexp = "emptydir.*dontexistspkg.*They are removed from exploration."
    )
  })

  unlink(tmplibpath, recursive = TRUE)

  expect_warning(
  expect_message(
    res <- find_remotes("dontexistspkg"),
    regexp = "dontexistspkg does not seem to be a package. It is removed from exploration."
  )
  )
  expect_null(res)
})

# Test core of find_remotes ----
test_that("extract_pkg_info extracts code", {

  # R-universe from tar.gz
  # One day, maybe we can specify to use r-universe by default
  fake_desc_universe <- list(
    list(
      RemoteType = "url",
      Repository = "https://thinkr-open.r-universe.dev",
      RemoteUrl = "https://thinkr-open.r-universe.dev/src/contrib/fusen_0.4.0.9000.tar.gz",
      RemoteRef = "HEAD",
      RemoteSha = "bfd6867bdc00976206479d5e784cdc9f057e0991",
      Package = "fusen"
    )
  ) %>% setNames("fusen")
  expect_equal(
    extract_pkg_info(fake_desc_universe)[["fusen"]],
    c("r-universe: need to set options to repos=\"https://thinkr-open.r-universe.dev\"" = NA))
  #"universe::thinkr-open/fusen"


  # R-universe from repos changed
  fake_desc_universe2 <- list(
    list(
      Repository = "https://thinkr-open.r-universe.dev",
      RemoteUrl = "https://github.com/ThinkR-open/fusen",
      RemoteRef = "HEAD",
      RemoteSha = "bfd6867bdc00976206479d5e784cdc9f057e0991",
      Version = "0.4.0.9000",
      Package = "fusen"
    )
  ) %>% setNames("fusen")
  expect_equal(
    extract_pkg_info(fake_desc_universe2)[["fusen"]],
    c("r-universe: need to set options to repos=\"https://thinkr-open.r-universe.dev\"" = NA))
  #"universe::thinkr-open/fusen"

  # Github
  fake_desc_github <- list(
    list(
      RemoteType = "github",
      RemoteHost = "api.github.com",
      RemoteRepo = "fusen",
      RemoteUsername = "ThinkR-open"
    )
  ) %>% setNames("fusen")
  expect_equal(extract_pkg_info(fake_desc_github)[["fusen"]], "ThinkR-open/fusen")

  # Github with branch
  fake_desc_github_with_branch <- list(
    list(
      RemoteType = "github",
      RemoteHost = "api.github.com",
      RemoteRepo = "fusen",
      RemoteUsername = "ThinkR-open",
      RemoteRef = "branchdev"
    )
  ) %>% setNames("fusen")
  expect_equal(extract_pkg_info(fake_desc_github_with_branch)[["fusen"]], "ThinkR-open/fusen@branchdev")

  # GitLab
  fake_desc_gitlab <- list(
    list(
      RemoteType = "gitlab",
      RemoteHost = "gitlab.com",
      RemoteRepo = "fakepkg",
      RemoteUsername = "statnmap"
    )
  ) %>% setNames("fakepkg")
  expect_equal(extract_pkg_info(fake_desc_gitlab)[["fakepkg"]], "gitlab::statnmap/fakepkg")


  # GitLab with branch
  fake_desc_gitlab_with_branch <- list(
    list(
      RemoteType = "gitlab",
      RemoteHost = "gitlab.com",
      RemoteRepo = "fakepkg",
      RemoteRef = "fakebranch",
      RemoteUsername = "statnmap"
    )
  ) %>% setNames("fakepkg")
  expect_equal(extract_pkg_info(fake_desc_gitlab_with_branch)[["fakepkg"]], "gitlab::statnmap/fakepkg@fakebranch")


  # Git
  fake_desc_git <- list(
    list(
      RemoteType = "xgit",
      RemoteUrl = "https://github.com/fakepkggit.git"
    )
  ) %>% setNames("fakepkggit")
  expect_equal(extract_pkg_info(fake_desc_git)[["fakepkggit"]], "git::https://github.com/fakepkggit.git")

  fake_desc_git2r <- list(
    list(
      RemoteType = "git2r",
      RemoteUrl = "https://MyForge.com/fakepkggit2r",
      RemoteRepo = NULL,
      RemoteUsername = NULL
    )
  ) %>% setNames("fakepkggit2r")
  expect_equal(extract_pkg_info(fake_desc_git2r)[["fakepkggit2r"]], "git::https://MyForge.com/fakepkggit2r")

  # Git with branch
  fake_desc_git2r_with_branch <- list(
    list(
      RemoteType = "git2r",
      RemoteUrl = "https://MyForge.com/fakepkggit2r",
      RemoteRepo = NULL,
      RemoteRef = "fakepkgbranch",
      RemoteUsername = NULL
    )
  ) %>% setNames("fakepkggit2r")
  expect_equal(extract_pkg_info(fake_desc_git2r_with_branch)[["fakepkggit2r"]], "git::https://MyForge.com/fakepkggit2r@fakepkgbranch")


  # Bioconductor
  fake_desc_bioc <- list(
    list(
      URL = "https://bioconductor.org/packages/fakepkgbioc",
      git_branch = "RELEASE_3_3",
      Package = "fakepkgbioc"
    )
  ) %>% setNames("fakepkgbioc")
  expect_equal(extract_pkg_info(fake_desc_bioc)[["fakepkgbioc"]], "bioc::3.3/fakepkgbioc")

  # Other installations
  # local package path
  fake_desc_local <- list(
    list(
      RemoteType = "local",
      RemoteUrl = "/path/fakelocal",
      RemoteHost = NULL,
      RemoteRepo = NULL,
      RemoteUsername = NULL
    )
  ) %>% setNames("fakelocal")

  expect_equal(extract_pkg_info(fake_desc_local)[["fakelocal"]], "local::/path/fakelocal")

  # Other specific unknown remotetype
  fake_desc_unknown <- list(
    list(
      RemoteType = "svn",
      RemoteHost = "host",
      RemoteRepo = "repo",
      RemoteUsername = "username"
    )
  ) %>% setNames("fakeunknown")

  expect_equal(extract_pkg_info(fake_desc_unknown)[["fakeunknown"]], c("Maybe ?" = "svn::host:username/repo"))

  # Other installations
  fake_desc_other <- list(
    list(
      RemoteType = NULL,
      RemoteHost = NULL,
      RemoteRepo = NULL,
      RemoteUsername = NULL
    )
  ) %>% setNames("fakenull")

  expect_true(is.na(extract_pkg_info(fake_desc_other)[["fakenull"]]))
  expect_equal(names(extract_pkg_info(fake_desc_other)[["fakenull"]]), "local maybe ?")

  # Test internal_remotes_to_desc ----
  tmpdir <- tempfile(pattern = "pkginternalremotes")
  dir.create(tmpdir)
  file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
  dummypackage <- file.path(tmpdir, "dummypackage")

  path.d <- file.path(dummypackage, "DESCRIPTION")
  cat("Remotes:\n    ThinkR-open/attachment\n", append = TRUE,
      file = path.d)

  remotes <- c(
    extract_pkg_info(fake_desc_github),
    extract_pkg_info(fake_desc_gitlab),
    extract_pkg_info(fake_desc_other),
    extract_pkg_info(fake_desc_git),
    extract_pkg_info(fake_desc_git2r),
    extract_pkg_info(fake_desc_bioc),
    extract_pkg_info(fake_desc_local)
  )

  expect_error(
    internal_remotes_to_desc(remotes, path.d = path.d, stop.local = TRUE),
    "'fakenull' was probably installed from source locally"
  )

  packages_names_sort <- sort(c('attachment', 'fusen', 'fakepkg', 'fakepkggit',
                                'fakepkggit2r', 'fakepkgbioc', 'fakelocal'))
  expect_message(
    expect_message(
      internal_remotes_to_desc(remotes, path.d = path.d,
                               stop.local = FALSE, clean = FALSE),
      "installed from source locally"
    ),
    paste("Remotes for",
          glue::glue_collapse(glue("'{packages_names_sort}'"), sep = ", ", last = " & "),
          "were added to DESCRIPTION.")
  )

  new_desc <- readLines(path.d)

  w.remotes <- grep('Remotes:', new_desc)
  expect_length(w.remotes, 1)
  expect_equal(
    gsub(",", "",
         new_desc[w.remotes + which(packages_names_sort == "attachment")]
    ),
    "    ThinkR-open/attachment")
  expect_equal(
    gsub(",", "",
         new_desc[w.remotes + which(packages_names_sort == "fusen")]
    ),
    "    ThinkR-open/fusen")
  expect_equal(
    gsub(",", "",
         new_desc[w.remotes + which(packages_names_sort == "fakepkg")]
    ),
    "    gitlab::statnmap/fakepkg")
  expect_equal(
    gsub(",", "",
         new_desc[w.remotes + which(packages_names_sort == "fakepkggit")]
    ),
    "    git::https://github.com/fakepkggit.git")
  expect_equal(
    gsub(",", "",
         new_desc[w.remotes + which(packages_names_sort == "fakepkgbioc")]
    ),
    "    bioc::3.3/fakepkgbioc")
  expect_equal(
    gsub(",", "",
         new_desc[w.remotes + which(packages_names_sort == "fakelocal")]
    ),
    "    local::/path/fakelocal")
  expect_equal(
    gsub(",", "",
         new_desc[w.remotes + which(packages_names_sort == "fakepkggit2r")]
    ),
    "    git::https://MyForge.com/fakepkggit2r")


  # Test clean before
  expect_message(
    internal_remotes_to_desc(remotes["fusen"], path.d = path.d,
                             stop.local = FALSE, clean = TRUE),
    regexp = "'fusen' was added to DESCRIPTION")

  new_desc <- readLines(path.d)
  w.remotes <- grep('Remotes:', new_desc)
  expect_length(w.remotes, 1)
  expect_equal(new_desc[w.remotes + 1], "    ThinkR-open/fusen")
  expect_false(any(grepl("attachment", new_desc)))
  expect_false(any(grepl("fakepkg", new_desc)))
  expect_false(any(grepl("fakepkggit", new_desc)))
  expect_false(any(grepl("fakepkggit2r", new_desc)))
  expect_false(any(grepl("fakelocal", new_desc)))
  expect_false(any(grepl("fakepkgbioc", new_desc)))

  # Test what happens if null and clean FALSE
  expect_message(
    internal_remotes_to_desc(NULL, path.d = path.d,
                             stop.local = FALSE, clean = FALSE),
    regexp = "'fusen' was added to DESCRIPTION")

  new_desc_null <- readLines(path.d)
  expect_equal(new_desc_null, new_desc)

  # Test what happens if null and clean FALSE
  expect_message(
    internal_remotes_to_desc(NULL, path.d = path.d,
                             stop.local = FALSE, clean = TRUE),
    regexp = "Remotes were cleaned, no new remote to add."
  )

  new_desc_null <- readLines(path.d)
  expect_false(any(grepl("Remotes", new_desc_null)))

  unlink(tmpdir, recursive = TRUE)
})

# test set_remotes_to_desc ----
tmpdir <- tempfile(pattern = "pkgset")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

test_that("set_remotes_to_desc return nothing if local installs", {

  skip_on_cran()
  # We do not know whether some packages are installed manually from source on CRAN

  pkgs <- att_amend_desc(dummypackage) %>%
    att_from_description()
  remotes <- find_remotes(pkgs)

  desc_file <- file.path(dummypackage, "DESCRIPTION")

  if (is.null(remotes)) {
    # Do not test if some are compiled locally
    expect_message(
      set_remotes_to_desc(desc_file),
      "no remote packages installed"
    )
  } else {
    pkgnames <- glue::glue_collapse(glue::glue("'{names(remotes)}'"), sep = ", ", last = " & ")
    nona <- unlist(lapply(remotes, is.na))
    expect_message(
      set_remotes_to_desc(desc_file),
      paste("Remotes for", pkgnames[nona])
    )
  }

  # Add attachment in DESCRIPTION as should be local installed
  desc_lines <- readLines(desc_file)
  desc_lines[desc_lines == "Suggests: "] <- "Suggests: \n    attachment,"
  writeLines(desc_lines, con = desc_file)

  expect_message(
    set_remotes_to_desc(desc_file),
    "Package 'attachment' was probably installed from source locally"
  )

  # Add pkglocal in DESCRIPTION as should be local installed from a path
  desc_lines <- readLines(desc_file)
  desc_lines <- c(desc_lines,"Remotes: \n    local::/path/pkglocal")
  writeLines(desc_lines, con = desc_file)

  expect_message(
    set_remotes_to_desc(desc_file),
    "Package 'attachment' was probably installed from source locally"
  )

})
unlink(tmpdir, recursive = TRUE)

# USE ONE OR THE OTHER and its gone.
touch Event-00001890-ITSProcessCommand_FindPoolAssignmentByPoolId.data
# rm Event-00001890-ITSProcessCommand_FindPoolAssignmentByPoolId.data

echo "Total Files"
ls | grep -c "$"

echo "data files"
ls | grep -c "data"

---
title: "a-Help during package development"
author: "Sébastien Rochette"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a-Help during package development}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<script type="text/javascript">
$(document).ready(function () {
  window.initializeCodeFolding("show" === "show");
});
</script>

```{r setuprmd, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Load package {attachment}
```{r}
library(attachment)
```

## Use "dev/dev_history.R"

When building your package, create a file called "dev_history.R" in a "dev/" directory. You will store all "manual" calls to `devtools::xxx` and `usethis::xxx` in this script.  
Its first line should be :

```r
usethis::use_build_ignore("dev")
```

You can then call {attachment} in this file to help you build your description file.

## Fill your DESCRIPTION file

What you really want is to fill and update your description file along with the modifications of your documentation. Indeed, only this function will really be called in your "dev/dev_history.R".  
Run `attachment::att_amend_desc()` each time before `devtools::check()`, this will save you some warnings and errors !

```{r, eval=FALSE}
att_amend_desc()
```

## Change default parameters

We recommend to store the {attachment} command in a "dev/dev_history.R" file in your package, so that you can update the parameters if not using the default ones.

### Use the configuration file

If you run `att_amend_desc()` directly in the console, it will use the last set of parameters run the last time you updated the configuration file.  
Hence, you can change the command in your "dev/dev_history.R" with `update.config = TRUE`, so that next time, you can run `attachment::att_amend_desc()` directly in the console using these updated parameters.

### Ignore some of the dependencies automatically detected

```{r, eval=FALSE}
att_amend_desc(pkg_ignore = c("fakepackage.to_ignore", "other.package"), update.config = TRUE)
```

### Add extra `Suggests` dependencies

```{r, eval=FALSE}
att_amend_desc(extra.suggests = c("suggested.package.not.detected"), update.config = TRUE)
```

### Move detected dependencies from `Imports` to `Suggests`

```{r, eval=FALSE}
att_amend_desc(pkg_ignore = c("package.to.move"), extra.suggests = c("package.to.move"), update.config = TRUE)
```

Next time, you can run `attachment::att_amend_desc()` without any parameters, it will use the config file.

## A package was not detected by `att_amend_desc`

### I have a Rmarkdown file / Quarto file in my "inst/" directory

In your package, you can have a wrapper around `knitr::knit()` with a specific template that you store in "inst/". 
In this case, {attachment} will not detect if there are important dependencies that you used inside your notebook file that are necessary for your package to work properly. Indeed, we can not anticipate if dependencies listed in files in "inst/" need to be in "Imports", "Suggests", or not listed, depending on your use of these files.

For instance, I have a notebook file "inst/my_template.Rmd" as follows, with {bookdown} being used only there in my package:

````
---
title: "My Super template"
author: "John Doe"
date: "`r Sys.Date()`"
output: bookdown::html_document2
---

# First title

My text here.
````

_Suggests_ - If I want to present this template in an example only, {bookdown} should be listed in "Suggests". Then I can run this once:

```{r, eval=FALSE}
att_amend_desc(extra.suggests = c("bookdown"), update.config = TRUE)
```

_Imports_ - If I want to knit it inside a major function of my package, it is recommended to declare the dependency in the 'roxygen' documentation of this function. 

```{r, eval=FALSE}
#' Knit my internal template
#'
#' @importFrom bookdown html_document2
#' @export
#'
my_knit <- function() {
  rmarkdown::render(system.file("my_template.Rmd", package = "my.package"))
}
```

Then I can directly run `att_amend_desc()`.

In the _Imports_ case, if for any reason I decide to delete this `my_knit()` function, then the {bookdown} dependency won't be needed anymore, and {attachment} will automatically remove it with the next `att_amend_desc()`

### I have an R script in my "inst/" directory

If the file under "inst/" is a plain R script rather than a notebook, for instance a deployment entry point "inst/main.R" that calls `pkg::fun()`, you can have {attachment} scan it by adding its directory to `dir.r`:

```{r, eval=FALSE}
att_amend_desc(dir.r = c("R", "inst"))
```

Packages detected this way are added to "Imports". A version already pinned by hand in DESCRIPTION for such a package is preserved.


## Example on a fake package

If you are running this inside a Rmd like here, you may need parameter `inside_rmd = TRUE`.  

```{r}
# Copy package in a temporary directory
tmpdir <- tempfile(pattern = "insidermd")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir, recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)
att_amend_desc(path = dummypackage, inside_rmd = TRUE, update.config = TRUE)

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)
```

## Propose content for "Remotes" field

`set_remotes_to_desc()` adds packages that were installed from other source than CRAN to `Remotes:` field in DESCRIPTION.  

For instance:

- For GitHub: `Remotes: thinkr-open/attachment`
- For GitLab: `Remotes: gitlab::jimhester/covr`
- For Git: `Remotes: git::https://theurl/package_git.git`
- For local package: `Remotes: local::c:\mylocalpackage` or `Remotes: local::subdir/mylocalpackage`
- For Bioconductor: `Remotes: bioc::3.3/package_bioc`
- For r-universe: Show a message to inform how to use r-universe with `options`, like `r-universe: need to set options to repos="https://thinkr-open.r-universe.dev"`


You may want to run it after `att_amend_desc()`.

```{r eval=FALSE}
att_amend_desc(dummypackage) %>%
  set_remotes_to_desc()
```

If you only want to find if packages were installed from other source than CRAN, without amending DESCRIPTION, you can use `find_remotes()`.

You can use it on a vector of packages names 
```{r eval=TRUE}
find_remotes(pkg = c("attachment", "desc", "glue"))
```

You may also want to combine it to `att_from_description()`
```{r, eval=FALSE}
att_from_description() %>%
  find_remotes()
```

If you want to get the complete list of packages installed on your computer with non-CRAN repositories:
```{r, eval=FALSE}
find_remotes(list.dirs(.libPaths(), full.names = FALSE, recursive = FALSE))
```

You can test it if you install {fusen} from GitHub or r-universe:

```{r, eval=FALSE}
# From GitHub
remotes::install_github("ThinkR-open/fusen",
                        quiet = TRUE, upgrade = "never")
attachment::find_remotes("fusen")
#> $fusen
#> [1] "ThinkR-open/fusen"

# From r-universe as default repos
install.packages("fusen", repos = "https://thinkr-open.r-universe.dev")
attachment::find_remotes("fusen")
#> r-universe: need to set options to repos="https://thinkr-open.r-universe.dev" 
```


## Create a file for package installation

Once your package is finished. Well, is a package ever finished ? Let's say, once you want to release a version of your package, you may want to deliver the list of dependencies your users will have to install. A little script like `install.packages(c(...all dep...))` would be so nice :

```{r, eval=FALSE}
create_dependencies_file()
```

This file will be placed in `inst/dependencies.R` and contains :

```{r, eval=FALSE}
# No Remotes ----
# remotes::install_github("ThinkR-open/fcuk")
# Attachments ----
to_install <- c("covr", "desc", "devtools", "glue", "knitr", "magrittr", "rmarkdown", "stats", "stringr", "testthat", "utils")
for (i in to_install) {
  message(paste("looking for ", i))
  if (!requireNamespace(i)) {
    message(paste("     installing", i))
    install.packages(i)
  }
}
```

## Other possibilities

Of course, you can also use {attachment} out of a package to list all package dependencies of R scripts using `att_from_rscripts()`, Rmd files using `att_from_rmds()` or examples from R scripts using `att_from_examples()`.

```{r, eval=TRUE}
dummypackage <- system.file("dummypackage", package = "attachment")

att_from_rscripts(path = file.path(dummypackage, "R"))
att_from_rmds(path = file.path(dummypackage, "vignettes"), inside_rmd = TRUE)
att_from_examples(dir.r = file.path(dummypackage, "R"))
```

---
title: "Bookdown, quarto and other scripts dependencies"
author: "Sébastien Rochette"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{b-bookdown and other scripts dependencies}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<script type="text/javascript">
$(document).ready(function () {
  window.initializeCodeFolding("show" === "show");
});
</script>

```{r setuprmd, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Load package {attachment}
```{r}
library(attachment)
```

## Use "dev/dev_history.R"

When building your package, create a file called "dev_history.R" in a "dev/" directory. You will store all "manual" calls to `devtools::xxx` and `usethis::xxx` in this script.  
Its first line should be :

```r
usethis::use_build_ignore("dev")
```

You can then call {attachment} in this file to help you build your description file.

## Use {attachment} with {bookdown} / {pagedown} / {quarto} dependencies

Dependencies of your Bookdown can be automatically installed if you use a "DESCRIPTION" file.  
Indeed, you can use this procedure to create the DESCRIPTION file for local installation or for Continuous Integration with GitLab Pages or GitHub Actions.
See for instance, the GitLab CI workflows in {gitlabr}: https://github.com/ThinkR-open/gitlabr/tree/main/inst/gitlab-ci

1. Create a "DESCRIPTION" file
```{r, eval=FALSE}
usethis::use_description()
```


2. Fill the "DESCRIPTION" file using `attachment::att_to_desc_from_is()`.
_Note that to include it directly in CI (as proposed in {gitlabr} templates), you may need to set `att_to_desc_from_is(must.exist = FALSE)`._   
_Note that `att_from_rmds()` also works for ".qmd" documents. You can use `att_from_qmds()` if you want, although for now there is no difference and it will also parse ".Rmd" documents._


```{r, eval=FALSE}
# bookdown Imports are in Rmds
imports <- c("bookdown", attachment::att_from_rmds("."))

attachment::att_to_desc_from_is(
  path.d = "DESCRIPTION",
  imports = imports, suggests = NULL, 
  must.exist = FALSE
)
```

3. Install dependencies from the DESCRIPTION file

```{r, eval=FALSE}
# install.packages("remotes")
remotes::install_deps()
```

4. Build your {bookdown} / {pagedown} / {quarto}

### Propose content for "Remotes" field

An interest of using DESCRIPTION to list your bookdown dependencies is to use packages from other sources than CRAN and list them in the 'Remotes' field.  
Here comes `set_remotes_to_desc()`, which adds packages that were installed from other source than CRAN to `Remotes:` field in DESCRIPTION.  

You can run it after `att_to_desc_from_is()`.

```{r eval=FALSE}
attachment::att_to_desc_from_is(
  path.d = "DESCRIPTION",
  imports = imports, suggests = NULL, 
  must.exist = FALSE
) %>%
  set_remotes_to_desc()
```


## Get all packages listed in "namespace"
You can get the list of packages in your package with `att_from_namespace()`

```{r, eval=FALSE}
att_from_namespace()
```

## Get all packages added using `pkg::function` or library/require
This reads all files in directories of R scripts (default to `R` directory of a package)

```{r, eval=FALSE}
att_from_rscripts()
```

Called in examples from R scripts: 

```{r, eval=FALSE}
att_from_examples()
```

## Get all packages called in your Rmd
If you have vignette, you may want to list extra libraries, not listed in your "Depends" list. This function applies to any Rmd file, of course.

```{r, eval=FALSE}
att_from_rmds()
```


## Get all packages called in your Rmd and show them in this same Rmd

Of course, you can also use {attachment} out of a package to list all package dependencies of R scripts using `att_from_rscripts()` or Rmd files using `att_from_rmds()`.  
If you want to run `att_from_rmds()` inside a Rmd as for this vignette, you will need to set `att_from_rmds(inside_rmd = TRUE)`

```{r, eval=TRUE}
dummypackage <- system.file("dummypackage", package = "attachment")

att_from_rmds(path = file.path(dummypackage, "vignettes"), inside_rmd = TRUE)
```

---
title: "Vignette Title"
author: "Vignette Author"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Vignette Title}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Vignettes are long form documentation commonly included in packages. Because they are part of the distribution of the package, they need to be as compact as possible. The `html_vignette` output type provides a custom style sheet (and tweaks some options) to ensure that the resulting html is as small as possible. The `html_vignette` format:

- Never uses retina figures
- Has a smaller default figure size
- Uses a custom CSS stylesheet instead of the default Twitter Bootstrap style

## Vignette Info

Note the various macros within the `vignette` section of the metadata block above. These are required in order to instruct R how to build the vignette. Note that you should change the `title` field and the `\VignetteIndexEntry` to match the title of your vignette.

## Styles

The `html_vignette` template includes a basic CSS theme. To override this theme you can specify your own CSS in the document metadata as follows:

    output: 
      rmarkdown::html_vignette:
        css: mystyles.css

## Figures

The figure sizes have been customised so that you can easily put two images side-by-side. 

```{r, fig.show='hold'}
plot(1:10)
plot(10:1)
```

You can enable figure captions by `fig_caption: yes` in YAML:

    output:
      rmarkdown::html_vignette:
        fig_caption: yes

Then you can use the chunk option `fig.cap = "Your figure caption."` in **knitr**.

## More Examples

You can write math expressions, e.g. $Y = X\beta + \epsilon$, footnotes^[A footnote here.], and tables, e.g. using `knitr::kable()`.

```{r, echo=FALSE, results='asis'}
knitr::kable(head(mtcars, 10))
```

Also a quote using `>`:

> "He who gives up [code] safety for [code] speed deserves neither."
([via](https://twitter.com/hadleywickham/status/504368538874703872))

```{r}
library(glue)
glue("This is an example for library call")
```

---
title: "Create dependencies file"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Create dependencies file}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r}
library(attachment)
```





  


# Write instructions to install all dependencies from a "DESCRIPTION" file

`create_dependencies_file()` creates "inst/dependencies.R" file with the instructions to install all dependencies listed in your "DESCRIPTION".
This accounts for the "Remotes" field and write instructions accordingly.   

Use `create_dependencies_file(to = NULL)` to only retrieve the output as a list of character and not save a "inst/dependencies.R" file in your project. 
This can be used in a `Readme.Rmd` file for instance, to get the full list of each dependency to install and how, from any "DESCRIPTION" file.


```{r examples-create_dependencies_file}
#| eval: yes

# Create a fake package
tmpdir <- tempfile(pattern = "depsfile")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
          recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

# Create the dependencies commands but no file
create_dependencies_file(
  path = file.path(dummypackage,"DESCRIPTION"),
  to = NULL,
  open_file = FALSE)

# Create the dependencies files in the package
create_dependencies_file(
  path = file.path(dummypackage,"DESCRIPTION"),
  to = file.path(dummypackage, "inst/dependencies.R"),
  open_file = FALSE)
list.files(file.path(dummypackage, "inst"))
# browseURL(dummypackage)

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)
```

---
title: "Create dependencies file"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Create dependencies file}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r}
library(attachment)
```

<!-- WARNING - This vignette is generated by {fusen} from dev/flat_create_dependencies_file.Rmd: do not edit by hand -->




  


# Write instructions to install all dependencies from a "DESCRIPTION" file

`create_dependencies_file()` creates "inst/dependencies.R" file with the instructions to install all dependencies listed in your "DESCRIPTION".
This accounts for the "Remotes" field and write instructions accordingly.   

Use `create_dependencies_file(to = NULL)` to only retrieve the output as a list of character and not save a "inst/dependencies.R" file in your project. 
This can be used in a `Readme.Rmd` file for instance, to get the full list of each dependency to install and how, from any "DESCRIPTION" file.


```{r examples-create_dependencies_file}
#| eval: yes

# Create a fake package
tmpdir <- tempfile(pattern = "depsfile")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
          recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

# Create the dependencies commands but no file
create_dependencies_file(
  path = file.path(dummypackage,"DESCRIPTION"),
  to = NULL,
  open_file = FALSE)

# Create the dependencies files in the package
create_dependencies_file(
  path = file.path(dummypackage,"DESCRIPTION"),
  to = file.path(dummypackage, "inst/dependencies.R"),
  open_file = FALSE)
list.files(file.path(dummypackage, "inst"))
# browseURL(dummypackage)

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)
```

---
title: "Use {renv} with developers tools"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{use_renv}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

`r lifecycle::badge("experimental")`

```{r setup}
library(attachment)
```

`create_renv_for_dev()` and `create_renv_for_prod()` functions create `renv.lock` files based on development projects.

# Create reproducible environments for your R projects with {renv}

Tool to create and maintain "renv.lock" files.
The idea is to have 2 distinct files, one for development and the other for deployment.
Indeed, although package like {attachment}, {fusen} or {pkgload} must be installed to develop, they are not necessary in your project, package or Shiny application.

Hence, create and update your "renv.lock" file in the development project with everything needed to work in the same conditions between collaborators and allow Continuous Integration to work properly.  
This currently includes: 

```{r, echo=FALSE}
attachment:::extra_dev_pkg
```

And thus, run `create_renv_for_dev()` before sending your commit to your remote git server. Use `_default` (with underscore), to use the default list.

```{r, eval=FALSE}
create_renv_for_dev() # with all default above
create_renv_for_dev(dev_pkg = "attachment") # with {attachment} only
create_renv_for_dev(dev_pkg = c("_default", "DT")) # for all default and {DT}
```

Later on, if you want to create a R project that can use your package developed with {renv}, run `create_renv_for_prod()`.  
Indeed, your users only need to install packages listed in your "DESCRIPTION" file, with the same packages versions you used during development.

```{r, eval=FALSE}
create_renv_for_prod(output = "renv.lock.prod")
```

---
title: "Untitled"
author: "Vincent"
date: "11 août 2018"
output: bookdown::html_document2
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

## R Markdown

This is an R Markdown document. Markdown is a simple formatting syntax for authoring HTML, PDF, and MS Word documents. For more details on using R Markdown see <http://rmarkdown.rstudio.com>.

When you click the **Knit** button a document will be generated that includes both content as well as the output of any embedded R code chunks within the document. You can embed an R code chunk like this:

```{r cars}
summary(cars)
```

## Including Plots

You can also embed plots, for example:

```{r pressure, echo=FALSE}
plot(pressure)
```

Note that the `echo = FALSE` parameter was added to the code chunk to prevent printing of the R code that generated the plot.

require(dontfind)

```{r}
library(find.me)
require(findme1)
require(   findme2   )
library(findme3)
library(   findme4   )
requireNamespace(findme5)
requireNamespace(     findme6      )
```


```{r}
require("findme1a")
require(   "findme2a"   )
library("findme3a")
library(   "findme4a"   )
requireNamespace("findme5a")
requireNamespace(     "findme6a"      )

# require(dontfindme1)
# require(   dontfindme2   )
# library(dontfindme3)
# library(   dontfindme4   )
# requireNamespace(dontfindme5)
# requireNamespace(     dontfindme6      )
#
#
# require("dontfindme1a")
# require(   "dontfindme2a"   )
# library("dontfindme3a")
# library(   "dontfindme4a"   )
# requireNamespace("dontfindme5a")
# requireNamespace(     "dontfindme6a"      )


1+1# require(dontfindme1)
1+1# require(   dontfindme2   )
1+1# library(dontfindme3)
1+1# library(   dontfindme4   )
1+1# requireNamespace(dontfindme5)
1+1# requireNamespace(     dontfindme6      )
1+1#
1+1#
1+1# require("dontfindme1a")
1+1# require(   "dontfindme2a"   )
1+1# library("dontfindme3a")
1+1# library(   "dontfindme4a"   )
1+1# requireNamespace("dontfindme5a")
1+1# requireNamespace(     "dontfindme6a"      )

```

## Inline code 

`r find.inline::fun()`

---
title: "A Rmd that runs the function"
author: "Sébastien Rochette"
date: "03/06/2020"
output: html_document
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

### To list information
Of course, you can also use {attachment} out of a package to list all package dependencies of R scripts using `att_from_rscripts()` or Rmd files using `att_from_rmds()`.  
If you are running this inside a Rmd, you may need parameter `inside_rmd = TRUE`.  

```{r, eval=TRUE}
dummypackage <- system.file("dummypackage", package = "attachment")

att_from_rscripts(path = dummypackage)
att_from_rmds(path = file.path(dummypackage, "vignettes"), inside_rmd = TRUE)
```

---
title: "quarto demo"
date: "5/22/2021"
format: 
  html:
    fig-width: 8
    fig-height: 4
    code-fold: true
---

## Air Quality

@fig-airquality further explores the impact of temperature on ozone level.

```{r}
#| label: fig-airquality
#| fig-cap: Temperature and ozone level.
#| warning: false

plot(pressure)
utils::packageVersion("knitr")
library("findme.quarto")
# library("dontfindme.quarto")

```

## Python should not be parsed

```{python}
#| label: fig-polar
#| fig-cap: "A line plot on a polar axis"

import numpy as np
import matplotlib.pyplot as plt

r = np.arange(0, 2, 0.01)
theta = 2 * np.pi * r
fig, ax = plt.subplots(
  subplot_kw = {'projection': 'polar'} 
)
ax.plot(theta, r)
ax.set_rticks([0.5, 1, 1.5, 2])
ax.grid(True)
plt.show()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_remotes.R
\name{complete_remote_branch}
\alias{complete_remote_branch}
\title{Construct a Remote Branch Reference String}
\usage{
complete_remote_branch(remote, remoteref = "HEAD")
}
\arguments{
\item{remote}{A character string representing the name of the remote repository (e.g., \code{"origin"}).}

\item{remoteref}{A character string representing the branch or reference within the remote repository.
Default is \code{"HEAD"}.}
}
\value{
A character string. If \code{remoteref} is \code{"HEAD"}, it returns just the \code{remote} string. If
\code{remoteref} is not \code{"HEAD"}, it returns a string in the format \code{"remote@remoteref"}.
}
\description{
This function constructs a reference string for a remote branch in a version control system like Git.
It returns a formatted string combining the remote name and a specified branch or reference.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install_from_description.R
\name{install_from_description}
\alias{install_from_description}
\title{Install missing package from DESCRIPTION}
\usage{
install_from_description(
  path = "DESCRIPTION",
  field = c("Depends", "Imports", "Suggests"),
  ...
)
}
\arguments{
\item{path}{path to the DESCRIPTION file}

\item{field}{DESCRIPTION fields to parse, "Depends", "Imports", "Suggests" by default}

\item{...}{Arguments to be passed to \code{\link[utils:install.packages]{utils::install.packages()}}}
}
\value{
Used for side effect. Installs R packages from DESCRIPTION file if missing.
}
\description{
Install missing package from DESCRIPTION
}
\examples{
\dontrun{
# This will install packages on your system
dummypackage <- system.file("dummypackage", package = "attachment")
# browseURL(dummypackage)

install_from_description(path = file.path(dummypackage, "DESCRIPTION"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-pipe.R
\name{\%>\%}
\alias{\%>\%}
\title{Pipe operator}
\usage{
lhs \%>\% rhs
}
\arguments{
\item{lhs}{A value or the magrittr placeholder.}

\item{rhs}{A function call using the magrittr semantics.}
}
\value{
The result of calling \code{rhs(lhs)}.
}
\description{
See \verb{magrittr::[\\\%>\\\%][magrittr::pipe]} for details.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/att_from_rscripts.R
\name{att_from_rscripts}
\alias{att_from_rscripts}
\title{Look for functions called with \code{::} and library/requires in folder of scripts}
\usage{
att_from_rscripts(
  path = "R",
  pattern = "*.[.](r|R)$",
  recursive = TRUE,
  folder_to_exclude = "renv",
  encoding = getOption("encoding")
)
}
\arguments{
\item{path}{directory with R scripts inside or vector of R scripts}

\item{pattern}{pattern to detect R script files}

\item{recursive}{logical. Should the listing recurse into directories?}

\item{folder_to_exclude}{Folder to exclude during scan to detect packages. 'renv' by default.}

\item{encoding}{Encoding passed to \code{\link[=readLines]{readLines()}} when reading \code{path}. Defaults
to \code{getOption("encoding")} so the system locale is respected (important on
Windows where scripts are often Latin-1 / Windows-1252).}
}
\value{
vector of character of packages names found in the R script
}
\description{
Look for functions called with \code{::} and library/requires in folder of scripts
}
\examples{
dummypackage <- system.file("dummypackage",package = "attachment")
# browseURL(dummypackage)

att_from_rscripts(path = file.path(dummypackage, "R"))
att_from_rscripts(path = list.files(file.path(dummypackage, "R"), full.names = TRUE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_dependencies_file.R
\name{create_dependencies_file}
\alias{create_dependencies_file}
\title{Create the list of instructions to install dependencies from a DESCRIPTION file}
\usage{
create_dependencies_file(
  path = "DESCRIPTION",
  field = c("Depends", "Imports"),
  to = "inst/dependencies.R",
  open_file = TRUE,
  ignore_base = TRUE,
  install_only_if_missing = FALSE
)
}
\arguments{
\item{path}{path to the DESCRIPTION file}

\item{field}{DESCRIPTION field to parse, "Import" and "Depends" by default. Can add "Suggests"}

\item{to}{path where to save the dependencies file. Set to "inst/dependencies.R" by default. Set to \code{NULL} if you do not want the file, but only the instructions as a list of character.}

\item{open_file}{Logical. Open the file created in an editor}

\item{ignore_base}{Logical. Whether to ignore package coming with base, as they cannot be installed (default TRUE)}

\item{install_only_if_missing}{Logical Modify the installation instructions to check, beforehand, if the packages are missing . (default FALSE)}
}
\value{
List of R instructions to install all dependencies from a DESCRIPTION file. Side effect: creates a R file containing these instructions.
}
\description{
Outputs the list of instructions and a "dependencies.R" file with instructions in the "inst/" directory
}
\examples{
# Create a fake package
tmpdir <- tempfile(pattern = "depsfile")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
          recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

# Create the dependencies commands but no file
create_dependencies_file(
  path = file.path(dummypackage,"DESCRIPTION"),
  to = NULL,
  open_file = FALSE)

# Create the dependencies files in the package
create_dependencies_file(
  path = file.path(dummypackage,"DESCRIPTION"),
  to = file.path(dummypackage, "inst/dependencies.R"),
  open_file = FALSE)
list.files(file.path(dummypackage, "inst"))
# browseURL(dummypackage)

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/my_mean.R
\name{my_mean}
\alias{my_mean}
\title{my_mean}
\usage{
my_mean(x)
}
\arguments{
\item{x}{a vector}
}
\description{
my_mean
}
\examples{
# example code
library(utils)
data("fruit", package = "stringr")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/att_from_namespace.R
\name{att_from_namespace}
\alias{att_from_namespace}
\title{return package dependencies from NAMESPACE file}
\usage{
att_from_namespace(path = "NAMESPACE", document = TRUE, clean = TRUE)
}
\arguments{
\item{path}{path to NAMESPACE file}

\item{document}{Run function roxygenise of roxygen2 package}

\item{clean}{Logical. Whether to remove the original NAMESPACE before updating}
}
\value{
a vector
}
\description{
return package dependencies from NAMESPACE file
}
\examples{
tmpdir <- tempfile(pattern = "namespace")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
 recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)
att_from_namespace(path = file.path(dummypackage, "NAMESPACE"))

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_from_examples.R
\name{att_from_examples}
\alias{att_from_examples}
\title{Get all packages called in examples from R files}
\usage{
att_from_examples(dir.r = "R")
}
\arguments{
\item{dir.r}{path to directory with R scripts.}
}
\value{
Character vector of packages called with library or require.
}
\description{
Get all packages called in examples from R files
}
\examples{
dummypackage <- system.file("dummypackage",package = "attachment")

# browseURL(dummypackage)
att_from_examples(dir.r = file.path(dummypackage,"R"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_renv.R
\name{create_renv_for_dev}
\alias{create_renv_for_dev}
\alias{create_renv_for_prod}
\title{Create reproducible environments for your R projects with \emph{renv}}
\usage{
create_renv_for_dev(
  path = ".",
  dev_pkg = "_default",
  folder_to_include = c("dev", "data-raw"),
  folder_to_exclude = c("renv"),
  output = "renv.lock",
  install_if_missing = TRUE,
  document = TRUE,
  pkg_ignore = NULL,
  check_if_suggests_is_installed = TRUE,
  ...
)

create_renv_for_prod(
  path = ".",
  output = "renv.lock.prod",
  dev_pkg = "remotes",
  check_if_suggests_is_installed = FALSE,
  folder_to_include = NULL,
  ...
)
}
\arguments{
\item{path}{Path to your current package source folder}

\item{dev_pkg}{Vector of packages you need for development. Use \verb{_default}
(with underscore before to avoid confusing with a package name), to
use the default list. Use \code{NULL} for no extra package.
Use \code{attachment:::extra_dev_pkg} for the list.}

\item{folder_to_include}{Folder to scan to detect development packages}

\item{folder_to_exclude}{Folder to exclude during scan to detect packages.'renv' by default}

\item{output}{Path and name of the file created, default is \code{./renv.lock}}

\item{install_if_missing}{Logical. Install missing packages. \code{TRUE} by default}

\item{document}{Logical. Whether to run \code{\link[=att_amend_desc]{att_amend_desc()}} before
detecting packages in DESCRIPTION.}

\item{pkg_ignore}{Vector of packages to ignore from being discovered in your files.
This does not prevent them to be in "renv.lock" if they are recursive dependencies.}

\item{check_if_suggests_is_installed}{Logical. Whether to require that packages in the Suggests section are installed.}

\item{...}{Other arguments to pass to \code{\link[renv:snapshot]{renv::snapshot()}}}
}
\value{
a renv.lock file
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Tool to create and maintain renv.lock files.
The idea is to have 2 distinct files, one for development and the other for deployment.
Indeed, although packages like \emph{attachment} or \emph{pkgload} must be installed to develop,
they are not necessary in your project, package or Shiny application.
}
\examples{
\dontrun{
# Writes a renv.lock a file in the user directory
create_renv_for_dev()
create_renv_for_dev(dev_pkg = "attachment")
create_renv_for_prod()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/att_from_rscripts.R
\name{att_from_rscript}
\alias{att_from_rscript}
\title{Look for functions called with \code{::} and library/requires in one script}
\usage{
att_from_rscript(path, encoding = getOption("encoding"))
}
\arguments{
\item{path}{path to R script file}

\item{encoding}{Encoding passed to \code{\link[=readLines]{readLines()}} when reading \code{path}. Defaults
to \code{getOption("encoding")} so the system locale is respected (important on
Windows where scripts are often Latin-1 / Windows-1252).}
}
\value{
a vector
}
\description{
Look for functions called with \code{::} and library/requires in one script
}
\details{
Uses the R parser to walk the syntax tree so that occurrences of \code{pkg::fun}
or \code{library()/require()/requireNamespace()/loadNamespace()/use()/getFromNamespace()}
inside string literals or comments are ignored.
Named arguments such as \code{library(package = "pkg")} are supported, as are
fully-qualified forms like \code{base::library(pkg)} or
\code{methods::getFromNamespace(fn, "pkg")}.
Introspection helpers such as \code{packageVersion()}, \code{getNamespace()},
\code{asNamespace()}, and \code{attachNamespace()} are \strong{not} treated as dependency
introducers, because they are commonly used for version or feature checks
on packages that may or may not be required at runtime.

If the file cannot be parsed as valid R (syntax error, corrupt encoding,
etc.), the function falls back to a regex-based detector and emits a
\code{warning()} naming the file so users can investigate.
}
\examples{
dummypackage <- system.file("dummypackage",package = "attachment")
# browseURL(dummypackage)

att_from_rscript(path = file.path(dummypackage,"R","my_mean.R"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install_from_description.R
\name{install_if_missing}
\alias{install_if_missing}
\title{install  packages if missing}
\usage{
install_if_missing(to_be_installed, ...)
}
\arguments{
\item{to_be_installed}{a character vector containing required packages names}

\item{...}{Arguments to be passed to \code{\link[utils:install.packages]{utils::install.packages()}}}
}
\value{
Used for side effect. Install missing packages from the character vector input.
}
\description{
install  packages if missing
}
\examples{
\dontrun{
# This will install packages on your system
install_if_missing(c("dplyr", "fcuk", "rusk"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/att_from_rmds.R
\name{att_from_rmd}
\alias{att_from_rmd}
\alias{att_from_qmd}
\title{Get all dependencies from a Rmd file}
\usage{
att_from_rmd(
  path,
  temp_dir = tempdir(),
  warn = -1,
  encoding = getOption("encoding"),
  inside_rmd = NULL,
  inline = TRUE
)

att_from_qmd(
  path,
  temp_dir = tempdir(),
  warn = -1,
  encoding = getOption("encoding"),
  inside_rmd = NULL,
  inline = TRUE
)
}
\arguments{
\item{path}{Path to a Rmd file}

\item{temp_dir}{Path to temporary script from purl vignette}

\item{warn}{-1 for quiet warnings with purl, 0 to see warnings}

\item{encoding}{Encoding of the input file; always assumed to be UTF-8 (i.e.,
this argument is effectively ignored).}

\item{inside_rmd}{Logical or \code{NULL}. Whether the function is being called
from inside a knit session, in which case the actual purl step must be
delegated to an external R process. When \code{NULL} (the default), this is
auto-detected via \code{knitr::opts_knit$get("out.format")}.}

\item{inline}{Logical. Default TRUE. Whether to explore inline code for dependencies.}
}
\value{
vector of character of packages names found in the Rmd
}
\description{
Get all dependencies from a Rmd file
}
\examples{

dummypackage <- system.file("dummypackage",package = "attachment")
# browseURL(dummypackage)
att_from_rmd(path = file.path(dummypackage,"vignettes/demo.Rmd"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_remotes.R
\name{set_remotes_to_desc}
\alias{set_remotes_to_desc}
\title{Add Remotes field to DESCRIPTION based on your local installation}
\usage{
set_remotes_to_desc(path.d = "DESCRIPTION", stop.local = FALSE, clean = TRUE)
}
\arguments{
\item{path.d}{path to description file.}

\item{stop.local}{Logical. Whether to stop if package was installed from local source.
Message otherwise.}

\item{clean}{Logical. Whether to clean all existing remotes before run.}
}
\value{
Used for side effect. Adds Remotes field in DESCRIPTION file.
}
\description{
Add Remotes field to DESCRIPTION based on your local installation
}
\examples{
tmpdir <- tempfile(pattern = "setremotes")
dir.create(tmpdir)
file.copy(system.file("dummypackage", package = "attachment"), tmpdir,
 recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# Add remotes field if there are Remotes locally
att_amend_desc(dummypackage) \%>\%
  set_remotes_to_desc()

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)

\dontrun{
# For your current package
att_amend_desc() \%>\%
  set_remotes_to_desc()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/attachment-package.R
\docType{package}
\name{attachment-package}
\alias{attachment}
\alias{attachment-package}
\title{attachment: Deal with Dependencies}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Manage dependencies during package development. This can retrieve all dependencies that are used in ".R" files in the "R/" directory, in ".Rmd" files in "vignettes/" directory and in 'roxygen2' documentation of functions. There is a function to update the "DESCRIPTION" file of your package with 'CRAN' packages or any other remote package. All functions to retrieve dependencies of ".R" scripts and ".Rmd" or ".qmd" files can be used independently of a package development.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://thinkr-open.github.io/attachment/}
  \item \url{https://github.com/ThinkR-open/attachment}
  \item Report bugs at \url{https://github.com/ThinkR-open/attachment/issues}
}

}
\author{
\strong{Maintainer}: Vincent Guyader \email{vincent@thinkr.fr} (\href{https://orcid.org/0000-0003-0671-9270}{ORCID})

Authors:
\itemize{
  \item Sébastien Rochette \email{sebastienrochettefr@gmail.com} (\href{https://orcid.org/0000-0002-1565-9313}{ORCID}) (previous maintainer)
  \item Murielle Delmotte \email{murielle@thinkr.fr} (\href{https://orcid.org/0000-0002-1339-2424}{ORCID})
  \item Swann Floc'hlay \email{swann@thinkr.fr} (\href{https://orcid.org/0000-0003-1477-830X}{ORCID})
}

Other contributors:
\itemize{
  \item ThinkR [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/attachment-deprecated.R
\name{attachment-deprecated}
\alias{attachment-deprecated}
\alias{att_to_description}
\title{Deprecated functions}
\usage{
att_to_description(
  path = ".",
  path.n = "NAMESPACE",
  path.d = "DESCRIPTION",
  dir.r = "R",
  dir.v = "vignettes",
  dir.t = "tests",
  extra.suggests = NULL,
  pkg_ignore = NULL,
  document = TRUE,
  normalize = TRUE,
  inside_rmd = NULL
)
}
\arguments{
\item{path}{path to the root of the package directory. Default to current directory.}

\item{path.n}{path to namespace file.}

\item{path.d}{path to description file.}

\item{dir.r}{path to directory with R scripts.}

\item{dir.v}{path to vignettes directory. Set to empty (dir.v = "") to ignore.}

\item{dir.t}{path to tests directory. Set to empty (dir.t = "") to ignore.}

\item{extra.suggests}{vector of other packages that should be added in Suggests (pkgdown, covr for instance)}

\item{pkg_ignore}{vector of packages names to ignore.}

\item{document}{Run function roxygenise of roxygen2 package}

\item{normalize}{Logical. Whether to normalize the DESCRIPTION file. See \code{\link[desc:desc_normalize]{desc::desc_normalize()}}}

\item{inside_rmd}{Logical or \code{NULL}. Whether the function is being called
from inside a knit session, in which case the actual purl step must be
delegated to an external R process. When \code{NULL} (the default), this is
auto-detected via \code{knitr::opts_knit$get("out.format")}.}
}
\value{
List of functions used for deprecation side effects.
Output depends on the deprecated function.
}
\description{
List of functions deprecated. They will be removed in a future release.
}
\section{Details}{

\tabular{rl}{
\code{att_to_description} \tab is now called \code{att_amend_desc}
so that it is the first function proposed when using auto-completion\cr
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/att_from_rmds.R
\name{att_from_rmds}
\alias{att_from_rmds}
\alias{att_from_qmds}
\title{Get all packages called in vignettes folder}
\usage{
att_from_rmds(
  path = "vignettes",
  pattern = "*.[.](Rmd|rmd|qmd)$",
  recursive = TRUE,
  warn = -1,
  inside_rmd = NULL,
  inline = TRUE,
  folder_to_exclude = "renv"
)

att_from_qmds(
  path = "vignettes",
  pattern = "*.[.](Rmd|rmd|qmd)$",
  recursive = TRUE,
  warn = -1,
  inside_rmd = NULL,
  inline = TRUE,
  folder_to_exclude = "renv"
)
}
\arguments{
\item{path}{path to directory with Rmds or vector of Rmd files}

\item{pattern}{pattern to detect Rmd files}

\item{recursive}{logical. Should the listing recurse into directories?}

\item{warn}{-1 for quiet warnings with purl, 0 to see warnings}

\item{inside_rmd}{Logical or \code{NULL}. Whether the function is being called
from inside a knit session, in which case the actual purl step must be
delegated to an external R process. When \code{NULL} (the default), this is
auto-detected via \code{knitr::opts_knit$get("out.format")}.}

\item{inline}{Logical. Default TRUE. Whether to explore inline code for dependencies.}

\item{folder_to_exclude}{Folder to exclude during scan to detect packages 'renv' by default}
}
\value{
Character vector of packages called with library or require.
When the directory contains "vignettes" in its path, \code{knitr} is always added
and the vignette engine is inferred from the files actually present:
\code{rmarkdown} is added when \code{.Rmd} files are found, \code{quarto} when \code{.qmd}
files are found (both when the directory mixes the two). If the directory
is empty, \code{rmarkdown} is added as a safe default.
}
\description{
Get all packages called in vignettes folder
}
\examples{
dummypackage <- system.file("dummypackage",package = "attachment")
# browseURL(dummypackage)
att_from_rmds(path = file.path(dummypackage,"vignettes"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/att_from_data.R
\name{att_from_data}
\alias{att_from_data}
\title{Look for functions called in data loading code}
\usage{
att_from_data(chr)
}
\arguments{
\item{chr}{A character vector containing the code as a string. The code should follow the pattern used for loading data with \code{data()}, specifying the dataset and package.}
}
\value{
A character vector containing the names of the packages from which datasets are being loaded.
}
\description{
Look for functions called in data loading code
}
\examples{

vec_char <- 'data("starwars", package = "dplyr")'
att_from_data(vec_char)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_remotes.R
\name{find_remotes}
\alias{find_remotes}
\title{Proposes values for Remotes field for DESCRIPTION file based on your installation}
\usage{
find_remotes(pkg)
}
\arguments{
\item{pkg}{Character. Packages to test for potential non-CRAN installation}
}
\value{
List of all non-CRAN packages and code to add in Remotes field in DESCRIPTION.
NULL otherwise.
}
\description{
Proposes values for Remotes field for DESCRIPTION file based on your installation
}
\examples{
# Find from vector of packages
find_remotes(pkg = c("attachment", "desc", "glue"))

# Find from Description file
dummypackage <- system.file("dummypackage", package = "attachment")
att_from_description(
path = file.path(dummypackage, "DESCRIPTION")) \%>\%
find_remotes()

\dontrun{
# For the current package directory
att_from_description() \%>\% find_remotes()
}

\donttest{
# For a specific package name
find_remotes("attachment")

# Find remotes from all installed packages
find_remotes(list.dirs(.libPaths(), full.names = FALSE, recursive = FALSE))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/att_from_description.R
\name{att_from_description}
\alias{att_from_description}
\title{Return all package dependencies from current package}
\usage{
att_from_description(
  path = "DESCRIPTION",
  dput = FALSE,
  field = c("Depends", "Imports", "Suggests")
)
}
\arguments{
\item{path}{path to the DESCRIPTION file}

\item{dput}{if FALSE return a vector instead of dput output}

\item{field}{DESCRIPTION field to parse, Import, Suggests and Depends by default}
}
\value{
A character vector with packages names
}
\description{
Return all package dependencies from current package
}
\examples{

dummypackage <- system.file("dummypackage", package = "attachment")
# browseURL(dummypackage)
att_from_description(path = file.path(dummypackage, "DESCRIPTION"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/att_to_description.R
\name{att_amend_desc}
\alias{att_amend_desc}
\alias{att_to_desc_from_pkg}
\title{Amend DESCRIPTION with dependencies read from package code parsing}
\usage{
att_amend_desc(
  path = ".",
  path.n = "NAMESPACE",
  path.d = "DESCRIPTION",
  dir.r = "R",
  dir.v = "vignettes",
  dir.t = "tests",
  extra.suggests = NULL,
  pkg_ignore = NULL,
  document = TRUE,
  normalize = TRUE,
  inside_rmd = NULL,
  must.exist = TRUE,
  check_if_suggests_is_installed = TRUE,
  update.config = FALSE,
  use.config = TRUE,
  path.c = "dev/config_attachment.yaml"
)

att_to_desc_from_pkg(
  path = ".",
  path.n = "NAMESPACE",
  path.d = "DESCRIPTION",
  dir.r = "R",
  dir.v = "vignettes",
  dir.t = "tests",
  extra.suggests = NULL,
  pkg_ignore = NULL,
  document = TRUE,
  normalize = TRUE,
  inside_rmd = NULL,
  must.exist = TRUE,
  check_if_suggests_is_installed = TRUE,
  update.config = FALSE,
  use.config = TRUE,
  path.c = "dev/config_attachment.yaml"
)
}
\arguments{
\item{path}{path to the root of the package directory. Default to current directory.}

\item{path.n}{path to namespace file.}

\item{path.d}{path to description file.}

\item{dir.r}{path to directory with R scripts.}

\item{dir.v}{path to vignettes directory. Set to empty (dir.v = "") to ignore.}

\item{dir.t}{path to tests directory. Set to empty (dir.t = "") to ignore.}

\item{extra.suggests}{vector of other packages that should be added in Suggests (pkgdown, covr for instance)}

\item{pkg_ignore}{vector of packages names to ignore.}

\item{document}{Run function roxygenise of roxygen2 package}

\item{normalize}{Logical. Whether to normalize the DESCRIPTION file. See \code{\link[desc:desc_normalize]{desc::desc_normalize()}}}

\item{inside_rmd}{Logical or \code{NULL}. Whether the function is being called
from inside a knit session, in which case the actual purl step must be
delegated to an external R process. When \code{NULL} (the default), this is
auto-detected via \code{knitr::opts_knit$get("out.format")}.}

\item{must.exist}{Logical. If TRUE then an error is given if packages do not exist
within installed packages. If NA, a warning.}

\item{check_if_suggests_is_installed}{Logical. Whether to require that packages in the Suggests section are installed.}

\item{update.config}{logical. Should the parameters used in this call be saved in the config file of the package}

\item{use.config}{logical. Should the command use the parameters from the config file to run}

\item{path.c}{character Path to the yaml config file where parameters are saved}
}
\value{
Update DESCRIPTION file.
}
\description{
Amend package DESCRIPTION file with the list of dependencies extracted from
R, examples, tests, vignettes files.
att_to_desc_from_pkg() is an alias of att_amend_desc(),
for the correspondence with \code{\link[=att_to_desc_from_is]{att_to_desc_from_is()}}.
}
\details{
Your daily use is to run \code{att_amend_desc()}, as is.
You will want to run this function sometimes with some extra information like
\code{att_amend_desc(pkg_ignore = "x", update.config = TRUE)} if you have to update
the configuration file.
Next time \code{att_amend_desc()} will use these parameters from the configuration
file directly.
}
\examples{

# Run on an external "dummypackage" as an example
# For your local use, you do not have to specify the `path` as below
# By default, `att_amend_desc()` will run on the current working directory

# Create a fake package for the example
tmpdir <- tempfile(pattern = "description")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
 recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")

# Update documentation and dependencies
att_amend_desc(path = dummypackage)

# You can look at the content of this external package
#' # browseURL(dummypackage)

# Update the config file with extra parameters
# We recommend that you store this code in a file in your "dev/" directory
# to run it when needed
att_amend_desc(path = dummypackage, extra.suggests = "testthat", update.config = TRUE)

# Next time, in your daily development
att_amend_desc(path = dummypackage)

# Clean after examples
unlink(tmpdir, recursive = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/att_to_description.R
\name{att_to_desc_from_is}
\alias{att_to_desc_from_is}
\title{Amend DESCRIPTION with dependencies from imports and suggests package list}
\usage{
att_to_desc_from_is(
  path.d = "DESCRIPTION",
  imports = NULL,
  suggests = NULL,
  check_if_suggests_is_installed = TRUE,
  normalize = TRUE,
  must.exist = TRUE
)
}
\arguments{
\item{path.d}{path to description file.}

\item{imports}{character vector of package names to add in Imports section}

\item{suggests}{character vector of package names to add in Suggests section}

\item{check_if_suggests_is_installed}{Logical. Whether to require that packages in the Suggests section are installed.}

\item{normalize}{Logical. Whether to normalize the DESCRIPTION file. See \code{\link[desc:desc_normalize]{desc::desc_normalize()}}}

\item{must.exist}{Logical. If TRUE then an error is given if packages do not exist
within installed packages. If NA, a warning.}
}
\value{
Fill in Description file
}
\description{
Amend DESCRIPTION with dependencies from imports and suggests package list
}
\details{
\code{must.exist} is better set to \code{TRUE} during package development.
This stops the process when a package does not exists on your system.
This avoids check errors with typos in package names in DESCRIPTION.
When used in CI to discover dependencies, for a bookdown for instance,
you may want to set to \code{FALSE} (no message at all) or \code{NA} (warning for not installed).
}
\examples{
tmpdir <- tempfile(pattern = "descfromis")
dir.create(tmpdir)
file.copy(system.file("dummypackage",package = "attachment"), tmpdir,
 recursive = TRUE)
dummypackage <- file.path(tmpdir, "dummypackage")
# browseURL(dummypackage)
att_to_desc_from_is(path.d = file.path(dummypackage, "DESCRIPTION"),
imports = c("magrittr", "attachment"), suggests = c("knitr"))

# In combination with other functions
att_to_desc_from_is(path.d = file.path(dummypackage, "DESCRIPTION"),
imports = att_from_rscripts(file.path(dummypackage, "R")),
suggests = att_from_rmds(file.path(dummypackage, "vignettes")))

# Clean temp files after this example
unlink(tmpdir, recursive = TRUE)
}
